@@ -1,7 +1,7 @@
 #
 #	Kest.R		Estimation of K function
 #
-#	$Revision: 5.103 $	$Date: 2014/02/15 03:39:44 $
+#	$Revision: 5.120 $	$Date: 2017/06/05 10:31:58 $
 #
 #
 # -------- functions ----------------------------------------
@@ -36,14 +36,14 @@
 
 "Lest" <- function(X, ...) {
   K <- Kest(X, ...)
-  L <- eval.fv(sqrt(K/pi))
+  L <- eval.fv(sqrt(K/pi), dotonly=FALSE)
   # handle variance estimates
   if(any(varcols <- colnames(K) %in% c("rip", "ls"))) {
     r <- with(L, .x)
     L[,varcols] <- as.data.frame(K)[,varcols]/(2 * pi * r)^2
     # fix 0/0
     n <- npoints(X)
-    A <- area.owin(as.owin(X))
+    A <- area(Window(X))
     if(any(colnames(K) == "rip"))
       L[r == 0, "rip"] <- (2 * A/(n-1)^2)/(4 * pi)
     if(any(colnames(K) == "ls"))
@@ -56,7 +56,7 @@
 }
 
 "Kest"<-
-function(X, ..., r=NULL, breaks=NULL, 
+function(X, ..., r=NULL, rmax=NULL, breaks=NULL, 
          correction=c("border", "isotropic", "Ripley", "translate"),
          nlarge=3000, domain=NULL, var.approx=FALSE,
          ratio=FALSE)
@@ -66,9 +66,9 @@ function(X, ..., r=NULL, breaks=NULL,
   rfixed <- !is.null(r) || !is.null(breaks)
   npts <- npoints(X)
   W <- X$window
-  area <- area.owin(W)
-  lambda <- npts/area
-  lambda2 <- (npts * (npts - 1))/(area^2)
+  areaW <- area(W)
+  lambda <- npts/areaW
+  lambda2 <- (npts * (npts - 1))/(areaW^2)
 
   if(!is.null(domain)) {
     # estimate based on contributions from a subdomain
@@ -86,7 +86,8 @@ function(X, ..., r=NULL, breaks=NULL,
     return(Kd)
   }
 
-  rmaxdefault <- rmax.rule("K", W, lambda)        
+  rmaxdefault <- rmax %orifnull% rmax.rule("K", W, lambda)
+  if(is.infinite(rmaxdefault)) rmaxdefault <- diameter(W)
   breaks <- handle.r.b.args(r, breaks, W, rmaxdefault=rmaxdefault)
   r <- breaks$r
   rmax <- breaks$max
@@ -104,10 +105,11 @@ function(X, ..., r=NULL, breaks=NULL,
                              trans="translate",
                              translate="translate",
                              translation="translate",
+                             rigid="rigid",
                              good="good",
                              best="best"),
                            multi=TRUE)
-  best.wanted <- ("best" %in% correction)
+#  best.wanted <- ("best" %in% correction)
   # replace 'good' by the optimal choice for this size of dataset
   if("good" %in% correction)
     correction[correction == "good"] <- good.correction.K(X)
@@ -124,7 +126,7 @@ function(X, ..., r=NULL, breaks=NULL,
 
   can.do.fast <- breaks$even
   large.n    <- (npts >= nlarge)
-  demand.best <- correction.given && best.wanted
+#  demand.best <- correction.given && best.wanted
   large.n.trigger <- large.n && !correction.given
   fastcorrections <- c("border", "bord.modif", "none")
   fastdefault     <- "border"
@@ -154,129 +156,164 @@ function(X, ..., r=NULL, breaks=NULL,
       Kb <- Kborder.engine(X, max(r), length(r), correction, ratio=ratio)
     if(none)
       Kn <- Knone.engine(X, max(r), length(r), ratio=ratio)
-    if(bord && none) 
-      return(cbind.fv(Kb, Kn[, names(Kn) != "theo"]))
+    if(bord && none) {
+      Kn <- Kn[ , names(Kn) != "theo"]
+      yn <- fvnames(Kb, ".y")
+      Kbn <- if(!ratio) bind.fv(Kb, Kn, preferred=yn) else
+             bind.ratfv(Kb, Kn, preferred=yn)
+      return(Kbn)
+    }
     if(bord) return(Kb)
     if(none) return(Kn) 
   }
 
-  ###########################################
-  # Fast code for rectangular window
-  ###########################################
+  do.fast.rectangle <-
+    can.do.fast && is.rectangle(W) &&
+      spatstat.options("use.Krect") && !any(correction == "rigid")
+  
+  if(do.fast.rectangle) {
+    ###########################################
+    ## Fast code for rectangular window
+    ###########################################
+    K <-  Krect.engine(X, rmax, length(r), correction, ratio=ratio)
+    attr(K, "alim") <- alim
+  } else {
+    ###########################################
+    ## Slower code
+    ###########################################
 
-  if(can.do.fast && is.rectangle(W) && spatstat.options("use.Krect")) {
-    Kr <-  Krect.engine(X, rmax, length(r), correction, ratio=ratio)
-    attr(Kr, "alim") <- alim
-    return(Kr)
-  }
+    ## this will be the output data frame
+    Kdf <- data.frame(r=r, theo = pi * r^2)
+    desc <- c("distance argument r", "theoretical Poisson %s")
+    denom <- lambda2 * areaW
+    K <- ratfv(Kdf, NULL, denom,
+               "r", quote(K(r)),
+               "theo", NULL, alim, c("r","%s[pois](r)"), desc, fname="K",
+               ratio=ratio)
   
-  ###########################################
-  # Slower code
-  ###########################################
-        
-        
-  # this will be the output data frame
-  Kdf <- data.frame(r=r, theo = pi * r^2)
-  desc <- c("distance argument r", "theoretical Poisson %s")
-  denom <- lambda2 * area
-  K <- ratfv(Kdf, NULL, denom,
-             "r", quote(K(r)),
-             "theo", NULL, alim, c("r","%s[pois](r)"), desc, fname="K",
-             ratio=ratio)
-  
-  # identify all close pairs
-  rmax <- max(r)
-  close <- closepairs(X, rmax)
-  DIJ <- close$d
+    ## identify all close pairs
+    rmax <- max(r)
+    what <- 
+	if(any(correction %in% c("translate", "isotropic"))) "all" else "ijd"
+    close <- closepairs(X, rmax, what=what)
+    DIJ <- close$d
 
-  if(any(correction == "none")) {
-    # uncorrected! For demonstration purposes only!
-    wh <- whist(DIJ, breaks$val)  # no weights
-    numKun <- cumsum(wh)
-    denKun <- lambda2 * area
-    # uncorrected estimate of K
-    K <- bind.ratfv(K,
-                    data.frame(un=numKun), denKun,
-                    "hat(%s)[un](r)",
-                    "uncorrected estimate of %s",
-                    "un",
-                    ratio=ratio)
-  }
-  
-  if(any(correction == "border" | correction == "bord.modif")) {
-  # border method
-  # Compute distances to boundary
-    b <- bdist.points(X)
-    I <- close$i
-    bI <- b[I]
-  # apply reduced sample algorithm
-    RS <- Kount(DIJ, bI, b, breaks)
-    if(any(correction == "bord.modif")) {
-      # modified border correction
-      denom.area <- eroded.areas(W, r)
-      numKbm <- RS$numerator
-      denKbm <- lambda2 * denom.area
+    ## precompute set covariance of window
+    gW <- NULL
+    if(any(correction %in% c("translate", "rigid", "isotropic")))
+      gW <- setcov(W)
+    
+    if(any(correction == "none")) {
+      ## uncorrected! For demonstration purposes only!
+      wh <- whist(DIJ, breaks$val)  # no weights
+      numKun <- cumsum(wh)
+      denKun <- lambda2 * areaW
+      ## uncorrected estimate of K
       K <- bind.ratfv(K,
-                      data.frame(bord.modif=numKbm),
-                      data.frame(bord.modif=denKbm),
-                      "hat(%s)[bordm](r)",
-                      "modified border-corrected estimate of %s",
-                      "bord.modif",
+                      data.frame(un=numKun), denKun,
+                      "hat(%s)[un](r)",
+                      "uncorrected estimate of %s",
+                      "un",
                       ratio=ratio)
     }
-    if(any(correction == "border")) {
-      numKb <- RS$numerator
-      denKb <- lambda * RS$denom.count
+  
+    if(any(correction == "border" | correction == "bord.modif")) {
+      ## border method
+      ## Compute distances to boundary
+      b <- bdist.points(X)
+      I <- close$i
+      bI <- b[I]
+      ## apply reduced sample algorithm
+      RS <- Kount(DIJ, bI, b, breaks)
+      if(any(correction == "bord.modif")) {
+        ## modified border correction
+        denom.area <- eroded.areas(W, r)
+        numKbm <- RS$numerator
+        denKbm <- lambda2 * denom.area
+        K <- bind.ratfv(K,
+                        data.frame(bord.modif=numKbm),
+                        data.frame(bord.modif=denKbm),
+                        "hat(%s)[bordm](r)",
+                        "modified border-corrected estimate of %s",
+                        "bord.modif",
+                        ratio=ratio)
+      }
+      if(any(correction == "border")) {
+        numKb <- RS$numerator
+        denKb <- lambda * RS$denom.count
+        K <- bind.ratfv(K,
+                        data.frame(border=numKb), 
+                        data.frame(border=denKb), 
+                        "hat(%s)[bord](r)",
+                        "border-corrected estimate of %s",
+                        "border",
+                        ratio=ratio)
+      }
+    }
+
+    if(any(correction == "translate")) {
+      ## Ohser-Stoyan translation correction
+      edgewt <- edge.Trans(dx=close$dx, dy=close$dy, W=W, paired=TRUE,
+                           gW = gW, give.rmax=TRUE)
+      wh <- whist(DIJ, breaks$val, edgewt)
+      numKtrans <- cumsum(wh)
+      denKtrans <- lambda2 * areaW
+      h <- attr(edgewt, "rmax")
+      numKtrans[r >= h] <- NA
       K <- bind.ratfv(K,
-                      data.frame(border=numKb), 
-                      data.frame(border=denKb), 
-                      "hat(%s)[bord](r)",
-                      "border-corrected estimate of %s",
-                      "border",
+                      data.frame(trans=numKtrans),
+                      denKtrans,
+                      "hat(%s)[trans](r)",
+                      "translation-corrected estimate of %s",
+                      "trans",
+                      ratio=ratio)
+    }
+    if(any(correction == "rigid")) {
+      ## Ohser-Stoyan rigid motion correction
+      CW <- rotmean(gW)
+      edgewt <- areaW/as.function(CW)(DIJ)
+      wh <- whist(DIJ, breaks$val, edgewt)
+      numKrigid <- cumsum(wh)
+      denKrigid <- lambda2 * areaW
+      h <- rmax.Rigid(X, gW) #sic: X not W
+      numKrigid[r >= h] <- NA
+      K <- bind.ratfv(K,
+                      data.frame(rigid=numKrigid),
+                      denKrigid,
+                      "hat(%s)[rigid](r)",
+                      "rigid motion-corrected estimate of %s",
+                      "rigid",
+                      ratio=ratio)
+    }
+    if(any(correction == "isotropic")) {
+      ## Ripley isotropic correction
+      XI <- ppp(close$xi, close$yi, window=W, check=FALSE)
+      edgewt <- edge.Ripley(XI, matrix(DIJ, ncol=1))
+      wh <- whist(DIJ, breaks$val, edgewt)
+      numKiso <- cumsum(wh)
+      denKiso <- lambda2 * areaW
+      h <- boundingradius(W)
+      numKiso[r >= h] <- NA
+      K <- bind.ratfv(K,
+                      data.frame(iso=numKiso),
+                      denKiso,
+                      "hat(%s)[iso](r)",
+                      "Ripley isotropic correction estimate of %s",
+                      "iso",
                       ratio=ratio)
     }
   }
 
-  if(any(correction == "translate")) {
-    ## Ohser-Stoyan translation correction
-    edgewt <- edge.Trans(dx=close$dx, dy=close$dy, W=W, paired=TRUE)
-    wh <- whist(DIJ, breaks$val, edgewt)
-    numKtrans <- cumsum(wh)
-    denKtrans <- lambda2 * area
-    h <- diameter(as.rectangle(W))/2
-    numKtrans[r >= h] <- NA
-    K <- bind.ratfv(K,
-                    data.frame(trans=numKtrans),
-                    denKtrans,
-                    "hat(%s)[trans](r)",
-                    "translation-corrected estimate of %s",
-                    "trans",
-                    ratio=ratio)
-  }
-  if(any(correction == "isotropic")) {
-    ## Ripley isotropic correction
-    XI <- ppp(close$xi, close$yi, window=W, check=FALSE)
-    edgewt <- edge.Ripley(XI, matrix(DIJ, ncol=1))
-    wh <- whist(DIJ, breaks$val, edgewt)
-    numKiso <- cumsum(wh)
-    denKiso <- lambda2 * area
-    h <- diameter(W)/2
-    numKiso[r >= h] <- NA
-    K <- bind.ratfv(K,
-                 data.frame(iso=numKiso),
-                 denKiso,
-                 "hat(%s)[iso](r)",
-                 "Ripley isotropic correction estimate of %s",
-                 "iso",
-                 ratio=ratio)
-  }
-  #
+  #############################
+  ##  VARIANCE APPROXIMATION
+  #############################
+
   if(var.approx) {
-    # Compute variance approximations
-    A <- area
+    ## Compute variance approximations
+    A <- areaW
     P <- perimeter(W)
     n <- npts
-    # Ripley asymptotic approximation
+    ## Ripley asymptotic approximation
     rip <- 2 * ((A/(n-1))^2) * (pi * r^2/A + 0.96 * P * r^3/A^2
                                 + 0.13 * (n/A) * P * r^5/A^2)
     if(!ratio) {
@@ -322,11 +359,13 @@ function(X, ..., r=NULL, breaks=NULL,
       }
     }
   }
-  # default plot will display all edge corrections
+
+  ### FINISH OFF #####
+  ## default plot will display all edge corrections
   formula(K) <- . ~ r
   nama <- rev(colnames(K))
-  nama <- nama[!(nama %in% c("r", "rip", "ls"))]
-  fvnames(K, ".") <- nama
+  fvnames(K, ".") <- setdiff(nama, c("r", "rip", "ls"))
+  ##
   unitname(K) <- unitname(X)
   # copy to other components
   if(ratio)
@@ -386,9 +425,9 @@ Kborder.engine <- function(X, rmax, nr=100,
   npts <- npoints(X)
   W <- as.owin(X)
 
-  area <- area.owin(W)
-  lambda <- npts/area
-  lambda2 <- (npts * (npts - 1))/(area^2)
+  areaW <- area(W)
+  lambda <- npts/areaW
+  lambda2 <- (npts * (npts - 1))/(areaW^2)
 
   if(missing(rmax))
     rmax <- diameter(W)/4
@@ -402,7 +441,7 @@ Kborder.engine <- function(X, rmax, nr=100,
 
   if(ratio) {
     # save numerator and denominator
-    denom <- lambda2 * area
+    denom <- lambda2 * areaW
     numK <- eval.fv(denom * Kfv)
     denK <- eval.fv(denom + Kfv * 0)
     attributes(numK) <- attributes(denK) <- attributes(Kfv)
@@ -423,8 +462,6 @@ Kborder.engine <- function(X, rmax, nr=100,
   b <- bdist.points(Xsort)
 
   # call the C code
-  DUP <- spatstat.options("dupC")
-  
   if(is.null(weights)) {
     # determine whether the numerator can be stored as an integer
     bigint <- .Machine$integer.max
@@ -439,8 +476,7 @@ Kborder.engine <- function(X, rmax, nr=100,
                 rmax=as.double(rmax),
                 numer=as.integer(integer(nr)),
                 denom=as.integer(integer(nr)),
-                DUP=DUP)
-#                PACKAGE="spatstat")
+                PACKAGE = "spatstat")
     } else {
       # no - need double precision storage
       res <- .C("KborderD",
@@ -452,8 +488,7 @@ Kborder.engine <- function(X, rmax, nr=100,
                 rmax=as.double(rmax),
                 numer=as.double(numeric(nr)),
                 denom=as.double(numeric(nr)),
-                DUP=DUP)
-#                PACKAGE="spatstat")
+                PACKAGE = "spatstat")
     }
     if("bord.modif" %in% correction) {
       denom.area <- eroded.areas(W, r)
@@ -501,7 +536,7 @@ Kborder.engine <- function(X, rmax, nr=100,
     } else {
       wim <- as.im(weights, W)
       weights <- wim[X, drop=FALSE]
-      if(any(is.na(weights)))
+      if(anyNA(weights))
         stop("domain of weights image does not contain all points of X")
     }
     weights.Xsort <- weights[orderX]
@@ -515,8 +550,7 @@ Kborder.engine <- function(X, rmax, nr=100,
               rmax=as.double(rmax),
               numer=as.double(numeric(nr)),
               denom=as.double(numeric(nr)),
-              DUP=DUP)
-#              PACKAGE="spatstat")
+              PACKAGE = "spatstat")
     if("border" %in% correction) {
       bord <- res$numer/res$denom
       Kfv <- bind.fv(Kfv, data.frame(border=bord), "hat(%s)[bord](r)",
@@ -574,10 +608,10 @@ Knone.engine <- function(X, rmax, nr=100,
   npts <- npoints(X)
   W <- as.owin(X)
 
-  area <- area.owin(W)
-  lambda <- npts/area
-  lambda2 <- (npts * (npts - 1))/(area^2)
-  denom <- lambda2 * area
+  areaW <- area(W)
+#  lambda <- npts/areaW
+  lambda2 <- (npts * (npts - 1))/(areaW^2)
+  denom <- lambda2 * areaW
 
   if(missing(rmax))
     rmax <- diameter(W)/4
@@ -608,8 +642,6 @@ Knone.engine <- function(X, rmax, nr=100,
   y <- Xsort$y
   
   # call the C code
-  DUP <- spatstat.options("dupC")
-  
   if(is.null(weights)) {
     # determine whether the numerator can be stored as an integer
     bigint <- .Machine$integer.max
@@ -622,8 +654,7 @@ Knone.engine <- function(X, rmax, nr=100,
                 nr=as.integer(nr),
                 rmax=as.double(rmax),
                 numer=as.integer(integer(nr)),
-                DUP=DUP)
-#                PACKAGE="spatstat")
+                PACKAGE = "spatstat")
     } else {
       # no - need double precision storage
       res <- .C("KnoneD",
@@ -633,12 +664,11 @@ Knone.engine <- function(X, rmax, nr=100,
                 nr=as.integer(nr),
                 rmax=as.double(rmax),
                 numer=as.double(numeric(nr)),
-                DUP=DUP)
-#                PACKAGE="spatstat")
+                PACKAGE = "spatstat")
     }
 
     numKun <- res$numer
-    denKun <- denom # = lambda2 * area
+    denKun <- denom # = lambda2 * areaW
     Kun <- numKun/denKun
   } else {
     # weighted version
@@ -648,7 +678,7 @@ Knone.engine <- function(X, rmax, nr=100,
     } else {
       wim <- as.im(weights, W)
       weights <- wim[X, drop=FALSE]
-      if(any(is.na(weights)))
+      if(anyNA(weights))
         stop("domain of weights image does not contain all points of X")
     }
     weights.Xsort <- weights[orderX]
@@ -660,8 +690,7 @@ Knone.engine <- function(X, rmax, nr=100,
               nr=as.integer(nr),
               rmax=as.double(rmax),
               numer=as.double(numeric(nr)),
-              DUP=DUP)
-#              PACKAGE="spatstat")
+              PACKAGE = "spatstat")
     numKun <- res$numer
     denKun <- sum(weights)
     Kun <- numKun/denKun
@@ -707,6 +736,11 @@ rmax.rule <- function(fun="K", W, lambda) {
            rlarge <- if(!missing(lambda)) sqrt(1000 /(pi * lambda)) else Inf
            rmax <- min(rlarge, ripley)
          },
+         Kscaled = {
+           ## rule of thumb for Kscaled
+           rdiam  <- diameter(as.rectangle(W))/2 * sqrt(lambda)
+           rmax <- min(10, rdiam)
+         },
          F = ,
          G = ,
          J = {
@@ -727,7 +761,7 @@ implemented.for.K <- function(correction, windowtype, explicit) {
   pixels <- (windowtype == "mask")
   if(any(correction == "best")) {
     # select best available correction
-    correction <- if(!pixels) "isotropic" else "translate"
+    correction[correction == "best"] <- if(!pixels) "isotropic" else "translate"
   } else {
     # available selection of edge corrections depends on window
     if(pixels) {
@@ -762,11 +796,11 @@ Krect.engine <- function(X, rmax, nr=100,
   npts <- npoints(X)
   W <- as.owin(X)
 
-  area <- area.owin(W)
+  areaW <- area(W)
   width <- sidelengths(W)[1]
   height <- sidelengths(W)[2]
-  lambda <- npts/area
-  lambda2 <- (npts * (npts - 1))/(area^2)
+  lambda <- npts/areaW
+  lambda2 <- (npts * (npts - 1))/(areaW^2)
 
   if(missing(rmax))
     rmax <- diameter(W)/4
@@ -779,7 +813,7 @@ Krect.engine <- function(X, rmax, nr=100,
     } else {
       wim <- as.im(weights, W)
       weights <- wim[X, drop=FALSE]
-      if(any(is.na(weights)))
+      if(anyNA(weights))
         stop("domain of weights image does not contain all points of X")
     }
   }
@@ -787,7 +821,7 @@ Krect.engine <- function(X, rmax, nr=100,
   # this will be the output data frame
   Kdf <- data.frame(r=r, theo= pi * r^2)
   desc <- c("distance argument r", "theoretical Poisson %s")
-  denom <- if(weighted) area else (lambda2 * area)
+  denom <- if(weighted) areaW else (lambda2 * areaW)
   Kfv <- ratfv(Kdf, NULL, denom,
                "r", quote(K(r)),
                "theo", NULL, c(0,rmax),
@@ -824,8 +858,6 @@ Krect.engine <- function(X, rmax, nr=100,
   ztrans <- numeric(if(doTrans) nr else 1L)
   
   ## call the C code
-  DUP <- spatstat.options("dupC")
-
   if(weighted) {
     ## weighted version
     zbnumer <- numeric(if(doBord) nr else 1L)
@@ -850,7 +882,7 @@ Krect.engine <- function(X, rmax, nr=100,
               bnumer=as.double(zbnumer),
               bdenom=as.double(zbdenom),
               unco=as.double(zunco),
-              DUP=DUP)
+              PACKAGE = "spatstat")
   } else if(npts < sqrt(.Machine$integer.max)) {
     ## unweighted
     ## numerator of border correction can be stored as an integer
@@ -876,7 +908,7 @@ Krect.engine <- function(X, rmax, nr=100,
               bnumer=as.integer(zbnumer),
               bdenom=as.integer(zbdenom),
               unco=as.integer(zunco),
-              DUP=DUP)
+              PACKAGE = "spatstat")
   } else {
     ## unweighted
     ## need double precision storage
@@ -901,7 +933,7 @@ Krect.engine <- function(X, rmax, nr=100,
               bnumer=as.double(zbnumer),
               bdenom=as.double(zbdenom),
               unco=as.double(zunco),
-              DUP=DUP)
+              PACKAGE = "spatstat")
   }
 
   ## Process corrections in reverse order of priority
@@ -909,7 +941,7 @@ Krect.engine <- function(X, rmax, nr=100,
   ## Uncorrected estimate
   if("none" %in% correction) {
     numKun <- res$unco
-    denKun <- if(weighted) area else (lambda2 * area)
+    denKun <- if(weighted) areaW else (lambda2 * areaW)
     Kfv <- bind.ratfv(Kfv,
                       data.frame(un=numKun),
                       denKun,
@@ -948,7 +980,7 @@ Krect.engine <- function(X, rmax, nr=100,
   ## translation correction
   if("translate" %in% correction) {
     numKtrans <- res$trans
-    denKtrans <- if(weighted) area else (lambda2 * area)
+    denKtrans <- if(weighted) areaW else (lambda2 * areaW)
     h <- diameter(as.rectangle(W))/2
     numKtrans[r >= h] <- NA
     Kfv <- bind.ratfv(Kfv,
@@ -962,7 +994,7 @@ Krect.engine <- function(X, rmax, nr=100,
   ## isotropic correction
   if("isotropic" %in% correction) {
     numKiso <- res$iso
-    denKiso <- if(weighted) area else (lambda2 * area)
+    denKiso <- if(weighted) areaW else (lambda2 * areaW)
     h <- diameter(as.rectangle(W))/2
     numKiso[r >= h] <- NA
     Kfv <- bind.ratfv(Kfv,
@@ -983,4 +1015,4 @@ Krect.engine <- function(X, rmax, nr=100,
 }
 
 
-
+  
