@@ -1,7 +1,7 @@
 #
 #           Kmeasure.R
 #
-#           $Revision: 1.46 $    $Date: 2014/02/17 02:17:49 $
+#           $Revision: 1.63 $    $Date: 2017/07/22 08:33:10 $
 #
 #     Kmeasure()         compute an estimate of the second order moment measure
 #
@@ -35,24 +35,24 @@ Kmeasure <- function(X, sigma, edge=TRUE, ..., varcov=NULL) {
     }
   }  
 
-  second.moment.calc(X, sigma=sigma, edge, "Kmeasure", varcov=varcov)
+  second.moment.calc(x=X, sigma=sigma, edge=edge,
+                     what="Kmeasure", varcov=varcov, ...)
 }
 
 second.moment.calc <- function(x, sigma=NULL, edge=TRUE,
-                               what="Kmeasure", debug=FALSE, ...,
-                               varcov=NULL, expand=FALSE) {
+                               what=c("Kmeasure", "kernel", "smooth", 
+                                 "Bartlett", "edge", "smoothedge", "all"),
+                               ..., 
+                               varcov=NULL, expand=FALSE, debug=FALSE) {
   if(is.null(sigma) && is.null(varcov))
     stop("must specify sigma or varcov")
-  choices <- c("kernel", "smooth", "Kmeasure", "Bartlett", "edge", "all", "smoothedge")
-  if(!(what %in% choices))
-    stop(paste("Unknown choice: what = ", sQuote(what),
-               "; available options are:",
-               paste(sQuote(choices), collapse=", ")))
+  what <- match.arg(what)
   sig <- if(!is.null(sigma)) sigma else max(c(diag(varcov), sqrt(det(varcov))))
 
   xtype <- if(is.ppp(x)) "ppp" else
-           if(is.im(x)) "im" else 
-           if(all(unlist(lapply(x, is.im)))) "imlist" else
+           if(is.im(x)) "im" else
+           if(inherits(x, "imlist")) "imlist" else 
+           if(all(sapply(x, is.im))) "imlist" else
            stop("x should be a point pattern or a pixel image")
 
   nimages <- switch(xtype,
@@ -83,7 +83,7 @@ second.moment.calc <- function(x, sigma=NULL, edge=TRUE,
            np <- NULL
          },
          imlist = {
-           Xlist <- x
+           X <- x
            np <- NULL
          })
 
@@ -142,10 +142,23 @@ second.moment.engine <-
            what=c("Kmeasure", "kernel", "smooth", 
              "Bartlett", "edge", "smoothedge", "all"),
            ...,
+           kernel="gaussian",
            obswin = as.owin(x), varcov=NULL,
            npts=NULL, debug=FALSE)
 {
   what <- match.arg(what)
+  validate2Dkernel(kernel)
+
+  is.second.order <- what %in% c("Kmeasure", "Bartlett", "all")
+  needs.kernel <- what %in% c("kernel", "all", "Kmeasure")
+  returns.several <- what %in% c("all", "smoothedge")
+
+  # check whether Fastest Fourier Transform in the West is available
+  west <- fftwAvailable()
+  
+  if(returns.several)
+    result <- list() # several results will be returned in a list
+
   if(is.ppp(x)) {
     # convert list of points to mass distribution
     X <- pixellate(x, ..., padzero=TRUE)
@@ -163,15 +176,14 @@ second.moment.engine <-
     names(blanklist) <- names(Xlist)
   } else stop("internal error: unrecognised format for x")
   unitsX <- unitname(X)
+  xstep <- X$xstep
+  ystep <- X$ystep
   # ensure obswin has same bounding frame as X
   if(!missing(obswin))
     obswin <- rebound.owin(obswin, as.rectangle(X))
   # go to work
   Y <- X$v
   Ylist <- lapply(Xlist, getElement, name="v")
-  #
-  xw <- X$xrange
-  yw <- X$yrange
   # pad with zeroes
   nr <- nrow(Y)
   nc <- ncol(Y)
@@ -182,63 +194,82 @@ second.moment.engine <-
   Ypad <- Ypadlist[[1]]
   lengthYpad <- 4 * nc * nr
   # corresponding coordinates
-  xw.pad <- xw[1] + 2 * c(0, diff(xw))
-  yw.pad <- yw[1] + 2 * c(0, diff(yw))
-  xcol.pad <- X$xcol[1] + X$xstep * (0:(2*nc-1))
-  yrow.pad <- X$yrow[1] + X$ystep * (0:(2*nr-1))
-  # set up Gauss kernel
-  xcol.G <- X$xstep * c(0:(nc-1),-(nc:1))
-  yrow.G <- X$ystep * c(0:(nr-1),-(nr:1))
-  Gpixarea <- X$xstep * X$ystep
-  if(!is.null(sigma)) {
-    densX.G <- dnorm(xcol.G, sd=sigma)
-    densY.G <- dnorm(yrow.G, sd=sigma)
-    Kern <- outer(densY.G, densX.G, "*") * Gpixarea
-  } else if(!is.null(varcov)) {
-    # anisotropic kernel
-    detSigma <- det(varcov)
-    Sinv <- solve(varcov)
-    halfSinv <- Sinv/2
-    constG <- Gpixarea/(2 * pi * sqrt(detSigma))
-    xsq <- matrix((xcol.G^2)[col(Ypad)], ncol=2*nc, nrow=2*nr)
-    ysq <- matrix((yrow.G^2)[row(Ypad)], ncol=2*nc, nrow=2*nr)
-    xy <- outer(yrow.G, xcol.G, "*")
-    Kern <- constG * exp(-(xsq * halfSinv[1,1]
-                           + xy * (halfSinv[1,2]+halfSinv[2,1])
-                           + ysq * halfSinv[2,2]))
-#    xx <- matrix(xcol.G[col(Ypad)], ncol=2*nc, nrow=2*nr)
-#    yy <- matrix(yrow.G[row(Ypad)], ncol=2*nc, nrow=2*nr)
-#    Kern <- const * exp(-(xx * (xx * Sinv[1,1] + yy * Sinv[1,2])
-#                          + yy * (xx * Sinv[2,1] + yy * Sinv[2,2]))/2)
-  } else 
-    stop("Must specify either sigma or varcov")
-
-  # these options call for several image outputs
-  if(what %in% c("all", "smoothedge"))
-    result <- list()
-  
-  if(what %in% c("kernel", "all")) {
-    # return the kernel
-    # first rearrange it into spatially sensible order (monotone x and y)
-    rtwist <- ((-nr):(nr-1)) %% (2 * nr) + 1
-    ctwist <- (-nc):(nc-1) %% (2*nc) + 1
-    if(debug) {
-      if(any(fave.order(xcol.G) != rtwist))
-        cat("something round the twist\n")
+  xcol.pad <- X$xcol[1] + xstep * (0:(2*nc-1))
+  yrow.pad <- X$yrow[1] + ystep * (0:(2*nr-1))
+  # compute kernel and its Fourier transform
+  if(!needs.kernel && 
+     identical(kernel, "gaussian") &&
+     is.numeric(sigma) && (length(sigma) == 1) &&
+     spatstat.options('developer')) {
+    # compute Fourier transform of kernel directly (*experimental*)
+    ii <- c(0:(nr-1), nr:1)
+    jj <- c(0:(nc-1), nc:1)
+    zz <- -sigma^2 * pi^2/2
+    uu <- exp(zz * ii^2)
+    vv <- exp(zz * jj^2)
+    fK <- outer(uu, vv, "*")
+  } else {
+    # set up kernel
+    xcol.ker <- xstep * c(0:(nc-1),-(nc:1))
+    yrow.ker <- ystep * c(0:(nr-1),-(nr:1))
+    kerpixarea <- xstep * ystep
+    if(identical(kernel, "gaussian")) {
+      if(!is.null(sigma)) {
+        densX.ker <- dnorm(xcol.ker, sd=sigma)
+        densY.ker <- dnorm(yrow.ker, sd=sigma)
+        #' WAS:  Kern <- outer(densY.ker, densX.ker, "*") * kerpixarea
+        Kern <- outer(densY.ker, densX.ker, "*")
+        Kern <- Kern/sum(Kern)
+      } else if(!is.null(varcov)) {
+        ## anisotropic kernel
+        detSigma <- det(varcov)
+        Sinv <- solve(varcov)
+        halfSinv <- Sinv/2
+        constker <- kerpixarea/(2 * pi * sqrt(detSigma))
+        xsq <- matrix((xcol.ker^2)[col(Ypad)], ncol=2*nc, nrow=2*nr)
+        ysq <- matrix((yrow.ker^2)[row(Ypad)], ncol=2*nc, nrow=2*nr)
+        xy <- outer(yrow.ker, xcol.ker, "*")
+        Kern <- constker * exp(-(xsq * halfSinv[1,1]
+                                 + xy * (halfSinv[1,2]+halfSinv[2,1])
+                                 + ysq * halfSinv[2,2]))
+        Kern <- Kern/sum(Kern)
+      } else 
+        stop("Must specify either sigma or varcov")
+    } else {
+      ## non-Gaussian kernel
+      ## evaluate kernel at array of points
+      xker <- as.vector(xcol.ker[col(Ypad)])
+      yker <- as.vector(yrow.ker[row(Ypad)])
+      Kern <- evaluate2Dkernel(kernel, xker, yker,
+                               sigma=sigma, varcov=varcov, ...) * kerpixarea
+      Kern <- matrix(Kern, ncol=2*nc, nrow=2*nr)
+      Kern <- Kern/sum(Kern)
     }
-    Kermit <- Kern[ rtwist, ctwist]
-    ker <- im(Kermit, xcol.G[ctwist], yrow.G[ rtwist], unitname=unitsX)
-    if(what == "kernel")
-      return(ker)
-    else 
-      result$kernel <- ker
+
+    if(what %in% c("kernel", "all")) {
+      ## kernel will be returned
+      ## first rearrange it into spatially sensible order (monotone x and y)
+      rtwist <- ((-nr):(nr-1)) %% (2 * nr) + 1
+      ctwist <- (-nc):(nc-1) %% (2*nc) + 1
+      if(debug) {
+        if(any(fave.order(xcol.ker) != rtwist))
+          cat("something round the twist\n")
+      }
+      Kermit <- Kern[ rtwist, ctwist]
+      ker <- im(Kermit, xcol.ker[ctwist], yrow.ker[ rtwist], unitname=unitsX)
+      if(what == "kernel")
+        return(ker)
+      else 
+        result$kernel <- ker
+    }
+    ## convolve using fft
+    fK <- fft2D(Kern, west=west)
   }
-  # convolve using fft
-  fK <- fft(Kern)
+  
   if(what != "edge") {
     if(nimages == 1) {
-      fY <- fft(Ypad)
-      sm <- fft(fY * fK, inverse=TRUE)/lengthYpad
+      fY <- fft2D(Ypad, west=west)
+      sm <- fft2D(fY * fK, inverse=TRUE, west=west)/lengthYpad
       if(debug) {
         cat(paste("smooth: maximum imaginary part=",
                   signif(max(Im(sm)),3), "\n"))
@@ -249,8 +280,9 @@ second.moment.engine <-
     } else {
       fYlist <- smlist <- blanklist
       for(i in 1:nimages) {
-        fYlist[[i]] <- fY.i <- fft(Ypadlist[[i]])
-        smlist[[i]] <- sm.i <- fft(fY.i * fK, inverse=TRUE)/lengthYpad
+        fYlist[[i]] <- fY.i <- fft2D(Ypadlist[[i]], west=west)
+        smlist[[i]] <- sm.i <-
+          fft2D(fY.i * fK, inverse=TRUE, west=west)/lengthYpad
         if(debug) {
           cat(paste("smooth component", i, ": maximum imaginary part=",
                     signif(max(Im(sm.i)),3), "\n"))
@@ -262,7 +294,7 @@ second.moment.engine <-
     }
   }
   if(what %in% c("smooth", "all", "smoothedge")) {
-    # return the smoothed point pattern without edge correction
+    # compute smoothed point pattern without edge correction
     if(nimages == 1) {
       smo <- im(Re(sm)[1:nr, 1:nc],
                 xcol.pad[1:nc], yrow.pad[1:nr],
@@ -278,7 +310,7 @@ second.moment.engine <-
         smolist[[i]] <- im(Re(smlist[[i]])[1:nr, 1:nc],
                            xcol.pad[1:nc], yrow.pad[1:nr],
                            unitname=unitsX)
-      smolist <- as.listof(smolist)
+      smolist <- as.solist(smolist)
       if(what == "smooth") {
         return(smolist)
       } else {
@@ -287,17 +319,17 @@ second.moment.engine <-
     }
   }
 
-  if(what != "edge") {
+  if(is.second.order) {
     # compute Bartlett spectrum
     if(nimages == 1) {
-      bart <- Mod(fY)^2 * fK
+      bart <- BartCalc(fY, fK)  ##  bart <- Mod(fY)^2 * fK
     } else {
-      bartlist <- lapply(fYlist, function(z, fK) { Mod(z)^2 * fK}, fK=fK)
+      bartlist <- lapply(fYlist, BartCalc, fK=fK)
     }
   }
   
   if(what %in% c("Bartlett", "all")) {
-     # return Bartlett spectrum
+     # Bartlett spectrum will be returned
      # rearrange into spatially sensible order (monotone x and y)
     rtwist <- ((-nr):(nr-1)) %% (2 * nr) + 1
     ctwist <- (-nc):(nc-1) %% (2*nc) + 1
@@ -314,7 +346,7 @@ second.moment.engine <-
         Bart <- (bartlist[[i]])[ rtwist, ctwist]
         Bartlist[[i]] <- im(Mod(Bart),(-nc):(nc-1), (-nr):(nr-1))
       }
-      Bartlist <- as.listof(Bartlist)
+      Bartlist <- as.solist(Bartlist)
       if(what == "Bartlett")
         return(Bartlist)
       else
@@ -324,9 +356,9 @@ second.moment.engine <-
   
   #### ------- Second moment measure --------------
   #
-  if(what != "edge") {
+  if(is.second.order) {
     if(nimages == 1) {
-      mom <- fft(bart, inverse=TRUE)/lengthYpad
+      mom <- fft2D(bart, inverse=TRUE, west=west)/lengthYpad
       if(debug) {
         cat(paste("2nd moment measure: maximum imaginary part=",
                   signif(max(Im(mom)),3), "\n"))
@@ -342,7 +374,7 @@ second.moment.engine <-
     } else {
       momlist <- blanklist
       for(i in 1:nimages) {
-        mom.i <- fft(bartlist[[i]], inverse=TRUE)/lengthYpad
+        mom.i <- fft2D(bartlist[[i]], inverse=TRUE, west=west)/lengthYpad
         if(debug) {
           cat(paste("2nd moment measure: maximum imaginary part=",
                     signif(max(Im(mom.i)),3), "\n"))
@@ -361,16 +393,16 @@ second.moment.engine <-
   }
   # edge correction
   if(edge || what %in% c("edge", "all", "smoothedge")) {
-    # compute kernel-smoothed set covariance
-    M <- as.mask(obswin, dimyx=c(nr, nc))$m
+    M <- as.mask(obswin, xy=list(x=X$xcol, y=X$yrow))$m
     # previous line ensures M has same dimensions and scale as Y 
     Mpad <- matrix(0, ncol=2*nc, nrow=2*nr)
     Mpad[1:nr, 1:nc] <- M
     lengthMpad <- 4 * nc * nr
-    fM <- fft(Mpad)
-    if(edge && what != "edge") {
+    fM <- fft2D(Mpad, west=west)
+    if(edge && is.second.order) {
+      # compute kernel-smoothed set covariance
       # apply edge correction      
-      co <- fft(Mod(fM)^2 * fK, inverse=TRUE)/lengthMpad
+      co <- fft2D(Mod(fM)^2 * fK, inverse=TRUE, west=west)/lengthMpad
       co <- Mod(co) 
       a <- sum(M)
       wt <- a/co
@@ -393,8 +425,9 @@ second.moment.engine <-
       }
     }
   }
-  if(what != "edge") {
-    # rearrange into spatially sensible order (monotone x and y)
+  if(is.second.order) {
+    # rearrange second moment measure
+    # into spatially sensible order (monotone x and y)
     rtwist <- ((-nr):(nr-1)) %% (2 * nr) + 1
     ctwist <- (-nc):(nc-1) %% (2*nc) + 1
     if(nimages == 1) {
@@ -403,14 +436,14 @@ second.moment.engine <-
       momlist <- lapply(momlist, "[", i=rtwist, j=ctwist)
     }
     if(debug) {
-      if(any(fave.order(xcol.G) != rtwist))
+      if(any(fave.order(xcol.ker) != rtwist))
         cat("internal error: something round the twist\n")
     }
   }
   if(what %in% c("edge", "all", "smoothedge")) {
     # return convolution of window with kernel
     # (evaluated inside window only)
-    con <- fft(fM * fK, inverse=TRUE)/lengthMpad
+    con <- fft2D(fM * fK, inverse=TRUE, west=west)/lengthMpad
     edg <- Mod(con[1:nr, 1:nc])
     edg <- im(edg, xcol.pad[1:nc], yrow.pad[1:nr], unitname=unitsX)
     if(what == "edge") 
@@ -422,26 +455,26 @@ second.moment.engine <-
     return(result)
   # Second moment measure, density estimate
   # Divide by number of points * lambda and convert mass to density
-  pixarea <- with(X, xstep * ystep)
+  pixarea <- xstep * ystep
   if(nimages == 1) {
-    mom <- mom * area.owin(obswin) / (pixarea * npts^2)
+    mom <- mom * area(obswin) / (pixarea * npts^2)
     # this is the second moment measure
-    mm <- im(mom, xcol.G[ctwist], yrow.G[rtwist], unitname=unitsX)
+    mm <- im(mom, xcol.ker[ctwist], yrow.ker[rtwist], unitname=unitsX)
     if(what == "Kmeasure")
       return(mm)
     else 
       result$Kmeasure <- mm
   } else {
-    ccc <- area.owin(obswin) / (pixarea * npts^2)
+    ccc <- area(obswin) / (pixarea * npts^2)
     mmlist <- blanklist
     for(i in 1:nimages) {
       mom.i <- momlist[[i]]
       mom.i <- mom.i * ccc
       # this is the second moment measure
       mmlist[[i]] <-
-        im(mom.i, xcol.G[ctwist], yrow.G[rtwist], unitname=unitsX)
+        im(mom.i, xcol.ker[ctwist], yrow.ker[rtwist], unitname=unitsX)
     }
-    mmlist <- as.listof(mmlist)
+    mmlist <- as.solist(mmlist)
     if(what == "Kmeasure")
       return(mmlist)
     else 
@@ -451,20 +484,22 @@ second.moment.engine <-
   return(result)
 }
 
-Kest.fft <- function(X, sigma, r=NULL, breaks=NULL) {
+BartCalc <- function(fY, fK) { Mod(fY)^2 * fK }
+  
+Kest.fft <- function(X, sigma, r=NULL, ..., breaks=NULL) {
   verifyclass(X, "ppp")
-  W <- X$window
-  lambda <- X$n/area.owin(W)
+  W <- Window(X)
+  lambda <- npoints(X)/area(W)
   rmaxdefault <- rmax.rule("K", W, lambda)        
   bk <- handle.r.b.args(r, breaks, W, rmaxdefault=rmaxdefault)
   breaks <- bk$val
   rvalues <- bk$r
-  u <- Kmeasure(X, sigma)
+  u <- Kmeasure(X, sigma, ...)
   xx <- rasterx.im(u)
   yy <- rastery.im(u)
   rr <- sqrt(xx^2 + yy^2)
   tr <- whist(rr, breaks, u$v)
-  K  <- cumsum(tr)
+  K  <- cumsum(tr) * with(u, xstep * ystep)
   rmax <- min(rr[is.na(u$v)])
   K[rvalues >= rmax] <- NA
   result <- data.frame(r=rvalues, theo=pi * rvalues^2, border=K)
