@@ -1,18 +1,31 @@
 #
 # closepairs.R
 #
-#   $Revision: 1.25 $   $Date: 2013/09/21 08:56:11 $
+#   $Revision: 1.39 $   $Date: 2017/06/05 10:31:58 $
 #
 #  simply extract the r-close pairs from a dataset
 # 
 #  Less memory-hungry for large patterns
 #
-closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
+
+closepairs <- function(X, rmax, ...) {
+  UseMethod("closepairs")
+}
+  
+closepairs.ppp <- function(X, rmax, twice=TRUE,
+                           what=c("all", "indices", "ijd"),
+                           distinct=TRUE, neat=TRUE, 
+                           ...) {
   verifyclass(X, "ppp")
   what <- match.arg(what)
-  stopifnot(is.numeric(rmax) && length(rmax) == 1)
+  stopifnot(is.numeric(rmax) && length(rmax) == 1L)
   stopifnot(is.finite(rmax))
   stopifnot(rmax >= 0)
+  ordered <- list(...)$ordered
+  if(missing(twice) && !is.null(ordered)) {
+    warning("Obsolete argument 'ordered' has been replaced by 'twice'")
+    twice <- ordered
+  }
   npts <- npoints(X)
   null.answer <- switch(what,
                         all = {
@@ -29,22 +42,35 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
                         indices = {
                           list(i=integer(0),
                                j=integer(0))
+                        },
+                        ijd = {
+                          list(i=integer(0),
+                               j=integer(0),
+                               d=numeric(0))
                         })
-   if(npts == 0)
-     return(null.answer)
+  if(npts == 0)
+    return(null.answer)
   # sort points by increasing x coordinate
   oo <- fave.order(X$x)
   Xsort <- X[oo]
-  # First make an OVERESTIMATE of the number of pairs
-  nsize <- ceiling(4 * pi * (npts^2) * (rmax^2)/area.owin(X$window))
+  # First make an OVERESTIMATE of the number of unordered pairs
+  nsize <- ceiling(2 * pi * (npts^2) * (rmax^2)/area(Window(X)))
   nsize <- max(1024, nsize)
+  if(nsize > .Machine$integer.max) {
+    warning("Estimated number of close pairs exceeds maximum possible integer",
+            call.=FALSE)
+    nsize <- .Machine$integer.max
+  }
   # Now extract pairs
   if(spatstat.options("closepairs.newcode")) {
     # ------------------- use new faster code ---------------------
+    # fast algorithms collect each distinct pair only once
+    got.twice <- FALSE
+    ng <- nsize
+    #
     x <- Xsort$x
     y <- Xsort$y
     r <- rmax
-    ng <- nsize
     storage.mode(x) <- "double"
     storage.mode(y) <- "double"
     storage.mode(r) <- "double"
@@ -52,33 +78,72 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
     switch(what,
            all = {
              z <- .Call("Vclosepairs",
-                        xx=x, yy=y, rr=r, nguess=ng)
-#                        PACKAGE="spatstat")
+                        xx=x, yy=y, rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
              if(length(z) != 9)
                stop("Internal error: incorrect format returned from Vclosepairs")
-             i  <- z[[1]]  # NB no increment required
-             j  <- z[[2]]
-             xi <- z[[3]]
-             yi <- z[[4]]
-             xj <- z[[5]]
-             yj <- z[[6]]
-             dx <- z[[7]]
-             dy <- z[[8]]
-             d  <- z[[9]]
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+             xi <- z[[3L]]
+             yi <- z[[4L]]
+             xj <- z[[5L]]
+             yj <- z[[6L]]
+             dx <- z[[7L]]
+             dy <- z[[8L]]
+             d  <- z[[9L]]
            },
            indices = {
              z <- .Call("VcloseIJpairs",
-                        xx=x, yy=y, rr=r, nguess=ng)
-#                        PACKAGE="spatstat")
+                        xx=x, yy=y, rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
              if(length(z) != 2)
                stop("Internal error: incorrect format returned from VcloseIJpairs")
-             i  <- z[[1]]  # NB no increment required
-             j  <- z[[2]]
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+           },
+           ijd = {
+             z <- .Call("VcloseIJDpairs",
+                        xx=x, yy=y, rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
+             if(length(z) != 3)
+               stop("Internal error: incorrect format returned from VcloseIJDpairs")
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+             d  <- z[[3L]]
            })
 
   } else {
     # ------------------- use older code --------------------------
-    DUP <- spatstat.options("dupC")
+    if(!distinct) {
+      ii <- seq_len(npts)
+      xx <- X$x
+      yy <- X$y
+      zeroes <- rep(0, npts)
+      null.answer <- switch(what,
+                            all = {
+                              list(i=ii,
+                                   j=ii,
+                                   xi=xx,
+                                   yi=yy,
+                                   xj=xx,
+                                   yj=yy,
+                                   dx=zeroes,
+                                   dy=zeroes,
+                                   d=zeroes)
+                            },
+                            indices = {
+                              list(i=ii,
+                                   j=ii)
+                            },
+                            ijd = {
+                              list(i=ii,
+                                   j=ii,
+                                   d=zeroes)
+                            })
+    }
+
+    got.twice <- TRUE
+    nsize <- nsize * 2
     z <-
       .C("Fclosepairs",
          nxy=as.integer(npts),
@@ -86,7 +151,7 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
          y=as.double(Xsort$y),
          r=as.double(rmax),
          noutmax=as.integer(nsize), 
-         nout=as.integer(integer(1)),
+         nout=as.integer(integer(1L)),
          iout=as.integer(integer(nsize)),
          jout=as.integer(integer(nsize)), 
          xiout=as.double(numeric(nsize)),
@@ -96,9 +161,8 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
          dxout=as.double(numeric(nsize)),
          dyout=as.double(numeric(nsize)),
          dout=as.double(numeric(nsize)),
-         status=as.integer(integer(1)),
-         DUP=DUP)
-#         PACKAGE="spatstat")
+         status=as.integer(integer(1L)),
+         PACKAGE = "spatstat")
 
     if(z$status != 0) {
       # Guess was insufficient
@@ -110,9 +174,8 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
                   x=as.double(Xsort$x),
                   y=as.double(Xsort$y),
                   rmaxi=as.double(rmaxplus),
-                  count=as.integer(integer(1)),
-                  DUP=DUP)$count
-#                  PACKAGE="spatstat")$count
+                  count=as.integer(integer(1L)),
+                  PACKAGE = "spatstat")$count
       if(nsize <= 0)
         return(null.answer)
       # add a bit more for safety
@@ -125,7 +188,7 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
            y=as.double(Xsort$y),
            r=as.double(rmax),
            noutmax=as.integer(nsize), 
-           nout=as.integer(integer(1)),
+           nout=as.integer(integer(1L)),
            iout=as.integer(integer(nsize)),
            jout=as.integer(integer(nsize)), 
            xiout=as.double(numeric(nsize)),
@@ -135,9 +198,8 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
            dxout=as.double(numeric(nsize)),
            dyout=as.double(numeric(nsize)),
            dout=as.double(numeric(nsize)),
-           status=as.integer(integer(1)),
-           DUP=DUP)
-#           PACKAGE="spatstat")
+           status=as.integer(integer(1L)),
+           PACKAGE = "spatstat")
       if(z$status != 0)
         stop(paste("Internal error: C routine complains that insufficient space was allocated:", nsize))
     }
@@ -148,37 +210,117 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
     actual <- seq_len(npairs)
     i  <- z$iout[actual]  # sic
     j  <- z$jout[actual]
-    if(what == "all") {
-      xi <- z$xiout[actual]
-      yi <- z$yiout[actual]
-      xj <- z$xjout[actual]
-      yj <- z$yjout[actual]
-      dx <- z$dxout[actual]
-      dy <- z$dyout[actual]
-      d <-  z$dout[actual]
-    }
+    switch(what,
+           indices={},
+           all={
+             xi <- z$xiout[actual]
+             yi <- z$yiout[actual]
+             xj <- z$xjout[actual]
+             yj <- z$yjout[actual]
+             dx <- z$dxout[actual]
+             dy <- z$dyout[actual]
+             d <-  z$dout[actual]
+           },
+           ijd = {
+             d <- z$dout[actual]
+           })
     # ------------------- end code switch ------------------------
   }
   
   # convert i,j indices to original sequence
   i <- oo[i]
   j <- oo[j]
-  # are (i, j) and (j, i) equivalent?
-  if(!ordered) {
-    ok <- (i < j)
-    i  <-  i[ok]
-    j  <-  j[ok]
-    if(what == "all") {
-      xi <- xi[ok]
-      yi <- yi[ok]
-      xj <- xj[ok]
-      yj <- yj[ok]
-      dx <- dx[ok]
-      dy <- dy[ok]
-      d  <-  d[ok]
+  if(twice) {
+    ## both (i, j) and (j, i) should be returned
+    if(!got.twice) {
+      ## duplication required
+      iold <- i
+      jold <- j
+      i <- c(iold, jold)
+      j <- c(jold, iold)
+      switch(what,
+             indices = { },
+             ijd = {
+               d <- rep(d, 2)
+             },
+             all = {
+               xinew <- c(xi, xj)
+               yinew <- c(yi, yj)
+               xjnew <- c(xj, xi)
+               yjnew <- c(yj, yi)
+               xi <- xinew
+               yi <- yinew
+               xj <- xjnew
+               yj <- yjnew
+               dx <- c(dx, -dx)
+               dy <- c(dy, -dy)
+               d <- rep(d, 2)
+             })
     }
+  } else {
+    ## only one of (i, j) and (j, i) should be returned
+    if(got.twice) {
+      ## remove duplication
+      ok <- (i < j)
+      i  <-  i[ok]
+      j  <-  j[ok]
+      switch(what,
+             indices = { },
+             all = {
+               xi <- xi[ok]
+               yi <- yi[ok]
+               xj <- xj[ok]
+               yj <- yj[ok]
+               dx <- dx[ok]
+               dy <- dy[ok]
+               d  <-  d[ok]
+             },
+             ijd = {
+               d  <-  d[ok]
+             })
+    } else if(neat) {
+      ## enforce i < j
+      swap <- (i > j)
+      tmp <- i[swap]
+      i[swap] <- j[swap]
+      j[swap] <- tmp
+      if(what == "all") {
+        xinew <- ifelse(swap, xj, xi)
+        yinew <- ifelse(swap, yj, yi)
+        xjnew <- ifelse(swap, xi, xj)
+        yjnew <- ifelse(swap, yi, yj)
+        xi <- xinew
+        yi <- yinew
+        xj <- xjnew
+        yj <- yjnew
+        dx[swap] <- -dx[swap]
+        dy[swap] <- -dy[swap]
+      }
+    } ## otherwise no action required
   }
-  # done
+  ## add pairs of identical points?
+  if(!distinct) {
+    ii <- seq_len(npts)
+    xx <- X$x
+    yy <- X$y
+    zeroes <- rep(0, npts)
+    i <- c(i, ii)
+    j <- c(j, ii)
+    switch(what,
+           ijd={
+             d  <- c(d, zeroes)
+           },
+           all = {
+             xi <- c(xi, xx)
+             yi <- c(yi, yy)
+             xj <- c(xj, xx)
+             yi <- c(yi, yy)
+             dx <- c(dx, zeroes)
+             dy <- c(dy, zeroes)
+             d  <- c(d, zeroes)
+           })
+  }
+  ## done
   switch(what,
          all = {
            answer <- list(i=i,
@@ -193,17 +335,24 @@ closepairs <- function(X, rmax, ordered=TRUE, what=c("all", "indices")) {
          },
          indices = {
            answer <- list(i = i, j = j)
+         },
+         ijd = {
+           answer <- list(i=i, j=j, d=d)
          })
   return(answer)
 }
 
 #######################
 
-crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
+crosspairs <- function(X, Y, rmax, ...) {
+  UseMethod("crosspairs")
+}
+
+crosspairs.ppp <- function(X, Y, rmax, what=c("all", "indices", "ijd"), ...) {
   verifyclass(X, "ppp")
   verifyclass(Y, "ppp")
   what <- match.arg(what)
-  stopifnot(is.numeric(rmax) && length(rmax) == 1 && rmax >= 0)
+  stopifnot(is.numeric(rmax) && length(rmax) == 1L && rmax >= 0)
   null.answer <- switch(what,
                         all = {
                           list(i=integer(0),
@@ -219,6 +368,11 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
                         indices = {
                           list(i=integer(0),
                                j=integer(0))
+                        },
+                        ijd = {
+                          list(i=integer(0),
+                               j=integer(0),
+                               d=numeric(0))
                         })
   nX <- npoints(X)
   nY <- npoints(Y)
@@ -231,8 +385,14 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
   if(spatstat.options("crosspairs.newcode")) {
     # ------------------- use new faster code ---------------------
     # First (over)estimate the number of pairs
-    nsize <- ceiling(2 * pi * (rmax^2) * nX * nY/area.owin(Y$window))
+    nsize <- ceiling(2 * pi * (rmax^2) * nX * nY/area(Window(Y)))
     nsize <- max(1024, nsize)
+    if(nsize > .Machine$integer.max) {
+      warning(
+        "Estimated number of close pairs exceeds maximum possible integer",
+        call.=FALSE)
+      nsize <- .Machine$integer.max
+    }
     # .Call
     Xx <- Xsort$x
     Xy <- Xsort$y
@@ -249,37 +409,48 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
              z <- .Call("Vcrosspairs",
                         xx1=Xx, yy1=Xy,
                         xx2=Yx, yy2=Yy,
-                        rr=r, nguess=ng)
-#                        PACKAGE="spatstat")
+                        rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
              if(length(z) != 9)
                stop("Internal error: incorrect format returned from Vcrosspairs")
-             i  <- z[[1]]  # NB no increment required
-             j  <- z[[2]]
-             xi <- z[[3]]
-             yi <- z[[4]]
-             xj <- z[[5]]
-             yj <- z[[6]]
-             dx <- z[[7]]
-             dy <- z[[8]]
-             d  <- z[[9]]
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+             xi <- z[[3L]]
+             yi <- z[[4L]]
+             xj <- z[[5L]]
+             yj <- z[[6L]]
+             dx <- z[[7L]]
+             dy <- z[[8L]]
+             d  <- z[[9L]]
            },
            indices = {
              z <- .Call("VcrossIJpairs",
                         xx1=Xx, yy1=Xy,
                         xx2=Yx, yy2=Yy,
-                        rr=r, nguess=ng)
-#                        PACKAGE="spatstat")
+                        rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
              if(length(z) != 2)
                stop("Internal error: incorrect format returned from VcrossIJpairs")
-             i  <- z[[1]]  # NB no increment required
-             j  <- z[[2]]
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+           }, 
+           ijd = {
+             z <- .Call("VcrossIJDpairs",
+                        xx1=Xx, yy1=Xy,
+                        xx2=Yx, yy2=Yy,
+                        rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
+             if(length(z) != 3)
+               stop("Internal error: incorrect format returned from VcrossIJDpairs")
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+             d  <- z[[3L]]
            })
            
   } else {
     # Older code 
     # obtain upper estimate of number of pairs
     # (to work around gcc bug 323)
-    DUP <- spatstat.options("dupC")
     rmaxplus <- 1.25 * rmax
     nsize <- .C("crosscount",
                 nn1=as.integer(X$n),
@@ -289,9 +460,8 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
                 x2=as.double(Ysort$x),
                 y2=as.double(Ysort$y),
                 rmaxi=as.double(rmaxplus),
-                count=as.integer(integer(1)),
-                DUP=DUP)$count
-#                PACKAGE="spatstat")$count
+                count=as.integer(integer(1L)),
+                PACKAGE = "spatstat")$count
     if(nsize <= 0)
       return(null.answer)
 
@@ -309,7 +479,7 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
          y2=as.double(Ysort$y),
          r=as.double(rmax),
          noutmax=as.integer(nsize), 
-         nout=as.integer(integer(1)),
+         nout=as.integer(integer(1L)),
          iout=as.integer(integer(nsize)),
          jout=as.integer(integer(nsize)), 
          xiout=as.double(numeric(nsize)),
@@ -319,9 +489,8 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
          dxout=as.double(numeric(nsize)),
          dyout=as.double(numeric(nsize)),
          dout=as.double(numeric(nsize)),
-         status=as.integer(integer(1)),
-         DUP=DUP)
-#         PACKAGE="spatstat")
+         status=as.integer(integer(1L)),
+         PACKAGE = "spatstat")
     if(z$status != 0)
       stop(paste("Internal error: C routine complains that insufficient space was allocated:", nsize))
     # trim vectors to the length indicated
@@ -357,18 +526,26 @@ crosspairs <- function(X, Y, rmax, what=c("all", "indices")) {
          },
          indices = {
            answer <- list(i=i, j=j)
+         },
+         ijd = {
+           answer <- list(i=i, j=j, d=d)
          })
   return(answer)
 }
 
-closethresh <- function(X, R, S, ordered=TRUE) {
+closethresh <- function(X, R, S, twice=TRUE, ...) {
   # list all R-close pairs
   # and indicate which of them are S-close (S < R)
   # so that results are consistent with closepairs(X,S)
   verifyclass(X, "ppp")
-  stopifnot(is.numeric(R) && length(R) == 1 && R >= 0)
-  stopifnot(is.numeric(S) && length(S) == 1 && S >= 0)
+  stopifnot(is.numeric(R) && length(R) == 1L && R >= 0)
+  stopifnot(is.numeric(S) && length(S) == 1L && S >= 0)
   stopifnot(S < R)
+  ordered <- list(...)$ordered
+  if(missing(twice) && !is.null(ordered)) {
+    warning("Obsolete argument 'ordered' has been replaced by 'twice'")
+    twice <- ordered
+  }
   npts <- npoints(X)
    if(npts == 0)
      return(list(i=integer(0), j=integer(0), t=logical(0)))
@@ -376,8 +553,13 @@ closethresh <- function(X, R, S, ordered=TRUE) {
   oo <- fave.order(X$x)
   Xsort <- X[oo]
   # First make an OVERESTIMATE of the number of pairs
-  nsize <- ceiling(4 * pi * (npts^2) * (R^2)/area.owin(X$window))
+  nsize <- ceiling(4 * pi * (npts^2) * (R^2)/area(Window(X)))
   nsize <- max(1024, nsize)
+  if(nsize > .Machine$integer.max) {
+    warning("Estimated number of close pairs exceeds maximum possible integer",
+            call.=FALSE)
+    nsize <- .Machine$integer.max
+  }
   # Now extract pairs
   x <- Xsort$x
   y <- Xsort$y
@@ -390,22 +572,24 @@ closethresh <- function(X, R, S, ordered=TRUE) {
   storage.mode(s) <- "double"
   storage.mode(ng) <- "integer"
   z <- .Call("Vclosethresh",
-             xx=x, yy=y, rr=r, ss=s, nguess=ng)
+             xx=x, yy=y, rr=r, ss=s, nguess=ng,
+             PACKAGE = "spatstat")
   if(length(z) != 3)
     stop("Internal error: incorrect format returned from Vclosethresh")
-  i  <- z[[1]]  # NB no increment required
-  j  <- z[[2]]
-  th <- as.logical(z[[3]])
+  i  <- z[[1L]]  # NB no increment required
+  j  <- z[[2L]]
+  th <- as.logical(z[[3L]])
   
   # convert i,j indices to original sequence
   i <- oo[i]
   j <- oo[j]
-  # are (i, j) and (j, i) equivalent?
-  if(!ordered) {
-    ok <- (i < j)
-    i  <-  i[ok]
-    j  <-  j[ok]
-    th <- th[ok]
+  # fast C code only returns i < j
+  if(twice) {
+    iold <- i
+    jold <- j
+    i <- c(iold, jold)
+    j <- c(jold, iold)
+    th <- rep(th, 2)
   }
   # done
   return(list(i=i, j=j, th=th))
@@ -426,4 +610,3 @@ crosspairquad <- function(Q, rmax, what=c("all", "indices")) {
   return(result)
 }
 
-  
