@@ -3,7 +3,7 @@
 #
 #  Method for 'density' for point patterns
 #
-#  $Revision: 1.85 $    $Date: 2016/11/13 01:54:32 $
+#  $Revision: 1.86 $    $Date: 2017/06/05 10:31:58 $
 #
 
 ksmooth.ppp <- function(x, sigma, ..., edge=TRUE) {
@@ -19,7 +19,7 @@ density.ppp <- function(x, sigma=NULL, ...,
                         adjust=1, diggle=FALSE, se=FALSE, 
                         kernel="gaussian",
                         scalekernel=is.character(kernel),
-                        positive=FALSE) {
+                        positive=FALSE, verbose=TRUE) {
   verifyclass(x, "ppp")
 
   output <- pickoption("output location type", at,
@@ -31,7 +31,7 @@ density.ppp <- function(x, sigma=NULL, ...,
     ## kernel is only partly implemented!
     if(se)
       stop("Standard errors are not implemented for non-Gaussian kernel")
-    if(is.function(sigma) || (is.null(sigma) && is.null(varcov)))
+    if(verbose && (is.function(sigma) || (is.null(sigma) && is.null(varcov))))
       warning("Bandwidth selection will be based on Gaussian kernel")
   }
   
@@ -65,7 +65,7 @@ density.ppp <- function(x, sigma=NULL, ...,
                                   weights=weights, edge=edge,
                                   leaveoneout=leaveoneout,
                                   diggle=diggle, ...)
-    if(!is.null(uhoh <- attr(result, "warnings"))) {
+    if(verbose && !is.null(uhoh <- attr(result, "warnings"))) {
       switch(uhoh,
              underflow=warning("underflow due to very small bandwidth"),
              warning(uhoh))
@@ -261,15 +261,15 @@ densitypointsEngine <- function(x, sigma, ...,
   }
   # validate weights
   if(is.null(weights)) {
-    k <- 1
+    k <- 1L
   } else if(is.matrix(weights) || is.data.frame(weights)) {
     k <- ncol(weights)
     stopifnot(nrow(weights) == npoints(x))
     weights <- as.data.frame(weights)
     weightnames <- colnames(weights)
   } else {
-    k <- 1
-    stopifnot(length(weights) == npoints(x) || length(weights) == 1)
+    k <- 1L
+    stopifnot(length(weights) == npoints(x) || length(weights) == 1L)
   }
   # evaluate edge correction weights at points 
   if(edge) {
@@ -281,9 +281,9 @@ densitypointsEngine <- function(x, sigma, ...,
       xx <- x$x
       yy <- x$y
       xprob <-
-        pnorm(xr[2], mean=xx, sd=sigma) - pnorm(xr[1], mean=xx, sd=sigma)
+        pnorm(xr[2L], mean=xx, sd=sigma) - pnorm(xr[1L], mean=xx, sd=sigma)
       yprob <-
-        pnorm(yr[2], mean=yy, sd=sigma) - pnorm(yr[1], mean=yy, sd=sigma)
+        pnorm(yr[2L], mean=yy, sd=sigma) - pnorm(yr[1L], mean=yy, sd=sigma)
       edgeweight <- xprob * yprob
     } else {
       edg <- second.moment.calc(x, sigma=sigma,
@@ -296,7 +296,7 @@ densitypointsEngine <- function(x, sigma, ...,
       # Diggle edge correction
       # edgeweight is attached to each point
       if(is.null(weights)) {
-        k <- 1
+        k <- 1L
         weights <- 1/edgeweight
       } else {
         weights <- weights/edgeweight
@@ -310,7 +310,7 @@ densitypointsEngine <- function(x, sigma, ...,
     if(debugging)
       cat('Using experimental code!\n')
     npts <- npoints(x)
-    result <- if(k == 1) numeric(npts) else matrix(, npts, k)
+    result <- if(k == 1L) numeric(npts) else matrix(, npts, k)
     xx <- x$x
     yy <- x$y
     ## transform to standard coordinates
@@ -319,8 +319,8 @@ densitypointsEngine <- function(x, sigma, ...,
       yy <- yy/(sqrt(2) * sigma)
     } else {
       xy <- cbind(xx, yy) %*% matrixsqrt(Sinv/2)
-      xx <- xy[,1]
-      yy <- xy[,2]
+      xx <- xy[,1L]
+      yy <- xy[,2L]
       sorted <- FALSE
     }
     ## cutoff in standard coordinates
@@ -337,10 +337,11 @@ densitypointsEngine <- function(x, sigma, ...,
                x       = as.double(xx),
                y       = as.double(yy),
                rmaxi   = as.double(cutoff),
-               result  = as.double(double(npts)))
+               result  = as.double(double(npts)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[oo] <- zz$result
       result <- result * const
-    } else if(k == 1) {
+    } else if(k == 1L) {
       wtsort <- if(sorted) weights else weights[oo]
       zz <- .C("Gwtdenspt",
                nxy     = as.integer(npts),
@@ -348,7 +349,8 @@ densitypointsEngine <- function(x, sigma, ...,
                y       = as.double(yy),
                rmaxi   = as.double(cutoff),
                weight  = as.double(wtsort),
-               result  = as.double(double(npts)))
+               result  = as.double(double(npts)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[oo] <- zz$result 
       result <- result * const
     } else {
@@ -361,7 +363,8 @@ densitypointsEngine <- function(x, sigma, ...,
                  y       = as.double(yy),
                  rmaxi   = as.double(cutoff),
                  weight  = as.double(wtsort[,j]),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result
       }
       result <- result * const
@@ -371,7 +374,7 @@ densitypointsEngine <- function(x, sigma, ...,
     if(debugging)
       cat('Using standard code.\n')
     npts <- npoints(x)
-    result <- if(k == 1) numeric(npts) else matrix(, npts, k)
+    result <- if(k == 1L) numeric(npts) else matrix(, npts, k)
     # sort into increasing order of x coordinate (required by C code)
     if(sorted) {
       xx <- x$x
@@ -390,9 +393,10 @@ densitypointsEngine <- function(x, sigma, ...,
                  y       = as.double(yy),
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
-      } else if(k == 1) {
+      } else if(k == 1L) {
         wtsort <- if(sorted) weights else weights[oo]
         zz <- .C("wtdenspt",
                  nxy     = as.integer(npts),
@@ -401,7 +405,8 @@ densitypointsEngine <- function(x, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
                  weight  = as.double(wtsort),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
        } else {
         # matrix of weights
@@ -414,7 +419,8 @@ densitypointsEngine <- function(x, sigma, ...,
                    rmaxi   = as.double(cutoff),
                    sig     = as.double(sd),
                    weight  = as.double(wtsort[,j]),
-                   result  = as.double(double(npts)))
+                   result  = as.double(double(npts)),
+                   PACKAGE = "spatstat")
           if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result
         }
       }
@@ -429,9 +435,10 @@ densitypointsEngine <- function(x, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  detsigma = as.double(detSigma),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
-      } else if(k == 1) {
+      } else if(k == 1L) {
         # vector of weights
         wtsort <- if(sorted) weights else weights[oo]
         zz <- .C("awtdenspt",
@@ -442,7 +449,8 @@ densitypointsEngine <- function(x, sigma, ...,
                  detsigma = as.double(detSigma),
                  sinv    = as.double(flatSinv),
                  weight  = as.double(wtsort),
-                 result   = as.double(double(npts)))
+                 result   = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result 
       } else {
         # matrix of weights
@@ -456,7 +464,8 @@ densitypointsEngine <- function(x, sigma, ...,
                    detsigma = as.double(detSigma),
                    sinv    = as.double(flatSinv),
                    weight  = as.double(wtsort[,j]),
-                   result  = as.double(double(npts)))
+                   result  = as.double(double(npts)),
+                   PACKAGE = "spatstat")
           if(sorted) result[,j] <- zz$result else result[oo,j] <- zz$result 
         }
       }
@@ -468,7 +477,7 @@ densitypointsEngine <- function(x, sigma, ...,
     j <- close$j
     d <- close$d
     npts <- npoints(x)
-    result <- if(k == 1) numeric(npts) else matrix(, npts, k)
+    result <- if(k == 1L) numeric(npts) else matrix(, npts, k)
     # evaluate contribution from each close pair (i,j)
     if(isgauss) { 
       if(is.null(varcov)) {
@@ -477,8 +486,8 @@ densitypointsEngine <- function(x, sigma, ...,
         ## anisotropic kernel
         dx <- close$dx
         dy <- close$dy
-        contrib <- const * exp(-(dx * (dx * Sinv[1,1] + dy * Sinv[1,2])
-                                 + dy * (dx * Sinv[2,1] + dy * Sinv[2,2]))/2)
+        contrib <- const * exp(-(dx * (dx * Sinv[1L,1L] + dy * Sinv[1L,2L])
+                                 + dy * (dx * Sinv[2L,1L] + dy * Sinv[2L,2L]))/2)
       }
     } else {
       contrib <- evaluate2Dkernel(kernel, close$dx, close$dy,
@@ -488,7 +497,7 @@ densitypointsEngine <- function(x, sigma, ...,
     ifac <- factor(i, levels=1:npts)
     if(is.null(weights)) {
       result <- tapply(contrib, ifac, sum)
-    } else if(k == 1) {
+    } else if(k == 1L) {
       wcontrib <- contrib * weights[j]
       result <- tapply(wcontrib, ifac, sum)
     } else {
@@ -514,7 +523,7 @@ densitypointsEngine <- function(x, sigma, ...,
 
   # ............. validate .................................
   npts <- npoints(x)
-  if(k == 1) {
+  if(k == 1L) {
     result <- as.numeric(result)
     if(length(result) != npts) 
       stop(paste("Internal error: incorrect number of lambda values",
@@ -566,7 +575,7 @@ resolve.2D.kernel <- function(..., sigma=NULL, varcov=NULL, x, mindist=NULL,
     # interpret the result as either sigma or varcov
     if(!is.numeric(bw))
       stop("bandwidth selector returned a non-numeric result")
-    if(length(bw) %in% c(1,2)) {
+    if(length(bw) %in% c(1L,2L)) {
       sigma <- as.numeric(bw)
       if(!all(sigma > 0)) {
         gripe <- "bandwidth selector returned negative value(s)"
@@ -582,7 +591,7 @@ resolve.2D.kernel <- function(..., sigma=NULL, varcov=NULL, x, mindist=NULL,
   varcov.given <- !is.null(varcov)
   if(sigma.given) {
     stopifnot(is.numeric(sigma))
-    stopifnot(length(sigma) %in% c(1,2))
+    stopifnot(length(sigma) %in% c(1L,2L))
     if(!allow.zero)
       stopifnot(all(sigma > 0))
   }
@@ -590,7 +599,7 @@ resolve.2D.kernel <- function(..., sigma=NULL, varcov=NULL, x, mindist=NULL,
     stopifnot(is.matrix(varcov) && nrow(varcov) == 2 && ncol(varcov)==2 )
   # reconcile
   ngiven <- varcov.given + sigma.given
-  switch(ngiven+1,
+  switch(ngiven+1L,
          {
            # default
            w <- x$window
@@ -635,15 +644,15 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
   cutoff <- 8 * sd
   # validate weights
   if(is.null(weights)) {
-    k <- 1
+    k <- 1L
   } else if(is.matrix(weights) || is.data.frame(weights)) {
     k <- ncol(weights)
     stopifnot(nrow(weights) == npoints(Xdata))
     weights <- as.data.frame(weights)
     weightnames <- colnames(weights)
   } else {
-    k <- 1
-    stopifnot(length(weights) == npoints(Xdata) || length(weights) == 1)
+    k <- 1L
+    stopifnot(length(weights) == npoints(Xdata) || length(weights) == 1L)
   }
   # evaluate edge correction weights at points 
   if(edge) {
@@ -666,9 +675,9 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
       xx <- xedge$x
       yy <- xedge$y
       xprob <-
-        pnorm(xr[2], mean=xx, sd=sigma) - pnorm(xr[1], mean=xx, sd=sigma)
+        pnorm(xr[2L], mean=xx, sd=sigma) - pnorm(xr[1L], mean=xx, sd=sigma)
       yprob <-
-        pnorm(yr[2], mean=yy, sd=sigma) - pnorm(yr[1], mean=yy, sd=sigma)
+        pnorm(yr[2L], mean=yy, sd=sigma) - pnorm(yr[1L], mean=yy, sd=sigma)
       edgeweight <- xprob * yprob
     } else {
       edg <- second.moment.calc(Xdata, sigma=sigma,
@@ -679,7 +688,7 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
       ## Diggle edge correction
       ## edgeweight is attached to each data point
       if(is.null(weights)) {
-        k <- 1
+        k <- 1L
         weights <- 1/edgeweight
       } else {
         weights <- weights/edgeweight
@@ -689,7 +698,7 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
   
   ndata <- npoints(Xdata)
   nquery <- npoints(Xquery)
-  result <- if(k == 1) numeric(nquery) else matrix(, nquery, k)
+  result <- if(k == 1L) numeric(nquery) else matrix(, nquery, k)
   ## coordinates
   xq <- Xquery$x
   yq <- Xquery$y
@@ -716,9 +725,10 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                yd      = as.double(yd),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)))
+               result  = as.double(double(nquery)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
-    } else if(k == 1) {
+    } else if(k == 1L) {
       wtsort <- if(sorted) weights else weights[ood]
       zz <- .C("wtcrdenspt",
                nquery  = as.integer(nquery),
@@ -730,7 +740,8 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                wd      = as.double(wtsort),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)))
+               result  = as.double(double(nquery)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
     } else {
       ## matrix of weights
@@ -746,7 +757,8 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                  wd      = as.double(wtsort[,j]),
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
-                 result  = as.double(double(nquery)))
+                 result  = as.double(double(nquery)),
+                 PACKAGE = "spatstat")
         if(sorted) result[,j] <- zz$result else result[ooq,j] <- zz$result
       }
       colnames(result) <- weightnames
@@ -765,9 +777,10 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                rmaxi   = as.double(cutoff),
                detsigma = as.double(detSigma),
                sinv    = as.double(flatSinv),
-               result  = as.double(double(nquery)))
+               result  = as.double(double(nquery)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
-    } else if(k == 1) {
+    } else if(k == 1L) {
       ## vector of weights
       wtsort <- if(sorted) weights else weights[ood]
       zz <- .C("awtcrdenspt",
@@ -781,7 +794,8 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                rmaxi   = as.double(cutoff),
                detsigma = as.double(detSigma),
                sinv    = as.double(flatSinv),
-               result   = as.double(double(nquery)))
+               result   = as.double(double(nquery)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[ooq] <- zz$result 
     } else {
       ## matrix of weights
@@ -798,7 +812,8 @@ densitycrossEngine <- function(Xdata, Xquery, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  detsigma = as.double(detSigma),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(nquery)))
+                 result  = as.double(double(nquery)),
+                 PACKAGE = "spatstat")
         if(sorted) result[,j] <- zz$result else result[ooq,j] <- zz$result 
       }
       colnames(result) <- weightnames
