@@ -3,29 +3,37 @@
 #
 #   method for 'distfun' for class 'lpp'
 #
-#   $Revision: 1.4 $ $Date: 2012/10/21 02:52:11 $
+#   $Revision: 1.2 $ $Date: 2016/02/11 09:36:11 $
 #
 
 distfun.lpp <- local({
   
-  distfun.lpp <- function(X, ...) {
+  distfun.lpp <- function(X, ..., k=1) {
     stopifnot(inherits(X, "lpp"))
     force(X)
+    force(k)
+    stopifnot(length(k) == 1)
     L <- as.linnet(X)
     f <- function(x, y=NULL, seg=NULL, tp=NULL, ...) {
       # L is part of the environment
       Y <- as.lpp(x=x, y=y, seg=seg, tp=tp, L=L)
-      d <- nncross.lpp(Y, X, what="dist")
+      d <- nncross.lpp(Y, X, what="dist", k=k)
       return(d)
     }
     f <- linfun(f, L)
+    assign("k", k, envir=environment(f))
+    assign("X", X, envir=environment(f))
     attr(f, "explain") <- uitleggen
     return(f)
   }
 
   uitleggen <- function(x, ...) {
-    cat("Distance function for lpp object\n")
-    X <-  get("X", envir=environment(x))
+    splat("Distance function for lpp object")
+    envx <- environment(x)
+    k <-  get("k", envir=envx)
+    if(k != 1L)
+      splat("Yields distance to", ordinal(k), "nearest point")
+    X <-  get("X", envir=envx)
     print(X)
   }
 
