@@ -1,7 +1,7 @@
 #
 #   edges2triangles.R
 #
-#   $Revision: 1.13 $  $Date: 2016/11/22 09:19:19 $
+#   $Revision: 1.14 $  $Date: 2017/06/05 10:31:58 $
 #
 
 edges2triangles <- function(iedge, jedge, nvert=max(iedge, jedge),
@@ -31,14 +31,14 @@ edges2triangles <- function(iedge, jedge, nvert=max(iedge, jedge),
   storage.mode(nvert) <- storage.mode(iedge) <- storage.mode(jedge) <- "integer"
   if(!usefriends) {
     zz <- .Call("triograph",
-                nv=nvert, iedge=iedge, jedge=jedge)
-#                PACKAGE="spatstat")
+                nv=nvert, iedge=iedge, jedge=jedge,
+                PACKAGE="spatstat")
   } else {
     fr <- as.logical(friendly)
     storage.mode(fr) <- "integer"
     zz <- .Call("trioxgraph",
-                nv=nvert, iedge=iedge, jedge=jedge, friendly=fr)
-#                PACKAGE="spatstat")
+                nv=nvert, iedge=iedge, jedge=jedge, friendly=fr,
+                PACKAGE="spatstat")
   }
   mat <- as.matrix(as.data.frame(zz))
   return(mat)
@@ -72,12 +72,14 @@ trianglediameters <- function(iedge, jedge, edgelength, ...,
   storage.mode(edgelength) <- "double"
   if(is.infinite(dmax)) {
     zz <- .Call("triDgraph",
-                nv=nvert, iedge=iedge, jedge=jedge, edgelength=edgelength)
+                nv=nvert, iedge=iedge, jedge=jedge, edgelength=edgelength,
+                PACKAGE = "spatstat")
   } else {
     storage.mode(dmax) <- "double"
     zz <- .Call("triDRgraph",
                 nv=nvert, iedge=iedge, jedge=jedge, edgelength=edgelength,
-                dmax=dmax)
+                dmax=dmax,
+                PACKAGE = "spatstat")
   }    
   df <- as.data.frame(zz)
   colnames(df) <- c("i", "j", "k", "diam")
@@ -112,8 +114,8 @@ edges2vees <- function(iedge, jedge, nvert=max(iedge, jedge),
   vees <- .Call("graphVees",
                 nv = nvert,
                 iedge = iedge,
-                jedge = jedge)
-#                PACKAGE="spatstat")
+                jedge = jedge,
+                PACKAGE="spatstat")
   names(vees) <- c("i", "j", "k")
   vees <- as.data.frame(vees)
   return(vees)
