@@ -1,132 +1,144 @@
 #
 #  hyperframe.R
 #
-# $Revision: 1.49 $  $Date: 2014/03/22 06:12:35 $
+# $Revision: 1.64 $  $Date: 2017/02/07 07:35:32 $
 #
 
-hyperframe <- function(...,
-                       row.names=NULL, check.rows=FALSE, check.names=TRUE,
-                       stringsAsFactors=default.stringsAsFactors()) {
-  aarg <- list(...)
-  nama <- names(aarg)
+hyperframe <- local({
 
-  # number of columns (= variables)
-  nvars <- length(aarg)
+  hyperframe <- function(...,
+                         row.names=NULL, check.rows=FALSE, check.names=TRUE,
+                         stringsAsFactors=default.stringsAsFactors()) {
+    aarg <- list(...)
+    nama <- names(aarg)
+
+    ## number of columns (= variables)
+    nvars <- length(aarg)
   
-  if(nvars == 0) {
-    # zero columns - return
-    result <- list(nvars=0,
-                   ncases=0,
-                   vname=character(0),
-                   vtype=factor(,
-                     levels=c("dfcolumn","hypercolumn","hyperatom")),
-                   vclass=character(0),
-                   df=data.frame(),
-                   hyperatoms=list(),
-                   hypercolumns=list())
+    if(nvars == 0) {
+      ## zero columns - return
+      result <- list(nvars=0,
+                     ncases=0,
+                     vname=character(0),
+                     vtype=factor(,
+                       levels=c("dfcolumn","hypercolumn","hyperatom")),
+                     vclass=character(0),
+                     df=data.frame(),
+                     hyperatoms=list(),
+                     hypercolumns=list())
+      class(result) <- c("hyperframe", class(result))
+      return(result)
+    }
+
+    ## check column names
+    if(is.null(nama))
+      nama <- paste("V", 1:nvars, sep="")
+    else if(any(unnamed <- (nama == ""))) 
+      nama[unnamed] <- paste("V", seq_len(sum(unnamed)), sep="")
+    nama <- make.names(nama, unique=TRUE)
+    names(aarg) <- nama
+  
+    ## Each argument must be either
+    ##    - a vector suitable as a column in a data frame
+    ##    - a list of objects of the same class
+    ##    - a single object of some class
+  
+    dfcolumns    <- sapply(aarg, is.dfcolumn)
+    hypercolumns <- sapply(aarg, is.hypercolumn)
+    hyperatoms   <- !(dfcolumns | hypercolumns)
+
+    ## Determine number of rows (= cases) 
+    columns <- dfcolumns | hypercolumns
+    if(!any(columns)) {
+      ncases <- 1
+    } else {
+      heights <- rep.int(1, nvars)
+      heights[columns] <-  lengths(aarg[columns])
+      u <- unique(heights)
+      if(length(u) > 1) {
+        u <- u[u != 1]
+        if(length(u) > 1)
+          stop(paste("Column lengths are inconsistent:",
+                     paste(u, collapse=",")))
+      }
+      ncases <- u
+      if(ncases > 1 && all(heights[dfcolumns] == 1)) {
+        ## force the data frame to have 'ncases' rows
+        aarg[dfcolumns] <- lapply(aarg[dfcolumns], rep, ncases)
+        heights[dfcolumns] <- ncases
+      }
+      if(any(stubs <- hypercolumns & (heights != ncases))) {
+        ## hypercolumns of height 1 should be hyperatoms
+        aarg[stubs] <- lapply(aarg[stubs], "[[", i=1L)
+        hypercolumns[stubs] <- FALSE
+        hyperatoms[stubs] <- TRUE
+      }
+    }
+  
+    ## Collect the data frame columns into a data frame
+    if(!any(dfcolumns))
+      df <- as.data.frame(matrix(, ncases, 0), row.names=row.names)
+    else {
+      df <- do.call(data.frame,
+                    append(aarg[dfcolumns],
+                           list(row.names=row.names,
+                                check.rows=check.rows,
+                                check.names=check.names,
+                                stringsAsFactors=stringsAsFactors)))
+      names(df) <- nama[dfcolumns]
+    }
+
+    ## Storage type of each variable
+    vtype <- character(nvars)
+    vtype[dfcolumns] <- "dfcolumn"
+    vtype[hypercolumns] <- "hypercolumn"
+    vtype[hyperatoms] <- "hyperatom"
+    vtype=factor(vtype, levels=c("dfcolumn","hypercolumn","hyperatom"))
+
+    ## Class of each variable
+    vclass <- character(nvars)
+    if(any(dfcolumns))
+      vclass[dfcolumns] <- unlist(lapply(as.list(df), class1))
+    if(any(hyperatoms))
+      vclass[hyperatoms] <- unlist(lapply(aarg[hyperatoms], class1))
+    if(any(hypercolumns))
+      vclass[hypercolumns] <- unlist(lapply(aarg[hypercolumns], class1of1))
+    ## Put the result together
+    result <- list(nvars=nvars,
+                   ncases=ncases,
+                   vname=nama,
+                   vtype=vtype,
+                   vclass=vclass,
+                   df=df,
+                   hyperatoms=aarg[hyperatoms],
+                   hypercolumns=aarg[hypercolumns])
+    
     class(result) <- c("hyperframe", class(result))
     return(result)
   }
 
-  # check column names
-  if(is.null(nama))
-    nama <- paste("V", 1:nvars, sep="")
-  else if(any(unnamed <- (nama == ""))) 
-    nama[unnamed] <- paste("V", seq_len(sum(unnamed)), sep="")
-  nama <- make.names(nama, unique=TRUE)
-  names(aarg) <- nama
-  
-  # Each argument must be either
-  #    - a vector suitable as a column in a data frame
-  #    - a list of objects of the same class
-  #    - a single object of some class
-  
   is.dfcolumn <- function(x) {
     is.atomic(x) && (is.vector(x) || is.factor(x))
   }
+  
   is.hypercolumn <- function(x) {
     if(!is.list(x))
       return(FALSE)
+    if(inherits(x, c("listof", "anylist")))
+      return(TRUE)
     if(length(x) <= 1)
       return(TRUE)
-    cla <- class(x[[1]])
-    all(sapply(x, function(xi,cla) { identical(class(xi), cla) }, cla=cla))
-  }
-  dfcolumns    <- sapply(aarg, is.dfcolumn)
-  hypercolumns <- sapply(aarg, is.hypercolumn)
-  hyperatoms   <- !(dfcolumns | hypercolumns)
-
-  # Determine number of rows (= cases) 
-  columns <- dfcolumns | hypercolumns
-  if(!any(columns)) 
-    ncases <- 1
-  else {
-    heights <- rep.int(1, nvars)
-    heights[columns] <-  unlist(lapply(aarg[columns], length))
-    u <- unique(heights)
-    if(length(u) > 1) {
-      u <- u[u != 1]
-      if(length(u) > 1)
-        stop(paste("Column lengths are inconsistent:",
-                   paste(u, collapse=",")))
-    }
-    ncases <- u
-    if(ncases > 1 && all(heights[dfcolumns] == 1)) {
-      # force the data frame to have 'ncases' rows
-      aarg[dfcolumns] <- lapply(aarg[dfcolumns], rep, ncases)
-      heights[dfcolumns] <- ncases
-    }
-    if(any(stubs <- hypercolumns & (heights != ncases))) {
-      ## hypercolumns of height 1 should be hyperatoms
-      aarg[stubs] <- lapply(aarg[stubs], "[[", 1)
-      hypercolumns[stubs] <- FALSE
-      hyperatoms[stubs] <- TRUE
-    }
+    cla <- lapply(x, class)
+    return(length(unique(cla)) == 1)
   }
 
-  # Collect the data frame columns into a data frame
-  if(!any(dfcolumns))
-    df <- as.data.frame(matrix(, ncases, 0), row.names=row.names)
-  else {
-    df <- do.call("data.frame", append(aarg[dfcolumns],
-                                     list(row.names=row.names,
-                                          check.rows=check.rows,
-                                          check.names=check.names,
-                                          stringsAsFactors=stringsAsFactors)))
-    names(df) <- nama[dfcolumns]
-  }
+  class1 <- function(x) { class(x)[1L] }
 
-  # Storage type of each variable
-  vtype <- character(nvars)
-  vtype[dfcolumns] <- "dfcolumn"
-  vtype[hypercolumns] <- "hypercolumn"
-  vtype[hyperatoms] <- "hyperatom"
-  vtype=factor(vtype, levels=c("dfcolumn","hypercolumn","hyperatom"))
-
-  # Class of each variable
-  class1 <- function(x) { class(x)[1] }
-  vclass <- character(nvars)
-  if(any(dfcolumns))
-    vclass[dfcolumns] <- unlist(lapply(as.list(df), class1))
-  if(any(hyperatoms))
-    vclass[hyperatoms] <- unlist(lapply(aarg[hyperatoms], class1))
-  if(any(hypercolumns))
-    vclass[hypercolumns] <- unlist(lapply(aarg[hypercolumns],
-                                          function(x) { class1(x[[1]]) }))
+  class1of1 <- function(x) { class(x[[1L]])[1L] }
   
-  # Put the result together
-  result <- list(nvars=nvars,
-                 ncases=ncases,
-                 vname=nama,
-                 vtype=vtype,
-                 vclass=vclass,
-                 df=df,
-                 hyperatoms=aarg[hyperatoms],
-                 hypercolumns=aarg[hypercolumns])
-  
-    class(result) <- c("hyperframe", class(result))
-    return(result)
-}
+  hyperframe
+})
+
 
 is.hyperframe <- function(x) inherits(x, "hyperframe")
 
@@ -135,13 +147,12 @@ print.hyperframe <- function(x, ...) {
   nvars <- ux$nvars
   ncases <- ux$ncases
   if(nvars * ncases == 0) {
-    cat(paste("NULL hyperframe with", ncases,
-              ngettext(ncases, "row (=case)", "rows (=cases)"),
-              "and", nvars,
-              ngettext(nvars, "column (=variable)", "columns (=variables)"),
-              "\n"))
+    splat("NULL hyperframe with", ncases,
+          ngettext(ncases, "row (=case)", "rows (=cases)"),
+          "and", nvars,
+          ngettext(nvars, "column (=variable)", "columns (=variables)"))
   } else {
-    cat("Hyperframe:\n")
+    if(waxlyrical('gory')) cat("Hyperframe:\n")
     print(as.data.frame(x, discard=FALSE), ...)
   }
   return(invisible(NULL))
@@ -165,9 +176,27 @@ summary.hyperframe <- function(object, ..., brief=FALSE) {
   y$classes <- classes
   # Ordinary data frame columns
   df <- x$df
-  y$dfnames <- names(df)
+  y$dfnames <- colnames(df)
   y$df <- if(length(df) > 0 && !brief) summary(df) else NULL
   y$row.names <- row.names(df)
+  # insert into full array
+  if(!brief && x$nvars > 0) {
+    isobject <- (x$vtype != "dfcolumn")
+    nobj <- sum(isobject)
+    if(nobj == 0) {
+      allcols <- y$df
+    } else {
+      nas <- rep(list(NA_character_), nobj)
+      names(nas) <- x$vname[isobject]
+      allcols <- do.call(cbind, append(list(y$df), nas))
+      acnames <- c(colnames(df), names(nas))
+      allcols <- allcols[ , match(x$vname, acnames), drop=FALSE]
+    }
+    pclass <- padtowidth(paren(classes), colnames(allcols), justify="right")
+    allcols <- as.table(rbind(class=pclass, as.table(allcols)))
+    row.names(allcols) <- rep("", nrow(allcols))
+    y$allcols <- allcols
+  }
   class(y) <- c("summary.hyperframe", class(y))
   return(y)
 }
@@ -175,21 +204,14 @@ summary.hyperframe <- function(object, ..., brief=FALSE) {
 print.summary.hyperframe <- function(x, ...) {
   nvars <- x$nvars
   ncases <- x$ncases
-  cat(paste(if(nvars * ncases == 0) "NULL" else NULL,
-            "hyperframe with", ncases,
-            ngettext(ncases, "row (=case)", "rows (=cases)"),
-            "and", nvars,
-            ngettext(nvars, "column (=variable)", "columns (=variables)"),
-            "\n"))
+  splat(if(nvars * ncases == 0) "NULL hyperframe" else "hyperframe",
+        "with", ncases,
+        ngettext(ncases, "row", "rows"),
+        "and", nvars,
+        ngettext(nvars, "column", "columns"))
   if(nvars == 0)
     return(invisible(NULL))
-  # Variable names and types
-  print(x$typeframe)
-  # Ordinary data frame columns
-  if(!is.null(x$df)) {
-    cat("Summary of data frame columns:\n")
-    print(x$df, ...)
-  }
+  print(if(any(x$storage == "dfcolumn")) x$allcols else noquote(x$classes))
   return(invisible(NULL))
 }
 
@@ -201,8 +223,11 @@ names.hyperframe <- function(x) { unclass(x)$vname }
   value <- make.names(value)
   if(length(value) != x$nvars)
     stop("Incorrect length for vector of names")
+  vtype <- x$vtype
+  names(x$df)           <- value[vtype == "dfcolumn"]
+  names(x$hyperatoms)   <- value[vtype == "hyperatom"]
+  names(x$hypercolumns) <- value[vtype == "hypercolumn"]
   x$vname <- value
-  names(x$df) <- value[x$vtype == "dfcolumn"]
   class(x) <- c("hyperframe", class(x))
   return(x)
 }
@@ -233,7 +258,7 @@ as.hyperframe.hyperframe <- function(x, ...) {
 
 as.hyperframe.data.frame <- function(x, ..., stringsAsFactors=FALSE) {
   xlist <- if(missing(x)) NULL else as.list(x)
-  do.call("hyperframe",
+  do.call(hyperframe,
           resolve.defaults(
                            xlist,
                            list(...),
@@ -242,13 +267,14 @@ as.hyperframe.data.frame <- function(x, ..., stringsAsFactors=FALSE) {
                            .StripNull=TRUE))
 }
 
+as.hyperframe.anylist <- 
 as.hyperframe.listof <- function(x, ...) {
   if(!missing(x)) {
     xname <- sensiblevarname(short.deparse(substitute(x)), "x")
     xlist <- list(x)
     names(xlist) <- xname
   } else xlist <- NULL
-  do.call("hyperframe",
+  do.call(hyperframe,
           resolve.defaults(
                            xlist,
                            list(...),
@@ -281,22 +307,37 @@ as.data.frame.hyperframe <- function(x, row.names = NULL,
   } else {
     lx <- as.list(x)
     nrows <- ux$ncases
-    vclassstring <- paste("(", vclass, ")", sep="")
+    vclassstring <- paren(vclass)
     if(any(!dfcol)) 
       lx[!dfcol] <- lapply(as.list(vclassstring[!dfcol]),
-                           function(x,n) { rep.int(x,n)}, n=nrows)
-    df <- do.call("data.frame", append(lx, list(row.names=row.names)))
+                           rep.int, times=nrows)
+    df <- do.call(data.frame, append(lx, list(row.names=row.names)))
+    colnames(df) <- ux$vname
   }
   return(df)
 }
 
 as.list.hyperframe <- function(x, ...) {
   ux <- unclass(x)
-  nama <- ux$vname
-  names(nama) <- nama
-  out <- lapply(nama, function(nam, x) { x[, nam, drop=TRUE] }, x=x)
-  if(ux$ncases == 1)
-    out <- lapply(lapply(out, listof), "names<-", value=row.names(x))
+  out <- vector(mode="list", length=ux$nvars)
+  vtype <- ux$vtype
+  df <- ux$df
+  if(any(dfcol <- (vtype == "dfcolumn")))
+    out[dfcol] <- as.list(df)
+  if(any(hypcol <- (vtype == "hypercolumn"))) {
+    hc <- lapply(ux$hypercolumns, as.solist, demote=TRUE)
+    out[hypcol] <- hc
+  }
+  if(any(hatom <- (vtype == "hyperatom"))) {
+    ha <- ux$hyperatoms
+    names(ha) <- NULL
+    hacol <- lapply(ha, list)
+    hacol <- lapply(hacol, rep.int, times=ux$ncases)
+    hacol <- lapply(hacol, as.solist, demote=TRUE)
+    out[hatom] <- hacol
+  }
+  out <- lapply(out, "names<-", value=row.names(df))
+  names(out) <- names(x)
   return(out)
 }
 
@@ -321,7 +362,7 @@ with.hyperframe <- function(data, expr, ..., simplify=TRUE, ee=NULL,
   out <- vector(mode="list", length=n)
   datalist <- as.list(data)
   for(i in 1:n) {
-    rowi <- lapply(datalist, "[[", i)  # ensures the result is always a list
+    rowi <- lapply(datalist, "[[", i=i)  # ensures the result is always a list
     outi <- eval(ee, rowi, enclos)
     if(!is.null(outi))
       out[[i]] <- outi
@@ -330,7 +371,7 @@ with.hyperframe <- function(data, expr, ..., simplify=TRUE, ee=NULL,
   if(simplify && all(unlist(lapply(out, is.vector)))) {
     # if all results are atomic vectors of equal length,
     # return a matrix or vector.
-    lenfs <- unlist(lapply(out, length))
+    lenfs <- lengths(out)
     if(all(unlist(lapply(out, is.atomic))) &&
             length(unique(lenfs)) == 1) {
       out <- t(as.matrix(as.data.frame(out)))
@@ -368,7 +409,7 @@ cbind.hyperframe <- function(...) {
       columns <- append(columns, nextcolumn)
     }
   }
-  result <- do.call("hyperframe", columns)
+  result <- do.call(hyperframe, columns)
   return(result)
 }
 
@@ -381,7 +422,7 @@ rbind.hyperframe <- function(...) {
   #
   nargh <- length(argh)
   if(nargh == 1)
-    return(argh[[1]])
+    return(argh[[1L]])
   # check for compatibility of dimensions & names
   dfs <- lapply(argh, as.data.frame, discard=FALSE)
   dfall <- do.call(rbind, dfs)
@@ -399,12 +440,10 @@ rbind.hyperframe <- function(...) {
       rslt[[k]] <- dfall[,k]
     } else {
       # hypercolumns or hyperatoms: extract them
-      hdata <- lapply(argh,
-                      function(x,nama) { x[, nama, drop=FALSE] },
-                      nama=nama)
+      hdata <- lapply(argh, "[", j=nama, drop=FALSE)
       hdata <- lapply(lapply(hdata, as.list), getElement, name=nama)
       # append them
-      hh <- hdata[[1]]
+      hh <- hdata[[1L]]
       for(j in 2:nargh) {
         hh <- append(hh, hdata[[j]])
       }
@@ -420,18 +459,19 @@ rbind.hyperframe <- function(...) {
 plot.hyperframe <-
   function(x, e, ..., main, arrange=TRUE,
            nrows=NULL, ncols=NULL,
-           parargs=list(mar=c(1,1,3,1) * marsize),
-           marsize=0.1) {
+           parargs=list(mar=mar * marsize),
+           marsize=1, mar=c(1,1,3,1)) {
   xname <- short.deparse(substitute(x))
   main <- if(!missing(main)) main else xname
-
+  mar <- rep(mar, 4)[1:4]
+  
   if(missing(e)) {
     # default: plot first column that contains objects
     ok <- (summary(x)$storage %in% c("hypercolumn", "hyperatom"))
     if(any(ok)) {
       j <- min(which(ok))
-      x <- x[,j, drop=TRUE]
-      x <- as.listof(x)
+      x <- x[,j, drop=TRUE, strip=FALSE]
+      x <- as.solist(x, demote=TRUE)
       plot(x, ..., main=main, arrange=arrange, nrows=nrows, ncols=ncols)
       return(invisible(NULL))
     } else {
@@ -480,7 +520,7 @@ plot.hyperframe <-
     # Increment existing panel numbers
     # New panel 1 is the banner
     panels <- (mat > 0)
-    mat[panels] <- mat[panels] + 1
+    mat[panels] <- mat[panels] + 1L
     mat <- rbind(rep.int(1,ncols), mat)
     heights <- c(0.1 * (1 + nlines), heights)
   }
@@ -495,7 +535,7 @@ plot.hyperframe <-
     text(0,0,main, cex=cex)
   }
   # plot panels
-  npa <- do.call("par", parargs)
+  npa <- do.call(par, parargs)
   if(!banner) opa <- npa
   with(x, ee=ee)
   # revert
@@ -510,12 +550,12 @@ str.hyperframe <- function(object, ...) {
   x <- unclass(object)
   argh <- resolve.defaults(list(...), list(nest.lev=0, indent.str="  .."))
   cat(paste("'hyperframe':\t",
-            d[1], ngettext(d[1], "row", "rows"),
+            d[1L], ngettext(d[1L], "row", "rows"),
             "and",
-            d[2], ngettext(d[2], "column", "columns"),
+            d[2L], ngettext(d[2L], "column", "columns"),
             "\n"))
-  nr <- d[1]
-  nc <- d[2]
+  nr <- d[1L]
+  nc <- d[2L]
   if(nc > 0) {
     vname <- x$vname
     vclass <- x$vclass
@@ -542,3 +582,59 @@ str.hyperframe <- function(object, ...) {
   }
   return(invisible(NULL))
 }
+
+subset.hyperframe <- function(x, subset, select, ...) {
+  stopifnot(is.hyperframe(x))
+  r <- if(missing(subset)) {
+    rep_len(TRUE, nrow(x))
+  } else {
+      r <- eval(substitute(
+        with(x, e, enclos=parent.frame()),
+        list(e=substitute(subset))))
+    if (!is.logical(r)) 
+      stop("'subset' must be logical")
+    r & !is.na(r)
+  }
+  vars <- if(missing(select)) { 
+    TRUE
+  } else {
+    nl <- as.list(seq_len(ncol(x)))
+    names(nl) <- names(x)
+    eval(substitute(select), nl, parent.frame())
+  }
+  nama <- names(x)
+  names(nama) <- nama
+  vars <- nama[vars]
+  z <- x[i=r, j=vars, ...]
+  return(z)
+}
+
+head.hyperframe <- function (x, n = 6L, ...) {
+  stopifnot(length(n) == 1L)
+  n <- if(n < 0L) max(nrow(x) + n, 0L) else min(n, nrow(x))
+  x[seq_len(n), , drop = FALSE]
+}
+
+tail.hyperframe <- function(x, n = 6L, ...) {
+  stopifnot(length(n) == 1L)
+  nrx <- nrow(x)
+  n <- if(n < 0L) max(nrx + n, 0L) else min(n, nrx)
+  sel <- seq.int(to = nrx, length.out = n)
+  x[sel, , drop = FALSE]
+}
+
+edit.hyperframe <- function(name, ...) {
+  x <- name
+  isdf <- unclass(x)$vtype == "dfcolumn"
+  if(!any(isdf)) {
+    warning("No columns of editable data", call.=FALSE)
+    return(x)
+  }
+  y <- x[,isdf]
+  ynew <- edit(as.data.frame(y), ...)
+  xnew <- x
+  for(na in names(ynew)) xnew[,na] <- ynew[,na]
+  losenames <- setdiff(names(y), names(ynew))
+  for(na in losenames) xnew[,na] <- NULL
+  return(xnew)
+}
