@@ -3,7 +3,7 @@
 #    
 #    Linear networks
 #
-#    $Revision: 1.60 $    $Date: 2016/12/21 08:56:23 $
+#    $Revision: 1.63 $    $Date: 2017/09/11 16:36:37 $
 #
 # An object of class 'linnet' defines a linear network.
 # It includes the following components
@@ -54,23 +54,23 @@ linnet <- function(vertices, m, edges, sparse=FALSE, warn=TRUE) {
     sparse <- !is.matrix(m)
     ## determine 'from' and 'to' vectors
     ij <- which(m, arr.ind=TRUE)
-    ij <- ij[ ij[,1] < ij[,2], , drop=FALSE]
-    from <- ij[,1]
-    to   <- ij[,2]
+    ij <- ij[ ij[,1L] < ij[,2L], , drop=FALSE]
+    from <- ij[,1L]
+    to   <- ij[,2L]
   } else {
     # check (from, to) pairs
     stopifnot(is.matrix(edges) && ncol(edges) == 2)
     if(any((edges %% 1) != 0))
       stop("Entries of edges list should be integers")
-    if(any(self <- (edges[,1] == edges[,2]))) {
+    if(any(self <- (edges[,1L] == edges[,2L]))) {
       warning("edge list should not join a vertex to itself; ignored")
       edges <- edges[!self, , drop=FALSE]
     }
     np <- npoints(vertices)
     if(any(edges > np))
       stop("index out-of-bounds in edges list")
-    from <- edges[,1]
-    to   <- edges[,2]
+    from <- edges[,1L]
+    to   <- edges[,2L]
     # convert to adjacency matrix
     if(!sparse) {
       m <- matrix(FALSE, np, np)
@@ -115,6 +115,8 @@ print.linnet <- function(x, ...) {
         nv, ngettext(nv, "vertex", "vertices"), 
         "and",
         nl, ngettext(nl, "line", "lines"))
+  if(!is.null(br <- x$boundingradius) && is.infinite(br))
+     splat("[Network is not connected]")
   print(as.owin(x), prefix="Enclosing window: ")
   return(invisible(NULL))
 }
@@ -128,6 +130,7 @@ summary.linnet <- function(object, ...) {
                  unitinfo = summary(unitname(object)),
                  totlength = sum(lengths.psp(object$lines)),
                  maxdegree = max(deg),
+		 ncomponents = length(levels(connected(object, what="labels"))),
                  win = as.owin(object),
                  sparse = sparse)
   if(!sparse) {
@@ -149,9 +152,16 @@ print.summary.linnet <- function(x, ...) {
     splat("Total length", signif(totlength, dig), 
           unitinfo$plural, unitinfo$explain)
     splat("Maximum vertex degree:", maxdegree)
-    if(sparse) splat("[Sparse matrix representation]") else {
-      splat("Diameter:", signif(diam, dig), unitinfo$plural)
-      splat("Bounding radius:", signif(boundrad, dig), unitinfo$plural)
+    if(sparse) splat("[Sparse matrix representation]") else
+    	       splat("[Non-sparse matrix representation]")
+    if(ncomponents > 1) {
+      splat("Network is disconnected: ", ncomponents, "connected components")
+    } else {
+      splat("Network is connected")
+      if(!sparse) {
+        splat("Diameter:", signif(diam, dig), unitinfo$plural)
+        splat("Bounding radius:", signif(boundrad, dig), unitinfo$plural)
+      }
     }
     if(!is.null(x$toler))
       splat("Numerical tolerance:", signif(x$toler, dig), unitinfo$plural)
@@ -243,8 +253,8 @@ as.linnet.linnet <- function(X, ..., sparse) {
     # convert adjacency to matrix
     X$m <- m <- as.matrix(X$m)
     edges <- which(m, arr.ind=TRUE)
-    from <- edges[,1]
-    to   <- edges[,2]
+    from <- edges[,1L]
+    to   <- edges[,2L]
     # compute distances to one-step neighbours
     n <- nrow(m)
     d <- matrix(Inf, n, n)
@@ -363,8 +373,8 @@ boundingradius.linnet <- function(x, ...) {
             nv = as.integer(nv), 
             dpath = as.double(dpath), 
             huge = as.double(huge), 
-            result = as.double(numeric(1))
-            )
+            result = as.double(numeric(1)),
+            PACKAGE = "spatstat")
     return(z$result)
   }
   sA <- sB <- matrix(Inf, nseg, nseg)
@@ -488,8 +498,15 @@ rescale.linnet <- function(X, s, unitname) {
   if(!is.owin(i))
     stop("In [.linnet: the index i should be a window", call.=FALSE)
   w <- i
+  wp <- as.polygonal(w)
+  if(is.mask(w)) {
+    ## protect against pixellation artefacts
+    pixel <- owin(w$xstep * c(-1,1)/2, w$ystep * c(-1,1)/2)
+    wp <- MinkowskiSum(wp, pixel)
+    wp <- intersect.owin(wp, Frame(w))
+  }
   ## Find vertices that lie inside window
-  vertinside <- inside.owin(x$vertices, w=w)
+  vertinside <- inside.owin(x$vertices, w=wp)
   from <- x$from
   to   <- x$to
   if(snip) {
@@ -499,13 +516,13 @@ rescale.linnet <- function(X, s, unitname) {
     ## extract relevant subset of network graph
     x <- thinNetwork(x, retainedges=okedge)
     ## Now add vertices at crossing points with boundary of 'w'
-    b <- crossing.psp(as.psp(x), edges(w))
+    b <- crossing.psp(as.psp(x), edges(wp))
     x <- insertVertices(x, unique(b))
     boundarypoints <- attr(x, "id")
     ## update data
     from <- x$from
     to   <- x$to
-    vertinside <- inside.owin(x$vertices, w=w)
+    vertinside <- inside.owin(x$vertices, w=wp)
     vertinside[boundarypoints] <- TRUE
   }
   ## find segments whose endpoints BOTH lie in 'w'
@@ -545,7 +562,7 @@ connected.linnet <- function(X, ..., what=c("labels", "components")) {
   what <- match.arg(what)
   nv <- npoints(vertices(X))
   ie <- X$from - 1L
-  je   <- X$to   - 1L
+  je   <- X$to - 1L
   ne <- length(ie)
   zz <- .C("cocoGraph",
            nv = as.integer(nv),
@@ -553,7 +570,8 @@ connected.linnet <- function(X, ..., what=c("labels", "components")) {
            ie = as.integer(ie),
            je = as.integer(je),
            label = as.integer(integer(nv)), 
-           status = as.integer(integer(1)))
+           status = as.integer(integer(1L)),
+           PACKAGE = "spatstat")
   if (zz$status != 0) 
     stop("Internal error: connected.linnet did not converge")
   lab <- zz$label + 1L
@@ -568,6 +586,14 @@ connected.linnet <- function(X, ..., what=c("labels", "components")) {
   return(nets)
 }
 
+is.connected.linnet <- function(X, ...) {
+  if(!is.null(dpath <- X$dpath))
+    return(all(is.finite(dpath)))
+  lab <- connected(X, what="labels")
+  npieces <- length(levels(lab))
+  return(npieces == 1)
+}
+
 crossing.linnet <- function(X, Y) {
   X <- as.linnet(X)
   if(!inherits(Y, c("linnet", "infline", "psp")))
