@@ -3,7 +3,7 @@
 #
 # code to plot transformation diagnostic
 #
-#   $Revision: 1.2 $  $Date: 2014/03/20 08:56:36 $
+#   $Revision: 1.9 $  $Date: 2016/12/30 01:44:07 $
 #
 
 parres <- function(model, covariate, ...,
@@ -18,6 +18,9 @@ parres <- function(model, covariate, ...,
     covname <- sensiblevarname(deparse(substitute(covariate)), "X")
   callstring <- paste(deparse(sys.call()), collapse = "")
 
+  if(is.marked(model))
+    stop("Sorry, this is not yet implemented for marked models")
+      
   if(!is.null(subregion)) 
     stopifnot(is.owin(subregion))
   
@@ -35,7 +38,7 @@ parres <- function(model, covariate, ...,
   
   # extract spatial locations
   Q <- quad.ppm(model)
-  datapoints <- Q$data
+#  datapoints <- Q$data
   quadpoints <- union.quad(Q)
   Z <- is.data(Q)
   wts <- w.quad(Q)
@@ -163,14 +166,14 @@ parres <- function(model, covariate, ...,
       # check whether covariate is separable
       if(any(conflict <- dmat[relevant, othercov, drop=FALSE])) {
         ## identify entangled covariates
-        entangled <- colnames(conflict)[apply(conflict, 2, any)]
+        entangled <- colnames(conflict)[matcolany(conflict)]
         ## not problematic if constant
         ok <- unlist(isconstant[entangled])
         conflict[ , ok] <- FALSE
         ## re-test
         if(any(conflict)) {
-          conflictterms <- apply(conflict, 1, any)
-          conflictcovs  <- apply(conflict, 2, any)
+          conflictterms <- matrowany(conflict)
+          conflictcovs  <- matcolany(conflict)
           stop(paste("The covariate", sQuote(covname),
                      "cannot be separated from the",
                      ngettext(sum(conflictcovs), "covariate", "covariates"),
@@ -235,14 +238,14 @@ parres <- function(model, covariate, ...,
       # check whether covariate is separable
       if(any(conflict<- offmat[relevant, othercov, drop=FALSE])) {
         ## identify entangled covariates
-        entangled <- colnames(conflict)[apply(conflict, 2, any)]
+        entangled <- colnames(conflict)[matcolany(conflict)]
         ## not problematic if constant
         ok <- unlist(isconstant[entangled])
         conflict[ , ok] <- FALSE
         ## re-test
         if(any(conflict)) {
-          conflictterms <- apply(conflict, 1, any)
-          conflictcovs  <- apply(conflict, 2, any)
+          conflictterms <- matrowany(conflict)
+          conflictcovs  <- matcolany(conflict)
           stop(paste("The covariate", sQuote(covname),
                      "cannot be separated from the",
                      ngettext(sum(conflictcovs), "covariate", "covariates"),
@@ -268,7 +271,7 @@ parres <- function(model, covariate, ...,
                    offnames[nbg],
                    "not found in model frame"))
       effex <- mf[, offnames, drop=FALSE]
-      effect <- effect + apply(effex, 1, sum)
+      effect <- effect + rowSums(effex)
       #
       # construct the corresponding function
       gd <- getglmdata(model)
@@ -290,7 +293,7 @@ parres <- function(model, covariate, ...,
         if(!all(d$offnames %in% colnames(mf))) 
           stop("Internal error: mismatch in term names in effectFun")
         moff <- mf[, d$offnames, drop=FALSE]
-        y <- apply(moff, 1, sum)
+        y <- rowSums(moff)
         return(y)
       }
     }
@@ -429,7 +432,7 @@ parres <- function(model, covariate, ...,
   ####################################################
   # Determine recommended plot range
 
-  xr <- range(x[Z], finite=TRUE)
+  xr <- range(as.vector(x[Z]), finite=TRUE)
   alim <- xr + 0.1 * diff(xr) * c(-1,1)
   alim <- intersect.ranges(alim, c(from, to))
   
@@ -583,7 +586,7 @@ print.parres <- function(x, ...) {
 
 plot.parres <- function(x, ...) {
   xname <- deparse(substitute(x))
-  do.call("plot.fv", resolve.defaults(list(x), list(...),
+  do.call(plot.fv, resolve.defaults(list(x), list(...),
                                       list(main=xname, shade=c("hi", "lo"))))
 }
 
