@@ -1,41 +1,121 @@
 #
 #   plot.im.R
 #
-#  $Revision: 1.84 $   $Date: 2014/05/08 05:23:48 $
+#  $Revision: 1.120 $   $Date: 2017/10/03 02:01:01 $
 #
 #  Plotting code for pixel images
 #
 #  plot.im
 #  image.im
 #  contour.im
-#  persp.im
 #
 ###########################################################################
 
 plot.im <- local({
 
-  # recognised additional arguments to image.default() and axis()
+  ## auxiliary functions
 
-  plotparams <- imageparams <- c("main", "asp", "sub", "axes", "ann",
-                   "cex", "font", 
-                   "cex.axis", "cex.lab", "cex.main", "cex.sub",
-                   "col.axis", "col.lab", "col.main", "col.sub",
-                   "font.axis", "font.lab", "font.main", "font.sub")
-
-  axisparams <- c("cex", 
-                  "cex.axis", "cex.lab",
-                  "col.axis", "col.lab",
-                  "font.axis", "font.lab",
-                  "mgp", "xaxp", "yaxp", "tck", "tcl", "las", "fg", "xpd")
-  
-  # auxiliary functions
-
-  image.doit <- function(..., extrargs=imageparams) {
-    do.call.matched("image.default",
-                    resolve.defaults(...),
-                    extrargs=extrargs)
+  image.doit <- function(imagedata, ...,
+                         extrargs=graphicsPars("image"), W,
+                         workaround=FALSE) {
+    aarg <- resolve.defaults(...)
+    add      <- resolve.1.default(list(add=FALSE),     aarg)
+    show.all <- resolve.1.default(list(show.all=!add), aarg)
+    addcontour <- resolve.1.default(list(addcontour=FALSE), aarg)
+    args.contour <- resolve.1.default(list(args.contour=list()), aarg)
+    if(add && show.all) {
+      ## set up the window space *with* the main title
+      ## using the same code as plot.owin, for consistency
+      do.call.matched(plot.owin,
+                      resolve.defaults(list(x=W, type="n"), aarg), 
+                      extrargs=graphicsPars("owin"))
+    }
+    if(workaround && identical(aarg$useRaster, TRUE)) {
+      #' workaround for bug 16035
+      #' detect reversed coordinates
+      usr <- par('usr')
+      xrev <- (diff(usr[1:2]) < 0) 
+      yrev <- (diff(usr[3:4]) < 0)
+      if(xrev || yrev) {
+        #' flip matrix of pixel values, because the device driver does not
+        z <- imagedata$z
+        d <- dim(z) # z is in the orientation expected for image.default
+        if(xrev) z <- z[d[1]:1,       , drop=FALSE]
+        if(yrev) z <- z[      , d[2]:1, drop=FALSE]
+        imagedata$z <- z
+      }
+    }
+    extrargs <- setdiff(extrargs, c("claim.title.space", "box"))
+    z <- do.call.matched(image.default,
+                         append(imagedata, aarg),
+                         extrargs=extrargs)
+    if(addcontour)
+      do.call(do.contour,
+              resolve.defaults(imagedata,
+                               list(add=TRUE),
+                               args.contour,
+                               list(col=par('fg')),
+                               aarg,
+                               .StripNull=TRUE))
+    return(z)
   }
 
+  do.contour <- function(x, y, z, ..., drawlabels=TRUE) {
+    nx <- length(x)
+    ny <- length(y)
+    nz <- dim(z)
+    if(nx > nz[1]) {
+      if(nz[1] == 1) {
+        z <- rbind(z, z)
+        nz <- dim(z)
+        drawlabels <- FALSE
+      } else {
+        x <- (x[-1] + x[-nx])/2
+        nx <- nx-1
+      }
+    }
+    if(ny > nz[2]) {
+      if(nz[2] == 1) {
+        z <- cbind(z, z)
+        nz <- dim(z)
+        drawlabels <- FALSE
+      } else {
+        y <- (y[-1] + y[-ny])/2
+        ny <- ny-1
+      }
+    }
+    do.call.matched(contour.default,
+                    list(x=x, y=y, z=z, ..., drawlabels=drawlabels))
+  }
+                 
+  do.box.etc <- function(bb, add, argh)
+    do.call(box.etc, append(list(bb=bb, add=add), argh))
+  
+  box.etc <- function(bb, ..., add=FALSE, axes=FALSE, box=!add) {
+    # axes for image
+    xr <- bb$xrange
+    yr <- bb$yrange
+    if(box)
+      rect(xr[1], yr[1], xr[2], yr[2])
+    if(axes) {
+      px <- pretty(xr)
+      py <- pretty(yr)
+      do.call.plotfun(graphics::axis,
+                      resolve.defaults(
+                                       list(side=1, at=px), 
+                                       list(...),
+                                       list(pos=yr[1])),
+                      extrargs=graphicsPars("axis"))
+      do.call.plotfun(graphics::axis,
+                      resolve.defaults(
+                                       list(side=2, at=py), 
+                                       list(...),
+                                       list(pos=xr[1])),
+                      extrargs=graphicsPars("axis"))
+    }
+  }
+  
+  
   clamp <- function(x, v, tol=0.02 * diff(v)) {
     ok <- (x >= v[1] - tol) & (x <= v[2] + tol)
     x[ok]
@@ -52,6 +132,12 @@ plot.im <- local({
     y[ok] <- log10(x[ok])
     return(y)
   }
+
+  Ticks <- function(usr, log=FALSE, nint=NULL, ...) {
+    #' same as axisTicks but accepts nint=NULL as if it were missing
+    if(!is.null(nint)) return(axisTicks(usr=usr, log=log, nint=nint, ...))
+    return(axisTicks(usr=usr, log=log, ...))
+  }
   
   # main function
   PlotIm <- function(x, ...,
@@ -62,26 +148,38 @@ plot.im <- local({
                      ribside=c("right", "left", "bottom", "top"),
                      ribsep=0.15, ribwid=0.05, ribn=1024,
                      ribscale=1, ribargs=list(), colargs=list(),
+                     useRaster=NULL, workaround=FALSE,
                      do.plot=TRUE) {
     if(missing(main)) main <- short.deparse(substitute(x))
     verifyclass(x, "im")
+    if(x$type == "complex") {
+      cl <- match.call()
+      cl$x <- solist(Re=Re(x), Im=Im(x), Mod=Mod(x), Arg=Arg(x))
+      cl[[1]] <- as.name('plot')
+      cl$main <- main
+      out <- eval(cl, parent.frame())
+      return(invisible(out))
+    }
     ribside <- match.arg(ribside)
     col.given <- !is.null(col)
     dotargs <- list(...)
 
     stopifnot(is.list(ribargs))
-
+    user.ticks <- ribargs$at
+    user.nint <- ribargs$nint
+    
     if(!is.null(clipwin)) {
       x <- x[as.rectangle(clipwin)]
       if(!is.rectangle(clipwin)) x <- x[clipwin, drop=FALSE]
     }
-    
+
     zlim <- dotargs$zlim
 
     x <- repair.image.xycoords(x)
 
     xtype <- x$type
-
+    xbox <- as.rectangle(x)
+    
     do.log <- identical(log, TRUE)
     if(do.log && !(x$type %in% c("real", "integer")))
       stop(paste("Log transform is undefined for an image of type",
@@ -172,52 +270,33 @@ plot.im <- local({
     }
     
     imagebreaks <- NULL
-    ribbonvalues <- ribbonbreaks <- NULL
+#    ribbonvalues <- ribbonbreaks <- NULL
+    ribbonvalues <- NULL
 
-    ## predetermined colour map?
-    ## (i.e. mapping from values to colours)
-    colmap <- if(valuesAreColours) colourmap(col=col, inputs=col) else
-              if(inherits(col, "colourmap")) col else NULL
-
-    ## colour map determined by a function?
-    colfun <- NULL
-    if(is.null(colmap) && (!col.given || is.function(col))) {
-      colfun <- if(is.function(col)) col else spatstat.options("image.colfun")
-      colargnames <- names(formals(colfun))
-      ## colfun could be a function(n) that generates n colours,
-      ## or a function(...) that generates a colour map.
-      ## If the latter, convert it to a 'colmap'
-      if("range" %in% colargnames && xtype %in% c("real", "integer")) {
-        # continuous 
-        vrange <- range(range(x, finite=TRUE), zlim)
-        cvals <- try(do.call.matched(colfun,
-                                     append(list(range=vrange), colargs)),
-                     silent=TRUE)
-        if(!inherits(cvals, "try-error")) {
-          colmap <- if(inherits(cvals, "colourmap")) cvals else
-            if(is.character(cvals)) colourmap(cvals, range=vrange) else NULL
-        }
-        if(!is.null(colmap)) colfun <- NULL
-      } else if("inputs" %in% colargnames && xtype != "real") {
-        # discrete
-        vpossible <- switch(xtype,
-                            logical = c(FALSE, TRUE),
-                            factor = levels(x),
-                            unique(as.matrix(x)))
-        if(!is.null(vpossible) && length(vpossible) < 256) {
-          cvals <- try(do.call.matched(colfun,
-                                       append(list(inputs=vpossible), colargs)),
-                       silent=TRUE)
-          if(!inherits(cvals, "try-error")) {
-            colmap <- if(inherits(cvals, "colourmap")) cvals else
-            if(is.character(cvals)) colourmap(cvals, inputs=vpossible) else NULL
-          }
-        if(!is.null(colmap)) colfun <- NULL
-        }
-      } else if(colargnames[1] != "n")
-        stop("Unrecognised syntax for colour function")
+    ## NOW DETERMINE THE COLOUR MAP
+    colfun <- colmap <- NULL
+    if(valuesAreColours) {
+      ## pixel values are colours; set of colours was determined earlier
+      colmap <- colourmap(col=col, inputs=col)
+    } else if(!col.given) {
+      ## no colour information given: use default
+      colfun <- spatstat.options("image.colfun")
+    } else if(inherits(col, "colourmap")) {
+      ## Bob's your uncle
+      colmap <- col
+    } else if(is.function(col)) {
+      ## Some kind of function determining a colour map
+      if(names(formals(col))[1] == "n") {
+        ## function(n) -> colour values
+        colfun <- col
+      } else {
+        ## colour map determined by a rule (e.g. 'beachcolours')
+        colmap <- invokeColourmapRule(col, x, zlim=zlim, colargs=colargs)
+        if(is.null(colmap))
+          stop("Unrecognised syntax for colour function")
+      }
     }
-       
+
     switch(xtype,
            real    = {
              vrange <- range(x, finite=TRUE)
@@ -243,7 +322,9 @@ plot.im <- local({
                                    length.out=ribn)
                ribbonrange <- vrange
                nominalrange <- Log(ribscale * Exp(ribbonrange))
-               nominalmarks <- axisTicks(nominalrange, log=do.log)
+               nominalmarks <- user.ticks %orifnull% Ticks(nominalrange,
+                                                           log=do.log,
+                                                           nint=user.nint)
                ribbonticks <- Log(nominalmarks/ribscale)
                ribbonlabels <- paste(nominalmarks)
              }
@@ -258,8 +339,14 @@ plot.im <- local({
              trivial <- (nvalues < 2)
              if(!trivial){
                nominalrange <- Log(ribscale * Exp(vrange))
-               nominalmarks <- axisTicks(nominalrange, log=do.log)
-               nominalmarks <- nominalmarks[nominalmarks %% 1 == 0]
+               if(!is.null(user.ticks)) {
+                 nominalmarks <- user.ticks
+               } else {
+                 nominalmarks <- Ticks(nominalrange,
+                                       log=do.log,
+                                       nint = user.nint)
+                 nominalmarks <- nominalmarks[nominalmarks %% 1 == 0]
+               }
                ribbonticks <- Log(nominalmarks/ribscale)
                ribbonlabels <- paste(nominalmarks)
                if(!do.log && identical(all.equal(ribbonticks,
@@ -296,8 +383,8 @@ plot.im <- local({
              imagebreaks <- c(-0.5, 0.5, 1.5)
              ribbonvalues <- c(0,1)
              ribbonrange <- range(imagebreaks)
-             ribbonbreaks <- imagebreaks
-             ribbonticks <- ribbonvalues
+#             ribbonbreaks <- imagebreaks
+             ribbonticks <- user.ticks %orifnull% ribbonvalues
              ribbonlabels <- c("FALSE", "TRUE")
              if(!is.null(colmap)) 
                col <- colmap(c(FALSE,TRUE))
@@ -312,8 +399,8 @@ plot.im <- local({
              imagebreaks <- c(intlev - 0.5, max(intlev) + 0.5)
              ribbonvalues <- intlev
              ribbonrange <- range(imagebreaks)
-             ribbonbreaks <- imagebreaks
-             ribbonticks <- ribbonvalues
+#             ribbonbreaks <- imagebreaks
+             ribbonticks <- user.ticks %orifnull% ribbonvalues
              ribbonlabels <- paste(lev)
              vrange <- range(intlev)
              if(!is.null(colmap) && !valuesAreColours) 
@@ -330,8 +417,8 @@ plot.im <- local({
              imagebreaks <- c(intlev - 0.5, max(intlev) + 0.5)
              ribbonvalues <- intlev
              ribbonrange <- range(imagebreaks)
-             ribbonbreaks <- imagebreaks
-             ribbonticks <- ribbonvalues
+#             ribbonbreaks <- imagebreaks
+             ribbonticks <- user.ticks %orifnull% ribbonvalues
              ribbonlabels <- paste(lev)
              vrange <- range(intlev)
              if(!is.null(colmap)) 
@@ -393,7 +480,29 @@ plot.im <- local({
              },
              NULL)
 
-    # ........ start plotting .................
+    ##  ........ decide whether to use rasterImage .........
+    
+    ## get device capabilities
+    ##      (this will start a graphics device if none is active)
+    rasterable <- dev.capabilities()$rasterImage
+    if(is.null(rasterable)) rasterable <- "no"
+    ##
+    can.use.raster <-
+      switch(rasterable,
+             yes=TRUE,
+             no=FALSE,
+             "non-missing"=!anyNA(x$v),
+             FALSE)
+    if(is.null(useRaster)) {
+      useRaster <- can.use.raster
+    } else if(useRaster && !can.use.raster) {
+        whinge <- "useRaster=TRUE is not supported by the graphics device"
+        if(rasterable == "non-missing")
+          whinge <- paste(whinge, "for images with NA values")
+        warning(whinge, call.=FALSE)
+    } 
+    
+    ## ........ start plotting .................
 
     if(!identical(ribbon, TRUE) || trivial) {
       ## no ribbon wanted
@@ -403,17 +512,21 @@ plot.im <- local({
         return(output.colmap)
 
       ## plot image without ribbon
-      image.doit(list(x=cellbreaks(x$xcol, x$xstep),
-                      y=cellbreaks(x$yrow, x$ystep),
-                      z=t(x$v)),
+      image.doit(imagedata=list(x=cellbreaks(x$xcol, x$xstep),
+                                y=cellbreaks(x$yrow, x$ystep),
+                                z=t(x$v)),
+                 W=xbox,
+                 workaround=workaround,
                  dotargs,
-                 list(useRaster=TRUE, add=add),
+                 list(useRaster=useRaster, add=add, show.all=show.all),
                  colourinfo,
                  list(xlab = "", ylab = ""),
-                 list(asp = 1, main = main, axes=FALSE)
-                 )
-      if(add && show.all)
-        fakemaintitle(x, main, dotargs)
+                 list(asp = 1, main = main, axes=FALSE))
+##      if(add && show.all)
+##        fakemaintitle(x, main, dotargs)
+
+      do.box.etc(Frame(x), add, dotargs)
+      
       return(invisible(output.colmap))
     }
     
@@ -450,54 +563,52 @@ plot.im <- local({
     bb.all <- boundingbox(bb.rib, bb)
 
     attr(output.colmap, "bbox") <- bb.all
+    attr(output.colmap, "bbox.legend") <- bb.rib
     if(!do.plot)
       return(output.colmap)
+
+    pt <- prepareTitle(main)
     
-    # establish coordinate system
-    if(!add)
-      do.call.plotfun("plot.default",
-                      resolve.defaults(list(x=0, y=0,  type="n", 
-                                            axes=FALSE, asp=1,
-                                            xlim=bb.all$xrange,
-                                            ylim=bb.all$yrange),
-                                       dotargs,
-                                       list(main=main, xlab="", ylab="")),
-                      extrargs=plotparams)
+    if(!add) {
+      ## establish coordinate system
+      do.call.plotfun(plot.owin,
+                      resolve.defaults(list(x=bb.all,
+                                            type="n",
+                                            main=pt$blank),
+                                       dotargs),
+                      extrargs=graphicsPars("owin"))
+    }
+    if(show.all) {
+      ## plot title centred over main image area 'bb'
+      do.call.plotfun(plot.owin,
+                      resolve.defaults(list(x=bb,
+                                            type="n",
+                                            main=main,
+                                            add=TRUE,
+                                            show.all=TRUE),
+                                       dotargs),
+                      extrargs=graphicsPars("owin"))
+      main <- ""
+    }
     # plot image
-    image.doit(list(x=cellbreaks(x$xcol, x$xstep),
-                    y=cellbreaks(x$yrow, x$ystep),
-                    z=t(x$v)),
-               list(add=TRUE),
+    image.doit(imagedata=list(x=cellbreaks(x$xcol, x$xstep),
+                              y=cellbreaks(x$yrow, x$ystep),
+                              z=t(x$v)),
+               W=xbox,
+               workaround=workaround,
+               list(add=TRUE, show.all=show.all),
                dotargs,
-               list(useRaster=TRUE),
+               list(useRaster=useRaster),
                colourinfo,
                list(xlab = "", ylab = ""),
                list(asp = 1, main = main))
 
-    if(add && show.all)
-      fakemaintitle(bb.all, main, ...)
+##    if(add && show.all)
+##      fakemaintitle(bb.all, main, ...)
     
-    # axes for image
-    imax <- identical(dotargs$axes, TRUE)
-    imbox <- !identical(dotargs$box, FALSE)
-    if(imbox)
-      rect(x$xrange[1], x$yrange[1], x$xrange[2], x$yrange[2])
-    if(imax) {
-      px <- pretty(bb$xrange)
-      py <- pretty(bb$yrange)
-      do.call.plotfun("axis",
-                      resolve.defaults(
-                                       list(side=1, at=px), 
-                                       dotargs,
-                                       list(pos=bb$yrange[1])),
-                      extrargs=axisparams)
-      do.call.plotfun("axis",
-                      resolve.defaults(
-                                       list(side=2, at=py), 
-                                       dotargs,
-                                       list(pos=bb$xrange[1])),
-                      extrargs=axisparams)
-    }
+    # box or axes for image
+    do.box.etc(bb, add, dotargs)
+
     # plot ribbon image containing the range of image values
     rib.npixel <- length(ribbonvalues) + 1
     switch(ribside,
@@ -509,7 +620,7 @@ plot.im <- local({
                                 to=bb.rib$yrange[2],
                                 length.out=rib.npixel)
              rib.z <- matrix(ribbonvalues, ncol=1)
-             rib.useRaster <- TRUE
+             rib.useRaster <- useRaster
            },
            top=,
            bottom={
@@ -522,9 +633,13 @@ plot.im <- local({
              # bug workaround
              rib.useRaster <- FALSE 
            })
-    image.doit(list(x=rib.xcoords, y=rib.ycoords,
-                    z=t(rib.z),
-                    add=TRUE),
+    image.doit(imagedata=list(x=rib.xcoords,
+                              y=rib.ycoords,
+                              z=t(rib.z)),
+               W=bb.rib,
+               workaround=workaround,
+               list(add=TRUE,
+                    show.all=show.all),
                ribargs,
                list(useRaster=rib.useRaster),
                list(main="", sub=""),
@@ -537,7 +652,15 @@ plot.im <- local({
     # scale axis for ribbon image
     ribaxis <- !(identical(resol$axes, FALSE) || identical(resol$ann, FALSE))
     if(ribaxis) {
-      axisargs <- list(side=rib.iside, labels=ribbonlabels)
+      ribaxis.iside <- rib.iside
+      ## check for user-supplied xlim, ylim with reverse order
+      ll <- resolve.defaults(ribargs, dotargs, list(xlim=NULL, ylim=NULL))
+      xlimflip <- is.numeric(ll$xlim) && (diff(ll$xlim) < 0)
+      ylimflip <- is.numeric(ll$ylim) && (diff(ll$ylim) < 0)
+      if(xlimflip) ribaxis.iside <- c(1, 4, 3, 2)[ribaxis.iside] 
+      if(ylimflip) ribaxis.iside <- c(3, 2, 1, 4)[ribaxis.iside]
+      ##
+      axisargs <- list(side=ribaxis.iside, labels=ribbonlabels)
       switch(ribside,
              right={
                scal <- diff(bb.rib$yrange)/diff(ribbonrange)
@@ -567,11 +690,11 @@ plot.im <- local({
                posargs <- list(pos=bb.rib$yrange[1],
                                xaxp=c(bb.rib$xrange, length(ribbonticks)))
              })
-      do.call.plotfun("axis",
-                      resolve.defaults(axisargs,
-                                       ribargs, dotargs,
+      do.call.plotfun(graphics::axis,
+                      resolve.defaults(ribargs,
+                                       axisargs, dotargs,
                                        posargs),
-                      extrargs=axisparams)
+                      extrargs=graphicsPars("axis"))
     }
     #
     return(invisible(output.colmap))
@@ -580,152 +703,142 @@ plot.im <- local({
   PlotIm
 })
 
-
+invokeColourmapRule <- function(colfun, x, ..., zlim=NULL, colargs=list()) {
+  ## utility for handling special functions that generate colour maps
+  ## either 
+  ##        function(... range) -> colourmap
+  ##        function(... inputs) -> colourmap
+  stopifnot(is.im(x))
+  stopifnot(is.function(colfun))
+  colargnames <- names(formals(colfun))
+  ## Convert it to a 'colourmap'
+  colmap <- NULL
+  xtype <- x$type
+  if(xtype %in% c("real", "integer") && "range" %in% colargnames) {
+    ## function(range) -> colourmap
+    vrange <- range(range(x, finite=TRUE), zlim)
+    cvals <- try(do.call.matched(colfun,
+                                 append(list(range=vrange), colargs)),
+                 silent=TRUE)
+    if(!inherits(cvals, "try-error")) {
+      colmap <- if(inherits(cvals, "colourmap")) cvals else
+      if(is.character(cvals)) colourmap(cvals, range=vrange) else NULL
+    }
+  } else if(xtype != "real" && "inputs" %in% colargnames) {
+    ## function(inputs) -> colourmap
+    vpossible <- switch(xtype,
+                        logical = c(FALSE, TRUE),
+                        factor = levels(x),
+                        unique(as.matrix(x)))
+    if(!is.null(vpossible) && length(vpossible) < 256) {
+      cvals <- try(do.call.matched(colfun,
+                                   append(list(inputs=vpossible),
+                                          colargs)),
+                   silent=TRUE)
+      if(!inherits(cvals, "try-error")) {
+        colmap <- if(inherits(cvals, "colourmap")) cvals else
+        if(is.character(cvals))
+          colourmap(cvals, inputs=vpossible) else NULL
+      }
+    }
+  }
+  return(colmap)
+}
 
 ########################################################################
 
 image.im <- plot.im
 
-########################################################################
-
-persp.im <- function(x, ..., colmap=NULL) {
-  xname <- deparse(substitute(x))
-  xinfo <- summary(x)
-  if(xinfo$type == "factor")
-    stop("Perspective plot is inappropriate for factor-valued image")
-  pop <- spatstat.options("par.persp")
-  ## check whether 'col' was specified when 'colmap' was intended
-  Col <- list(...)$col
-  if(is.null(colmap) && !is.null(Col) && !is.matrix(Col) && length(Col) != 1)
-    warning("Argument col is not a matrix. Did you mean colmap?")
-  # colour map?
-  if(is.null(colmap)) {
-    colinfo <- list(col=NULL)
-  } else if(inherits(colmap, "colourmap")) {
-    # colour map object
-    # apply colour function to image data
-    colval <- eval.im(colmap(x))
-    colval <- t(as.matrix(colval))
-    # strip one row and column for input to persp.default
-    colval <- colval[-1, -1]
-    # replace NA by arbitrary value
-    isna <- is.na(colval)
-    if(any(isna)) {
-      stuff <- attr(colmap, "stuff")
-      colvalues <- stuff$outputs
-      colval[isna] <- colvalues[1]
-    }
-    # pass colour matrix (and suppress lines)
-    colinfo <- list(col=colval, border=NA)
-  } else {
-    # interpret 'colmap' as colour map
-    if(is.list(colmap) && all(c("breaks", "col") %in% names(colmap))) {
-      breaks <- colmap$breaks
-      colvalues <- colmap$col
-    } else if(is.vector(colmap)) {
-      colvalues <- colmap
-      breaks <- quantile(x, seq(from=0,to=1,length.out=length(colvalues)+1))
-      if(!all(ok <- !duplicated(breaks))) {
-        breaks <- breaks[ok]
-        colvalues <- colvalues[ok[-1]]
-      }
-    } else warning("Unrecognised format for colour map")
-    # apply colour map to image values
-    colid <- cut.im(x, breaks=breaks, include.lowest=TRUE)
-    colval <- eval.im(colvalues[unclass(colid)])
-    colval <- t(as.matrix(colval))
-    nr <- nrow(colval)
-    nc <- ncol(colval)
-    colval <- colval[-1, -1]
-    colval[is.na(colval)] <- colvalues[1]
-    # pass colour matrix (and suppress lines)
-    colinfo <- list(col=colval, border=NA)
-  }
-
-  if(spatstat.options("monochrome"))
-    colinfo$col <- to.grey(colinfo$col)
-  
-  # get reasonable z scale while fixing x:y aspect ratio
-  if(xinfo$type %in% c("integer", "real")) {
-    zrange <- xinfo$range
-    if(diff(zrange) > 0) {
-      xbox <- as.rectangle(x)
-      zscale <- 0.5 * mean(diff(xbox$xrange), diff(xbox$yrange))/diff(zrange)
-      zlim <- zrange
-    } else {
-      zscale <- NULL
-      zlim <- c(0,2) * xinfo$mean
-    }
-  } else 
-    zscale <- zlim <- NULL
-
-  jawab <-
-    do.call.matched("persp",
-                    resolve.defaults(list(x=x$xcol, y=x$yrow, z=t(x$v)),
-                                     list(...),
-                                     pop,
-                                     colinfo,
-                                     list(xlab="x", ylab="y", zlab=xname),
-                                     list(scale=FALSE, expand=zscale,
-                                          zlim=zlim),
-                                     list(main=xname),
-                                     .StripNull=TRUE),
-                    funargs=.Spatstat.persp.args)
-  return(invisible(jawab))
-}
-
-.Spatstat.persp.args <- list("x", "y", "z",
-                             "xlim", "ylim", "zlim",
-                             "xlab", "ylab", "zlab",
-                             "main", "sub",
-                             "theta", "phi", "r", "d", "scale",
-                             "expand", "col", "border",
-                             "ltheta", "lphi", "shade", "box",
-                             "axes", "nticks", "ticktype")
-
 ######################################################################
 
 contour.im <- function (x, ..., main, axes=FALSE, add=FALSE,
+                        col=par("fg"), 
                         clipwin=NULL, show.all=!add, do.plot=TRUE)
 {
   defaultmain <- deparse(substitute(x))
+  dotargs <- list(...)
+  bb <- Frame(x)
   ## return value
-  z <- as.rectangle(x)
-  attr(z, "bbox") <- z
-  if(!do.plot) return(z)
-  ## 
+  result <- bb
+  attr(result, "bbox") <- bb
+  if(!do.plot) return(result)
+  ## main title
   sop <- spatstat.options("par.contour")
   if(missing(main)) 
-    main <- resolve.defaults(sop, list(main=defaultmain))$main
-  if(missing(add))
-    add <- resolve.defaults(sop, list(add=FALSE))$add
-  if(missing(axes))
-     axes <- resolve.defaults(sop, list(axes=TRUE))$axes
+    main <- resolve.1.default(list(main=defaultmain), sop)
+  pt <- prepareTitle(main)
+  ## plotting parameters
+  if(missing(add)) {
+    force(add) ## use default in formal arguments, unless overridden
+    add <- resolve.1.default(list(add=add), sop)
+  }
+  if(missing(axes)) {
+    force(axes)
+    axes <- resolve.1.default(list(axes=axes), sop)
+  }
+  axes <- axes && !add
+  col0 <- if(inherits(col, "colourmap")) par("fg") else col
+  ## clip to subset
   if(!is.null(clipwin))
     x <- x[clipwin, drop=FALSE]
-  if(show.all) {
-    if(axes) # with axes
-      do.call.plotfun("plot.default",
+  #' start plotting
+  if(!add) {
+    ## new plot - establish coordinate system
+    if(axes && show.all) {
+      #' standard plot initialisation in base graphics
+      do.call.plotfun(plot.default,
                       resolve.defaults(
                                        list(x = range(x$xcol),
                                             y = range(x$yrow),
-                                            type = "n", add=add),
+                                            type = "n"),
                                        list(...),
-                                       list(asp = 1, xlab = "x",
-                                            ylab = "y", main = main)))
-    else { # box without axes
-      rec <- owin(x$xrange, x$yrange)
-      do.call.matched("plot.owin",
-                      resolve.defaults(list(x=rec, add=add),
-                                       list(...),
-                                       list(main=main)))
+                                       list(asp = 1,
+                                            xlab = "x",
+                                            ylab = "y",
+                                            col = col0,
+                                            main = main)))
+    } else {
+      #' plot invisible bounding box
+      do.call.plotfun(plot.owin,
+                      resolve.defaults(list(x=bb,
+                                            type="n",
+                                            main=pt$blank),
+                                       dotargs),
+                      extrargs=graphicsPars("owin"))
+    }
+  } 
+  if(show.all && !axes) {
+    ## plot title centred over contour region
+    do.call.plotfun(plot.owin,
+                    resolve.defaults(list(x=bb,
+                                          main=main,
+                                          add=TRUE,
+                                          show.all=TRUE),
+                                     dotargs,
+                                     list(col.main=col0)),
+                    extrargs=graphicsPars("owin"))
+  }
+  #' plot contour lines
+  if(!inherits(col, "colourmap")) {
+    do.call.plotfun(contour.default,
+                    resolve.defaults(list(x=x$xcol, y=x$yrow, z=t(x$v)),
+                                     list(add=TRUE, col=col),
+                                     list(...)))
+  } else {
+    clin <- do.call.matched(contourLines,
+                            append(list(x=x$xcol, y=x$yrow, z=t(x$v)),
+                                   list(...)))
+    linpar <- graphicsPars("lines")
+    for(i in seq_along(clin)) {
+      lini <- clin[[i]]
+      levi <- lini$level
+      coli <- col(levi)
+      argi <- resolve.defaults(lini[c("x", "y")],
+                               list(...),
+                               list(col=coli))
+      do.call.matched(lines.default, argi, extrargs=linpar)
     }
   }
-  do.call.plotfun("contour.default",
-                  resolve.defaults(list(x=x$xcol, y=x$yrow, z=t(x$v)),
-                                   list(add=TRUE),
-                                   list(...)))
-  return(invisible(z))
+  return(invisible(result))
 }
 
-
