@@ -3,7 +3,7 @@
 #
 #  class of general point patterns in any dimension
 #
-#  $Revision: 1.42 $  $Date: 2014/03/17 05:55:43 $
+#  $Revision: 1.60 $  $Date: 2017/06/05 10:31:58 $
 #
 
 ppx <- local({
@@ -101,7 +101,7 @@ print.ppx <- function(x, ...) {
   invisible(NULL)
 }
 
-summary.ppx <- function(object, ...) { print(object, ...) }
+summary.ppx <- function(object, ...) { object }
 
 plot.ppx <- function(x, ...) {
   xname <- short.deparse(substitute(x))
@@ -112,7 +112,7 @@ plot.ppx <- function(x, ...) {
     coo <- coo[,1]
     ran <- diff(range(coo))
     ylim <- c(-1,1) * ran/20
-    do.call("plot.default",
+    do.call(plot.default,
             resolve.defaults(list(coo, numeric(length(coo))),
                              list(...),
                              list(asp=1, ylim=ylim,
@@ -122,7 +122,7 @@ plot.ppx <- function(x, ...) {
     if(is.null(dom)) {
       # plot x, y coordinates only
       nama <- names(coo)
-      do.call.matched("plot.default",
+      do.call.matched(plot.default,
                       resolve.defaults(list(x=coo[,1], y=coo[,2], asp=1),
                                        list(...),
                                        list(main=xname),
@@ -131,7 +131,7 @@ plot.ppx <- function(x, ...) {
       add <- resolve.defaults(list(...), list(add=FALSE))$add
       if(!add) {
         # plot domain, whatever it is
-        do.call("plot", resolve.defaults(list(dom),
+        do.call(plot, resolve.defaults(list(dom),
                                        list(...),
                                        list(main=xname)))
       }
@@ -139,7 +139,7 @@ plot.ppx <- function(x, ...) {
       x2 <- ppp(coo[,1], coo[,2], window=as.owin(dom),
                 marks=as.data.frame(marks(x)), check=FALSE)
       # invoke plot.ppp
-      return(do.call("plot", resolve.defaults(list(x2),
+      return(do.call(plot, resolve.defaults(list(x2),
                                               list(add=TRUE),
                                               list(...))))
     }
@@ -150,7 +150,7 @@ plot.ppx <- function(x, ...) {
     x3 <- pp3(coo[,1], coo[,2], coo[,3], dom)
     # invoke plot.pp3
     nama <- names(coo)
-    do.call("plot",
+    do.call(plot,
             resolve.defaults(list(x3),
                              list(...),
                              list(main=xname),
@@ -160,14 +160,48 @@ plot.ppx <- function(x, ...) {
   return(invisible(NULL))
 }
 
-"[.ppx" <- function (x, i, ...) {
+"[.ppx" <- function (x, i, drop=FALSE, ...) {
   da <- x$data
-  daij <- da[i, , drop=FALSE]
-  out <- list(data=daij, ctype=x$ctype, domain=x$domain)
+  dom <- x$domain
+  if(!missing(i)) {
+    if(inherits(i, c("boxx", "box3"))) {
+      dom <- i
+      i <- inside.boxx(da, w=i)
+    }
+    da <- da[i, , drop=FALSE]
+  }
+  out <- list(data=da, ctype=x$ctype, domain=dom)
   class(out) <- "ppx"
+  if(drop) {
+    # remove unused factor levels
+    mo <- marks(out)
+    switch(markformat(mo),
+           none = { },
+           vector = {
+             if(is.factor(mo))
+               marks(out) <- factor(mo)
+           },
+           dataframe = {
+             isfac <- sapply(mo, is.factor)
+             if(any(isfac))
+               mo[, isfac] <- lapply(mo[, isfac], factor)
+             marks(out) <- mo
+           },
+           hyperframe = {
+             lmo <- as.list(mo)
+             isfac <- sapply(lmo, is.factor)
+             if(any(isfac))
+               mo[, isfac] <- as.hyperframe(lapply(lmo[isfac], factor))
+             marks(out) <- mo
+           })
+  }
   return(out)
 }
 
+domain <- function(X, ...) { UseMethod("domain") }
+
+domain.ppx <- function(X, ...) { X$domain }
+
 coords <- function(x, ...) {
   UseMethod("coords")
 }
@@ -177,7 +211,7 @@ coords.ppx <- function(x, ..., spatial=TRUE, temporal=TRUE, local=TRUE) {
   chosen <- (ctype == "spatial" & spatial) |
             (ctype == "temporal" & temporal) | 
             (ctype == "local" & local) 
-  as.data.frame(x$data[, chosen])
+  as.data.frame(x$data[, chosen, drop=FALSE])
 }
 
 coords.ppp <- function(x, ...) { data.frame(x=x$x,y=x$y) }
@@ -225,7 +259,7 @@ marks.ppx <- function(x, ..., drop=TRUE) {
 "marks<-.ppx" <- function(x, ..., value) {
   ctype <- x$ctype
   retain <- (ctype != "mark")
-  coorddata <- x$data[, retain, drop=TRUE]
+  coorddata <- x$data[, retain, drop=FALSE]
   if(is.null(value)) {
     newdata <- coorddata
     newctype <- ctype[retain]
@@ -244,6 +278,8 @@ marks.ppx <- function(x, ..., drop=TRUE) {
       newdata <- coorddata
       newctype <- ctype[retain]
     } else {
+      if(nrow(coorddata) == 0) 
+        value <- value[integer(0), , drop=FALSE]
       newdata <- cbind(coorddata, value)
       newctype <- factor(c(as.character(ctype[retain]),
                            rep.int("mark", ncol(value))),
@@ -251,7 +287,7 @@ marks.ppx <- function(x, ..., drop=TRUE) {
     }
   }
   out <- list(data=newdata, ctype=newctype, domain=x$domain)
-  class(out) <- "ppx"
+  class(out) <- class(x)
   return(out)
 }
 
@@ -283,6 +319,33 @@ boxx <- function(..., unitname=NULL) {
   return(out)
 }
 
+as.boxx <- function(..., warn.owin = TRUE) {
+  a <- list(...)
+  n <- length(a)
+  if (n == 0) 
+    stop("No arguments given")
+  if (n == 1) {
+    a <- a[[1]]
+    if (inherits(a, "boxx")) 
+      return(a)
+    if (inherits(a, "box3")) 
+      return(boxx(a$xrange, a$yrange, a$zrange, unitname = a$units))
+    if (inherits(a, "owin")) {
+      if (!is.rectangle(a) && warn.owin) 
+        warning("The owin object does not appear to be rectangular - the bounding box is used!")
+      return(boxx(a$xrange, a$yrange, unitname = a$units))
+    }
+    if (is.numeric(a)) {
+      if ((length(a)%%2) == 0) 
+        return(boxx(split(a, rep(1:(length(a)/2), each = 2))))
+      stop(paste("Don't know how to interpret", length(a), "numbers as a box"))
+    }
+    if (!is.list(a)) 
+      stop("Don't know how to interpret data as a box")
+  }
+  return(do.call(boxx, a))
+}
+
 print.boxx <- function(x, ...) {
   m <- ncol(x$ranges)
   cat(paste(m, "-dimensional box:\n", sep=""))
@@ -338,27 +401,59 @@ shortside.boxx <- function(x) {
   return(min(sidelengths(x)))
 }
 
-eroded.volumes.boxx <- function(x, r) {
-  len <- sidelengths(x)
-  ero <- sapply(as.list(len), function(z, r) { pmax.int(0, z - 2 * r)}, r=r)
-  apply(ero, 1, prod)
+eroded.volumes.boxx <- local({
+
+  eroded.volumes.boxx <- function(x, r) {
+    len <- sidelengths(x)
+    ero <- sapply(as.list(len), erode1side, r=r)
+    apply(ero, 1, prod)
+  }
+
+  erode1side <- function(z, r) { pmax.int(0, z - 2 * r)}
+  
+  eroded.volumes.boxx
+})
+
+
+runifpointx <- function(n, domain, nsim=1, drop=TRUE) {
+  check.1.integer(n)
+  check.1.integer(nsim)
+  stopifnot(inherits(domain, "boxx"))
+  ra <- domain$ranges
+  d <- length(ra)
+  result <- vector(mode="list", length=nsim)
+  for(i in 1:nsim) {
+    if(n == 0) {
+      coo <- matrix(, nrow=0, ncol=d)
+    } else {
+      coo <- mapply(runif,
+                    n=rep(n, d),
+                    min=ra[1,],
+                    max=ra[2,])
+    }
+    colnames(coo) <- colnames(ra)
+    df <- as.data.frame(coo)
+    result[[i]] <- ppx(df, domain)
+  }
+  if(nsim == 1 && drop)
+    return(result[[1]])
+  result <- as.anylist(result)
+  names(result) <- paste("Simulation", 1:nsim)
+  return(result)
 }
 
-runifpointx <- function(n, domain) {
+rpoisppx <- function(lambda, domain, nsim=1, drop=TRUE) {
   stopifnot(inherits(domain, "boxx"))
-  coo <- lapply(domain$ranges,
-                function(ra, n) { runif(n, min=ra[1], max=ra[2]) },
-                n=n)
-  df <- do.call("data.frame", coo)
-  ppx(df, domain)
-}
-
-rpoisppx <- function(lambda, domain) {
-  stopifnot(inherits(domain, "boxx"))
-  vol <- volume.boxx(domain)
   stopifnot(is.numeric(lambda) && length(lambda) == 1 && lambda >= 0)
-  n <- rpois(1, lambda * vol)
-  runifpointx(n, domain)
+  n <- rpois(nsim, lambda * volume.boxx(domain))
+  result <- vector(mode="list", length=nsim)
+  for(i in 1:nsim) 
+    result[[i]] <- runifpointx(n[i], domain)
+  if(nsim == 1 && drop)
+    return(result[[1]])
+  result <- as.anylist(result)
+  names(result) <- paste("Simulation", 1:nsim)
+  return(result)
 }
 
 unique.ppx <- function(x, ..., warn=FALSE) {
@@ -375,8 +470,74 @@ duplicated.ppx <- function(x, ...) {
   return(dup)
 }
 
+anyDuplicated.ppx <- function(x, ...) {
+  anyDuplicated(as.data.frame(x), ...)
+}
+
+
 multiplicity.ppx <- function(x) {
   mul <- multiplicity(as.data.frame(x))
   return(mul)
 }
 
+intensity.ppx <- function(X, ...) {
+  if(!is.multitype(X)) {
+    n <- npoints(X)
+  } else {
+    mks <- marks(X)
+    n <- as.vector(table(mks))
+    names(n) <- levels(mks)
+  }
+  v <- volume(domain(X))
+  return(n/v)
+}
+
+grow.boxx <- function(W, left, right = left){
+  W <- as.boxx(W)
+  ra <- W$ranges
+  d <- length(ra)
+  if(any(left < 0) || any(right < 0))
+    stop("values of left and right margin must be nonnegative.")
+  if(length(left)==1) left <- rep(left, d)
+  if(length(right)==1) right <- rep(right, d)
+  if(length(left)!=d || length(right)!=d){
+    stop("left and right margin must be either of length 1 or the dimension of the boxx.")
+  }
+  W$ranges[1,] <- ra[1,]-left
+  W$ranges[2,] <- ra[2,]+right
+  return(W)
+}
+
+inside.boxx <- function(..., w = NULL){
+  if(is.null(w))
+    stop("Please provide a boxx using the named argument w.")
+  w <- as.boxx(w)
+  dat <- list(...)
+  if(length(dat)==1){
+    dat1 <- dat[[1]]
+    if(inherits(dat1, "ppx"))
+      dat <- coords(dat1)
+    if(inherits(dat1, "hyperframe"))
+      dat <- as.data.frame(dat1)
+  }
+  ra <- w$ranges
+  if(length(ra)!=length(dat))
+    stop("Mismatch between dimension of boxx and number of coordinate vectors.")
+  ## Check coord. vectors have equal length
+  n <- length(dat[[1]])
+  if(any(lengths(dat)!=n))
+    stop("Coordinate vectors have unequal length.")
+  index <- rep(TRUE, n)
+  for(i in seq_along(ra)){
+    index <- index & inside.range(dat[[i]], ra[[i]])
+  }
+  return(index)
+}
+
+
+spatdim <- function(X) {
+  if(is.sob(X)) 2L else
+  if(inherits(X, "box3")) 3 else
+  if(inherits(X, "boxx")) length(X$ranges) else 
+  if(is.ppx(X)) as.integer(sum(X$ctype == "spatial")) else NA_integer_
+}
