@@ -1,7 +1,7 @@
 #
 #    predict.ppm.S
 #
-#	$Revision: 1.80 $	$Date: 2014/05/04 02:12:00 $
+#	$Revision: 1.100 $	$Date: 2016/12/19 09:13:07 $
 #
 #    predict.ppm()
 #	   From fitted model obtained by ppm(),	
@@ -13,14 +13,18 @@
 
 predict.ppm <- local({
   ##
-  ##  extract undocumented arguments and trap others
+  ##  extract undocumented/outdated arguments, and trap others
   ##
-  xtract <- function(..., newdata=NULL, sumobj=NULL, E=NULL) {
+  xtract <- function(..., newdata=NULL, sumobj=NULL, E=NULL, total=NULL,
+                     getoutofjail=FALSE) {
     if(!is.null(newdata))
       warning(paste("The use of the argument", sQuote("newdata"),
                     "is out-of-date. See help(predict.ppm)"))
+    if(!is.null(total)) 
+      message(paste("The use of the argument", sQuote("total"),
+                    "is out-of-date. See help(predict.ppm)"))
     trap.extra.arguments(..., .Context="In predict.ppm")
-    return(list(sumobj=sumobj, E=E))
+    return(list(sumobj=sumobj, E=E, total=total, getoutofjail=getoutofjail))
   }
   ##
   ## confidence/prediction intervals for number of points
@@ -49,22 +53,50 @@ predict.ppm <- local({
     out
   }
 
+  typepublic <- c("trend", "cif", "intensity", "count")
+  typeaccept <- c(typepublic, "lambda", "se", "SE", "covariates")
+  typeuse    <- c(typepublic, "cif",    "se", "se", "covariates")
+  
   predict.ppm <- function(object, window=NULL, ngrid=NULL, locations=NULL,
-                          covariates=NULL, type="trend",
+                          covariates=NULL,
+                          type=c("trend", "cif", "intensity", "count"),
+                          se=FALSE,
                           interval=c("none", "confidence", "prediction"),
                           level = 0.95,
-                          total=NULL, 
                           X=data.ppm(object),
                           correction,
                           ..., new.coef=NULL, check=TRUE, repair=TRUE) {
-    type <- pickoption("type", type,
-                       c(trend="trend", cif="cif", lambda="cif",
-                         se="se", SE="se"))
     interval <- match.arg(interval)
     ## extract undocumented arguments 
     xarg <- xtract(...)
     sumobj <- xarg$sumobj
     E      <- xarg$E
+    total  <- xarg$total
+    getoutofjail <- xarg$getoutofjail
+    ## match 'type' argument including 'legacy' options
+    seonly <- FALSE
+    if(missing(type)) type <- type[1] else {
+      if(length(type) > 1) stop("Argument 'type' should be a single value")
+      mt <- pmatch(type, typeaccept)
+      if(is.na(mt)) stop("Argument 'type' should be one of",
+                         commasep(sQuote(typepublic), " or "))
+      type <- typeuse[mt]
+      if(type == "se") {
+        if(!getoutofjail)
+          message(paste("Outdated syntax:",
+                        "type='se' should be replaced by se=TRUE;",
+                        "then the standard error is predict(...)$se"))
+        type <- "trend"
+        se <- TRUE
+        seonly <- TRUE
+      }
+    } 
+    if(!is.null(total)) {
+      message("Outdated argument 'total': use 'window' and set type='count'")
+      type <- "count" 
+      if(!is.logical(total))
+        window <- if(is.tess(total)) total else as.owin(total)
+    }
     ##
     model <- object
     verifyclass(model, "ppm")
@@ -93,7 +125,7 @@ predict.ppm <- local({
     ##       find out what kind of model it is
     if(is.null(sumobj))
       sumobj <- summary(model, quick="entries")  # undocumented hack!
-    stationary  <- sumobj$stationary
+#    stationary  <- sumobj$stationary
     poisson     <- sumobj$poisson
     marked      <- sumobj$marked
     multitype   <- sumobj$multitype
@@ -101,7 +133,8 @@ predict.ppm <- local({
     changedcoef <- sumobj$changedcoef || !is.null(new.coef)
     trivial     <- poisson && notrend
   
-    need.covariates <- sumobj$has.covars
+    need.covariates <- sumobj$uses.covars
+    covnames.needed <- sumobj$covars.used
 
     if(sumobj$antiquated)
       warning("The model was fitted by an out-of-date version of spatstat")  
@@ -114,72 +147,74 @@ predict.ppm <- local({
         types <- levels(marks(sumobj$entries$data))
     }
 
-    ##    Standard error only available for Poisson models
-    if(type == "se" && !poisson)
-      stop(paste("Standard error calculation",
-                 "is only available for Poisson models"), call.=FALSE)
+    ## For Poisson models cif=intensity=trend
+    if(poisson && type %in% c("cif", "intensity"))
+      type <- "trend"
 
-    ## Point estimates for total count in a region
-    if(!is.null(total) && interval == "none") {
-      what <- switch(type,
-                     trend="estimate",
-                     se="se",
-                     cif=stop(
-                       paste("Prediction for total count",
-                             "is not implemented for type='cif'"),
-                       call.=FALSE))
-      if(identical(total, TRUE)) {
+    ## ............. trap un-implemented cases ...................
+    
+    ## Standard errors not yet available for cif, intensity
+    if(se && type %in% c("cif", "intensity"))
+      stop(paste("Standard error for", type, "is not yet implemented"),
+           call.=FALSE)
+
+    ## Intervals are only available for unmarked Poisson models
+    if(type == "count" && interval != "none" && (marked || !poisson)) {
+      stop(paste0(interval, " intervals for counts are only implemented for",
+                  if(marked) " unmarked" else "",
+                  if(!poisson) " Poisson",
+                  " models"),
+           call.=FALSE)
+    }
+
+    if(interval == "prediction" && type != "count")
+      stop("Prediction intervals are only available for type='count'",
+           call.=FALSE)
+    
+    if(interval == "confidence" && type %in% c("intensity", "cif")) 
+      stop(paste("Confidence intervals are not yet available for", type),
+           call.=FALSE)
+
+    estimatename <- if(interval == "none") "estimate" else interval
+    
+    ## ............. start computing .............................
+    
+    ## Total count in a region
+    
+    if(type == "count") {
+      ## point or interval estimate, optionally with SE
+      if(is.null(window)) {
         ## domain of the original data
-        result <- predconfPois(NULL, model, level, what)
-      } else if(is.tess(total)) {
+        if(!seonly) est <- predconfPois(NULL, model, level, estimatename)
+        if(se) sem <- predconfPois(NULL, model, level, "se")
+      } else if(is.tess(window)) {
         ## quadrats
-        result <- unlist(lapply(tiles(total), predconfPois,
-                                object=model, level=level, what=what))
+        tilz <- tiles(window)
+        if(!seonly) {
+          est <- unlist(lapply(tilz, predconfPois,
+                               object=model, level=level, what=estimatename))
+          if(interval != "none") # reshape
+            est <- matrix(unlist(est), byrow=TRUE, ncol=2,
+                          dimnames=list(names(est), names(est[[1]])))
+        }
+        if(se) sem <- unlist(lapply(tilz, predconfPois,
+                                    object=model, level=level, what="se"))
       } else {
         ## window
-        total <- as.owin(total)
-        result <- predconfPois(total, model, level, what)
+        if(!seonly) est <- predconfPois(window, model, level, estimatename)
+        if(se) sem <- predconfPois(window, model, level, "se")
       }
+      if(!se) return(est)
+      if(seonly) return(sem)
+      result <- list(est, sem)
+      names(result) <- c(estimatename, "se")
       return(result)
     }
+
+    ## .....   Predict a spatial function .......
     
-    ##  Confidence/prediction intervals 
     if(interval != "none") {
-      ## Only available for unmarked Poisson models
-      if(marked || !poisson)
-        stop(paste0(interval, " intervals are only implemented for",
-                    if(marked) " unmarked" else "",
-                    if(!poisson) " Poisson",
-                    " models"),
-             call.=FALSE)
-      ## Confidence/prediction interval for total count in a region
-      if(!is.null(total)) {
-        if(identical(total, TRUE)) {
-          ## domain of the original data
-          result <- predconfPois(NULL, model, level, interval)
-        } else if(is.tess(total)) {
-          ## quadrats
-          zz <- lapply(tiles(total), predconfPois,
-                       object=model, level=level, what=interval)
-          result <- matrix(unlist(zz), byrow=TRUE, ncol=2,
-                           dimnames=list(names(zz), names(zz[[1]])))
-        } else {
-          ## window
-          total <- as.owin(total)
-          result <- predconfPois(total, model, level, interval)
-        }
-        return(result)
-      }
-      ## Interval for trend etc
-      if(interval == "prediction")
-        stop("Prediction intervals are meaningless for intensity function",
-             call.=FALSE)
-      if(type != "trend")
-        warning(paste("Ignored argument: type =", sQuote(type),
-                      "because confidence intervals are specified"),
-                call.=FALSE)
-      ## Confidence interval for trend
-      type <- "confint"
+      ## Prepare for confidence interval 
       alpha2 <- (1-level)/2
       pp <- sort(c(alpha2, 1-alpha2))
       ci.names <- paste0(signif(100 * pp, 3), "%")
@@ -190,10 +225,15 @@ predict.ppm <- local({
     ##      (arguments present)    (output)  
     ##         window, ngrid    ->   image
     ##         locations (mask) ->   image
+    ##         locations (image) ->   image
     ##         locations (rectangle) ->  treat locations as 'window'
     ##         locations (polygonal) ->  treat locations as 'window'
     ##         locations (other) ->  data frame
     ##
+
+    if(is.im(locations))
+      locations <- as.owin(locations)
+    
     if(is.null(window) && is.owin(locations) && !is.mask(locations)) {
       window <- locations
       locations <- NULL
@@ -290,10 +330,9 @@ predict.ppm <- local({
       ## ------------------------------------ End Hack
       ##
       ## Finally, determine x and y vectors for grid
-      xx <- raster.x(masque)
-      yy <- raster.y(masque)
-      xpredict <- xx[masque$m]
-      ypredict <- yy[masque$m]
+      rxy <- rasterxy.mask(masque, drop=TRUE)
+      xpredict <- rxy$x
+      ypredict <- rxy$y 
     }
 
     ## ################  CREATE DATA FRAME  ##########################
@@ -332,6 +371,10 @@ predict.ppm <- local({
                      "Prediction is not possible at new locations"))
         covariates <- oldcov
       }
+      ## restrict to covariates actually required for formula
+      covariates <- if(is.data.frame(covariates)) {
+        covariates[,covnames.needed, drop=FALSE]
+      } else covariates[covnames.needed]
       covfunargs <- model$covfunargs
       covariates.df <-
         mpl.get.covariates(covariates,
@@ -356,45 +399,52 @@ predict.ppm <- local({
 
     if(!trivial) {
       Vnames <- model$internal$Vnames
+      vnameprefix <- model$internal$vnameprefix
       glmdata <- getglmdata(model)
       glmfit <- getglmfit(model)
       if(object$method=="logi")
         newdata$.logi.B <- rep(glmdata$.logi.B[1], nrow(newdata))
     }
-  
+
+    ## Undocumented secret exit
+    if(type == "covariates")
+      return(list(newdata=newdata, mask=if(want.image) masque else NULL))
+             
     ## ##########  COMPUTE PREDICTION ##############################
     ##
     ##   Compute the predicted value z[i] for each row of 'newdata'
     ##   Store in a vector z and reshape it later
     ##
+    ##
     ## #############################################################
-  
+
+    needSE <- se || (interval != "none")
+    
     if(trivial) {
       ## ###########  UNIFORM POISSON PROCESS #####################
 
       lambda <- exp(coeffs[[1]])
-      switch(type,
-             cif=,
-             trend={
+      if(needSE) {
+        npts <- nobs(model)
+        se.lambda <- lambda/sqrt(npts)
+      }
+      switch(interval,
+             none = {
                z <- rep.int(lambda, nrow(newdata))
              },
-             se={
-               npts <- nobs(model)
-               se.lambda <- lambda/sqrt(npts)
-               z <- rep.int(se.lambda, nrow(newdata))
-             },
-             confint={
-               npts <- npoints(data.ppm(model))
-               se.lambda <- lambda/sqrt(npts)
+             confidence = {
                z <- matrix(lambda + se.lambda * ci.q, 
                            byrow=TRUE,
                            nrow=nrow(newdata), ncol=2,
                            dimnames=list(NULL, ci.names))
              },
-             stop("Internal error: unrecognised type"))
+             stop("Internal error: unreached"))
+
+      if(se) 
+        zse <- rep.int(se.lambda, nrow(newdata))
     
       ## ##############################################################
-    } else if((type %in% c("trend","se")) || poisson) {
+    } else if((type %in% c("trend", "intensity")) || poisson) {
       ##
       ## ##########  COMPUTE TREND ###################################
       ##	
@@ -404,39 +454,38 @@ predict.ppm <- local({
       for(vn in Vnames)    
         newdata[[vn]] <- zeroes
       ##
-      ##   predict
+      ##   predict trend
       ##
-      lambda <- GLMpredict(glmfit, newdata, coeffs, 
-                           changecoef=changedcoef)
+      z <- lambda <- GLMpredict(glmfit, newdata, coeffs, 
+                                changecoef=changedcoef)
       ##
-      switch(type,
-             cif=,
-             trend={
-               z <- lambda
-             },
-             confint=,
-             se={
-               ## extract variance-covariance matrix of parameters
-               vc <- vcov(model)
-               ## compute model matrix
-               fmla <- formula(model)
-               mf <- model.frame(fmla, newdata, ..., na.action=na.pass)
-               mm <- model.matrix(fmla, mf, ..., na.action=na.pass)
-               if((nr <- nrow(mm)) != nrow(newdata))
-                 stop("Internal error: row mismatch in SE calculation")
-               ## compute relative variance = diagonal of quadratic form
-               vv <- quadform(mm, vc)
-               ## standard error
-               SE <- lambda * sqrt(vv)
-               if(type == "se") {
-                 z <- SE
-               } else if(type == "confint") {
-                 z <- lambda + outer(SE, ci.q, "*")
-                 colnames(z) <- ci.names
-               }
-             },
-             stop("Internal error: unrecognised type"))
-
+      if(type == "intensity") 
+        z <- PoisSaddle(z, fitin(model))
+      
+      ##
+      if(needSE) {
+        ## extract variance-covariance matrix of parameters
+        vc <- vcov(model)
+        ## compute model matrix
+        fmla <- formula(model)
+#        mf <- model.frame(fmla, newdata, ..., na.action=na.pass)
+#        mm <- model.matrix(fmla, mf, ..., na.action=na.pass)
+        mf <- model.frame(fmla, newdata, na.action=na.pass)
+        mm <- model.matrix(fmla, mf, na.action=na.pass)
+        if(nrow(mm) != nrow(newdata))
+          stop("Internal error: row mismatch in SE calculation")
+        ## compute relative variance = diagonal of quadratic form
+        vv <- quadform(mm, vc)
+        ## standard error
+        SE <- lambda * sqrt(vv)
+        if(se) 
+          zse <- SE
+        if(interval == "confidence") {
+          z <- lambda + outer(SE, ci.q, "*")
+          colnames(z) <- ci.names
+        } 
+      } 
+      
       ## ############################################################  
     } else if(type == "cif" || type =="lambda") {
       ## ####### COMPUTE FITTED CONDITIONAL INTENSITY ################
@@ -447,7 +496,7 @@ predict.ppm <- local({
       W <- as.owin(data.ppm(model))
       U <- ppp(newdata$x, y=newdata$y, window=W, check=FALSE)
       if(marked) 
-        marks(U) <- Umarks <- newdata$marks
+        marks(U) <- newdata$marks
       ## determine which prediction points are data points
       if(is.null(E))
         E <- equalpairs(U, X, marked)
@@ -465,7 +514,7 @@ predict.ppm <- local({
         ## Assign values to a column of the same name in newdata
         newdata[[Vnames]] <- as.vector(Vnew)
       ##
-      } else if(is.null(dimnames(Vnew)[[2]])) {
+      } else if(is.null(avail <- colnames(Vnew))) {
         ## Potential is vector-valued (Vnew is a matrix)
         ## with unnamed components.
         ## Assign the components, in order of their appearance,
@@ -477,9 +526,20 @@ predict.ppm <- local({
         ## Potential is vector-valued (Vnew is a matrix)
         ## with named components.
         ## Match variables by name
-        for(vn in Vnames)    
-          newdata[[vn]] <- Vnew[,vn]
-        ##
+        if(all(Vnames %in% avail)) {
+          for(vn in Vnames)
+            newdata[[ vn ]] <- Vnew[ , vn]
+        } else if(all(Vnames %in% (Pavail <- paste0(vnameprefix, avail)))) {
+          for(vn in Vnames)
+            newdata[[ vn ]] <- Vnew[ , match(vn, Pavail)]
+        } else
+          stop(paste(
+            "Internal error: unable to match names",
+            "of available interaction terms",
+            commasep(sQuote(avail)),
+            "to required interaction terms",
+            commasep(sQuote(Vnames))
+            ), call.=FALSE)
       }
       ## invoke predict.glm or compute prediction
       z <- GLMpredict(glmfit, newdata, coeffs, 
@@ -497,36 +557,67 @@ predict.ppm <- local({
     ##
     ## reshape the result
     ##
-    if(!want.image) 
-      out <- as.vector(z)
+    if(!want.image) {
+      if(!se) {
+        out <- as.vector(z)
+      } else if(seonly) {
+        out <- as.vector(zse)
+      } else {
+        out <- list(as.vector(z), as.vector(zse))
+        names(out) <- c(estimatename, "se")
+      }
+    }
     else {
-      ## make an image of the right shape
-      imago <- as.im(masque)
-      imago <- eval.im(as.double(imago))
-      M <- masque$m
+      ## make an image of the right shape and value
+      imago <- as.im(masque, value=1.0)
       if(!marked && interval=="none") {
         ## single image
-        out <- imago
-        ## set entries
-        out$v[M] <- z
+        if(!se) {
+          out <- imago
+          ## set entries
+          out[] <- z
+        } else if(seonly) {
+          out <- imago
+          out[] <- zse
+        } else {
+          est <- std <- imago
+          est[] <- z
+          std[] <- zse
+          out <- list(est, std)
+          names(out) <- c(estimatename, "se")
+        }
       } else if(interval != "none") {
         ## list of 2 images for CI
-        hi <- lo <- imago
-        hi$v[M] <- z[,1]
-        lo$v[M] <- z[,2]
-        out <- listof(hi, lo)
-        names(out) <- ci.names
+        if(!seonly) {
+          hi <- lo <- imago
+          hi[] <- z[,1]
+          lo[] <- z[,2]
+          est <- solist(hi, lo)
+          names(est) <- ci.names
+        }
+        if(se) {
+          std <- imago
+          std[] <- zse
+        }
+        if(!se) {
+          out <- est
+        } else if(seonly) {
+          out <- std
+        } else {
+          out <- list(est, std)
+          names(out) <- c(estimatename, "se")
+        }
       } else {
         ## list of images, one for each level of marks
         out <- list()
         for(i in seq_along(types)) {
           outi <- imago
           ## set entries
-          outi$v[masque$m] <- z[newdata$marks == types[i]]
+          outi[] <- z[newdata$marks == types[i]]
           out[[i]] <- outi
         }
-        out <- as.listof(out)
-        names(out) <- paste("mark", types, sep="")
+        out <- as.solist(out)
+        names(out) <- as.character(types)
       }
     }
     ##  
@@ -592,17 +683,35 @@ model.se.image <- function(fit, W=as.owin(fit), ..., what="sd") {
   return(U)
 }
 
-GLMpredict <- function(fit, data, coefs, changecoef=TRUE) {
-  if(!changecoef) {
-    answer <- predict(fit, newdata=data, type="response")
+GLMpredict <- function(fit, data, coefs, changecoef=TRUE,
+                       type=c("response", "link")) {
+  ok <- is.finite(coefs)
+  type <- match.arg(type)
+  if(!changecoef && all(ok)) {
+    answer <- predict(fit, newdata=data, type=type)
   } else {
     # do it by hand
     fmla <- formula(fit)
     data$.mpl.Y <- 1
-    fram <- model.frame(fmla, data=data)
+    fram <- model.frame(fmla, data=data, na.action=NULL)
     # linear predictor
     mm <- model.matrix(fmla, data=fram)
-    eta <- as.vector(mm %*% coefs)
+    # ensure all required coefficients are present
+    coefs <- fill.coefs(coefs, colnames(mm))
+    ok <- is.finite(coefs)
+    #
+    if(all(ok)) {
+      eta <- as.vector(mm %*% coefs)
+    } else {
+      #' ensure 0 * anything = 0
+      eta <- as.vector(mm[ , ok, drop=FALSE] %*% coefs[ok])
+      for(j in which(!ok)) {
+        mmj <- mm[, j]
+        nonzero <- is.na(mmj) | (mmj != 0)
+        if(any(nonzero))
+          eta[nonzero] <- eta[nonzero] + mmj[nonzero] * coefs[j]
+      }
+    }
     # offset
     mo <- model.offset(fram)
     if(!is.null(mo)) {
@@ -610,15 +719,19 @@ GLMpredict <- function(fit, data, coefs, changecoef=TRUE) {
         mo <- apply(mo, 1, sum)
       eta <- mo + eta
     }
-    # response
-    linkinv <- family(fit)$linkinv
-    answer <- linkinv(eta)
+    switch(type,
+           link = {
+             answer <- eta
+           },
+           response = {
+             linkinv <- family(fit)$linkinv
+             answer <- linkinv(eta)
+           })
   }
   # Convert from fitted logistic prob. to lambda for logistic fit
-  if(family(fit)$family=="binomial")
+  if(type == "response" && family(fit)$family=="binomial")
     answer <- fit$data$.logi.B[1] * answer/(1-answer)
   return(answer)
-
 }
 
 # An 'equalpairs' matrix E is needed in the ppm class
@@ -646,3 +759,35 @@ equalpairs <- function(U, X, marked=FALSE) {
 }
 
   
+fill.coefs <- function(coefs, required) {
+  # 'coefs' should contain all the 'required' values
+  coefsname <- deparse(substitute(coefs))
+  nama <- names(coefs)
+  if(is.null(nama)) {
+    #' names cannot be matched
+    if(length(coefs) != length(required))
+      stop(paste("The unnamed argument", sQuote(coefsname),
+                 "has", length(coefs), "entries, but",
+                 length(required), "are required"),
+           call.=FALSE)
+    # blithely assume they match 1-1
+    names(coefs) <- required
+    return(coefs)
+  }
+  stopifnot(is.character(required))
+  if(identical(nama, required)) return(coefs)
+  inject <- match(nama, required)
+  if(any(notneeded <- is.na(inject))) {
+    warning(paste("Internal glitch: some coefficients were not required:",
+                  commasep(sQuote(nama[notneeded]))),
+            call.=FALSE)
+    coefs <- coefs[!notneeded]
+    nama <- names(coefs)
+    inject <- match(nama, required)
+  }
+  y <- numeric(length(required))
+  names(y) <- required
+  y[inject] <- coefs
+  return(y)
+}
+ 
