@@ -1,7 +1,7 @@
 #
 #  psp.R
 #
-#  $Revision: 1.74 $ $Date: 2014/04/21 04:41:40 $
+#  $Revision: 1.89 $ $Date: 2017/09/13 09:52:15 $
 #
 # Class "psp" of planar line segment patterns
 #
@@ -79,7 +79,7 @@ as.psp <- function(x, ..., from=NULL, to=NULL) {
       stop(paste("The point patterns", sQuote("from"), "and", sQuote("to"),
                  "have different numbers of points.\n"))
     uni <- union.owin(from$window, to$window)
-    Y <- do.call("psp",
+    Y <- do.call(psp,
                  resolve.defaults(list(from$x, from$y, to$x, to$y),
                                   list(...),
                                   list(window=uni)))
@@ -123,7 +123,7 @@ as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
     bb <- boundingbox(window)
     rmax <- max(rr)
     bigbox <- owin(bb$xrange + c(-1,1) * rmax, bb$yrange + c(-1,1) * rmax)
-    pattern <- psp(x$x - dx, x$y - dy, x$x + dx, x$y + dy,
+    pattern <- psp(x$xmid - dx, x$ymid - dy, x$xmid + dx, x$ymid + dy,
                    window=bigbox,check=FALSE)
     out <- pattern[window]
     x <- x[-match(c("xmid","ymid","length","angle"),names(x))]
@@ -134,7 +134,7 @@ as.psp.data.frame <- function(x, ..., window=NULL, marks=NULL,
     x <- x[-(1:4)]
   }
   else if(fatal)
-    stop("Unable to interpret x as a line segment pattern.\n")
+    stop("Unable to interpret x as a line segment pattern.", call.=FALSE)
   else out <- NULL
 
   if(!is.null(out)) {
@@ -190,8 +190,9 @@ as.psp.owin <- function(x, ..., window=NULL,
   edges(x, ..., window=window, check=check)
 }
 
+
 edges <- function(x, ...,
-                  window=NULL, check=spatstat.options("checksegments")) {
+                  window=NULL, check=FALSE) {
   x <- as.owin(x)
   if(is.null(window)) window <- as.rectangle(x)
   x <- as.polygonal(x)
@@ -210,6 +211,13 @@ edges <- function(x, ...,
   return(out)
 }
 
+xypolygon2psp <- function(p, w, check=spatstat.options("checksegments")) {
+  verify.xypolygon(p)
+  n <- length(p$x)
+  nxt <- c(2:n, 1)
+  return(psp(p$x, p$y, p$x[nxt], p$y[nxt], window=w, check=check))
+}
+         
 
 #################
 
@@ -289,8 +297,7 @@ marks.psp <- function(x, ..., dfok = TRUE) {
             }
             else {
                 if (nrow(m) == 1 || nseg == 0) {
-                  marx <- as.data.frame(lapply(as.list(m),function(x,k) {
-                    rep.int(x, k)}, k = nseg))
+                  marx <- as.data.frame(lapply(as.list(m), rep.int, times=nseg))
                 }
                 else stop("Number of rows of data frame != number of points.\n")
             }
@@ -317,7 +324,10 @@ unmark.psp <- function(X) {
 #  plot and print methods
 #################################################
 
-plot.psp <- function(x, ..., main, add=FALSE, show.all=!add, which.marks=1,
+plot.psp <- function(x, ..., main, add=FALSE,
+                     show.all=!add, 
+                     show.window=show.all,
+                     which.marks=1,
                      ribbon=show.all, ribsep=0.15, ribwid=0.05, ribn=1024,
                      do.plot=TRUE) {
   if(missing(main) || is.null(main))
@@ -331,13 +341,19 @@ plot.psp <- function(x, ..., main, add=FALSE, show.all=!add, which.marks=1,
   do.ribbon <- identical(ribbon, TRUE) && use.colour 
   ##
   ## ....   initialise plot; draw observation window  ......
+  owinpars <- setdiff(graphicsPars("owin"), "col")
   if(!do.ribbon) {
     ## window of x only
     bb.all <- as.rectangle(as.owin(x))
-    if(do.plot && show.all)
-      do.call.matched("plot.owin", 
-                      append(list(x=x$window, main=main, add=add),
-                             list(...)))
+    if(do.plot && (!add || show.window))
+      do.call.plotfun(plot.owin, 
+                      resolve.defaults(list(x=x$window,
+		                            main=if(show.all) main else "",
+                                            add=add,
+                                            type = if(show.window) "w" else "n",
+                                            show.all=show.all),
+                                       list(...)),
+                      extrargs=owinpars)
   } else {
     ## enlarged window with room for colour ribbon
     ## x at left, ribbon at right
@@ -349,23 +365,28 @@ plot.psp <- function(x, ..., main, add=FALSE, show.all=!add, which.marks=1,
                    bb$yrange)
     bb.all <- boundingbox(bb.rib, bb)
     if(do.plot) {
+      pt <- prepareTitle(main)
       ## establish coordinate system
       if(!add)
-        do.call.plotfun("plot.default",
-                        resolve.defaults(list(x=0, y=0, type="n",
-                                              axes=FALSE, asp=1,
-                                              xlim=bb.all$xrange,
-                                              ylim=bb.all$yrange),
-                                         list(...),
-                                         list(main=main, xlab="", ylab="")))
+      do.call.plotfun(plot.owin,
+                      resolve.defaults(list(x=bb.all,
+                                            type="n",
+                                            main=pt$blank),
+                                       list(...)),
+                      extrargs=owinpars)
       ## now plot window of x
-      if(show.all) 
-        do.call.matched("plot.owin", 
+      ## with title centred on this window
+      if(show.window) {
+        do.call.plotfun(plot.owin, 
                         resolve.defaults(list(x=x$window,
                                               add=TRUE,
                                               main=main,
-                                              show.all=add),
-                                         list(...)))
+                                              show.all=TRUE),
+                                         list(...)),
+                        extrargs=owinpars)
+        ## title done. 
+        main <- ""
+      }
     }
   }
 
@@ -404,7 +425,7 @@ plot.psp <- function(x, ..., main, add=FALSE, show.all=!add, which.marks=1,
 
   if(do.plot) {
     ## plot segments
-    do.call.matched("segments",
+    do.call.plotfun(segments,
                     resolve.defaults(as.list(x$ends),
                                      list(...),
                                      list(col=col),
@@ -423,33 +444,30 @@ plot.psp <- function(x, ..., main, add=FALSE, show.all=!add, which.marks=1,
 }
 
 print.psp <- function(x, ...) {
-   verifyclass(x, "psp")
-    ism <- is.marked(x, dfok = TRUE)
-    cat(paste(if(ism)
-        "marked"
-    else NULL, "planar line segment pattern:", x$n, "line", ngettext(x$n, "segment",
-        "segments"), "\n"))
-    if (ism) {
-        mks <- marks(x, dfok = TRUE)
-        if (is.data.frame(mks)) {
-            cat(paste("Mark variables: ", paste(names(mks), collapse = ", "),
-                "\n"))
-        }
-        else {
-            if (is.factor(mks)) {
-                cat("multitype, with ")
-                cat(paste("levels =", paste(levels(mks), collapse = "\t"), 
-                  "\n"))
-            }
-            else {
-                cat(paste("marks are", if (is.numeric(mks)) 
-                  "numeric,", "of type", sQuote(typeof(mks)), 
-                  "\n"))
-            }
-        }
+  verifyclass(x, "psp")
+  n <- x$n
+  ism <- is.marked(x, dfok = TRUE)
+  splat(if(ism) "marked" else NULL,
+        "planar line segment pattern:",
+        n, ngettext(n, "line segment", "line segments"))
+  if(ism) {
+    mks <- marks(x, dfok = TRUE)
+    if(is.data.frame(mks)) {
+      splat("Mark variables: ",
+            paste(names(mks), collapse = ", "))
+    } else {
+      if(is.factor(mks)) {
+        splat("multitype, with levels =",
+              paste(levels(mks), collapse = "\t"))
+      } else {
+        splat("marks are",
+              if(is.numeric(mks)) "numeric," else NULL,
+              "of type", sQuote(typeof(mks)))
+      }
     }
-    print(x$window)
-    return(invisible(NULL))
+  }
+  print(x$window)
+  return(invisible(NULL))
 }
 
 unitname.psp <- function(x) {
@@ -527,9 +545,10 @@ midpoints.psp <- function(x) {
   ppp(x=xm, y=ym, window=win, check=FALSE)
 }
 
-lengths.psp <- function(x) {
+lengths.psp <- function(x, squared=FALSE) {
   verifyclass(x, "psp")
-  eval(expression(sqrt((x1-x0)^2 + (y1-y0)^2)), envir=x$ends)
+  lengths2 <- eval(expression((x1-x0)^2 + (y1-y0)^2), envir=x$ends)
+  return(if(squared) lengths2 else sqrt(lengths2))
 }
 
 angles.psp <- function(x, directed=FALSE) {
@@ -577,7 +596,7 @@ print.summary.psp <- function(x, ...) {
 ########################################################
 
 "[.psp" <-
-  function(x, i, j, drop, ...) {
+  function(x, i, j, drop, ..., fragments=TRUE) {
 
     verifyclass(x, "psp")
     
@@ -588,7 +607,7 @@ print.summary.psp <- function(x, ...) {
       style <- if(inherits(i, "owin")) "window" else "index"
       switch(style,
              window={
-               x <- clip.psp(x, window=i, check=FALSE)
+               x <- clip.psp(x, window=i, check=FALSE, fragments=fragments)
              },
              index={
                enz <- x$ends[i, ]
@@ -670,10 +689,12 @@ rotate.psp <- function(X, angle=pi/2, ..., centre=NULL) {
 
 is.empty.psp <- function(x) { return(x$n == 0) } 
 
-identify.psp <- function(x, ..., labels=seq_len(nsegments(x)), n=nsegments(x), plot=TRUE) {
+identify.psp <- function(x, ..., labels=seq_len(nsegments(x)),
+                         n=nsegments(x), plot=TRUE) {
   Y <- x
   W <- as.owin(Y)
   mids <- midpoints.psp(Y)
+  poz <- c(1, 2,4, 3)[(floor(angles.psp(Y)/(pi/4)) %% 4) + 1L]
   if(!(is.numeric(n) && (length(n) == 1) && (n %% 1 == 0) && (n >= 0)))
     stop("n should be a single integer")
   out <- integer(0)
@@ -693,7 +714,11 @@ identify.psp <- function(x, ..., labels=seq_len(nsegments(x)), n=nsegments(x), p
         # Display
         mi <- mids[ident]
         li <- labels[ident]
-        text(mi$x, mi$y, labels=li)
+        po <- poz[ident]
+        do.call.matched(graphics::text.default,
+                        resolve.defaults(list(x=mi$x, y=mi$y, labels=li),
+                                         list(...),
+                                         list(pos=po)))
       }
       out <- c(out, ident)
     }
@@ -718,3 +743,27 @@ as.ppp.psp <- function (X, ..., fatal=TRUE)
   return(Y)
 }
 
+domain.psp <- Window.psp <- function(X, ...) { as.owin(X) }
+
+"Window<-.psp" <- function(X, ..., value) {
+  verifyclass(value, "owin")
+  X[value]
+}
+
+edit.psp <- function(name, ...) {
+  x <- name
+  y <- edit(as.data.frame(x), ...)
+  xnew <- as.psp(y, window=Window(x))
+  return(xnew)
+}
+
+text.psp <- function(x, ...) {
+  mids <- midpoints.psp(x)
+  poz <- c(1, 2,4, 3)[(floor(angles.psp(x)/(pi/4)) %% 4) + 1L]
+  do.call.matched(graphics::text.default,
+                  resolve.defaults(list(x=mids$x, y=mids$y),
+                                   list(...),
+                                   list(pos=poz),
+                                   .StripNull=TRUE))
+  return(invisible(NULL))
+}
