@@ -1,9 +1,11 @@
 #
-# $Id: rmh.default.R,v 1.97 2013/04/25 06:37:43 adrian Exp adrian $
+# $Id: rmh.default.R,v 1.108 2017/06/05 10:31:58 adrian Exp adrian $
 #
 rmh.default <- function(model,start=NULL,
                         control=default.rmhcontrol(model),
-                        ..., verbose=TRUE, snoop=FALSE) {
+                        ...,
+                        nsim=1, drop=TRUE, saveinfo=TRUE,
+                        verbose=TRUE, snoop=FALSE) {
 #
 # Function rmh.  To simulate realizations of 2-dimensional point
 # patterns, given the conditional intensity function of the 
@@ -32,6 +34,8 @@ rmh.default <- function(model,start=NULL,
     control <- update(control, ...)
 
   control <- rmhResolveControl(control, model)
+
+  saveinfo <- as.logical(saveinfo)
   
   # retain "..." arguments unrecognised by rmhcontrol
   # These are assumed to be arguments of functions defining the trend
@@ -103,14 +107,14 @@ rmh.default <- function(model,start=NULL,
     if(is.im(trend))
       as.owin(trend)
     else if(is.list(trend) && any(ok <- unlist(lapply(trend, is.im))))
-      as.owin((trend[ok])[[1]])
+      as.owin((trend[ok])[[1L]])
     else NULL
 
 ##  Clipping window (for final result)
   
   w.clip <-
-    if(!is.null(model$w))
-      model$w
+    if(!is.null(w.model))
+      w.model
     else if(!will.expand(control$expand)) {
       if(start$given == "x" && is.ppp(x.start))
         x.start$window
@@ -153,9 +157,7 @@ rmh.default <- function(model,start=NULL,
     if(any(images)) {
       iwindows <- lapply(trends[images], as.owin)
       nimages <- length(iwindows)
-      misfit <- !unlist(lapply(iwindows,
-                               function(x,w) { is.subset.owin(w,x) },
-                               w = w.sim))
+      misfit <- !sapply(iwindows, is.subset.owin, A=w.sim)
       nmisfit <- sum(misfit)
       if(nmisfit > 1) 
         stop(paste("Expanded simulation window is not contained in",
@@ -198,7 +200,7 @@ rmh.default <- function(model,start=NULL,
   
   condtype <- control$condtype
   x.cond   <- control$x.cond
-  n.cond   <- control$n.cond
+#  n.cond   <- control$n.cond
 
   switch(condtype,
          none={
@@ -248,6 +250,10 @@ rmh.default <- function(model,start=NULL,
 
 ###################### Starting state data ############################
 
+# whether the initial state should be thinned
+  
+  thin <- (start$given != "x") && (control$fixing == "none")
+  
 # There must now be a starting state.
   
   if(start$given == "none") {
@@ -256,10 +262,10 @@ rmh.default <- function(model,start=NULL,
       stop("No starting state given")
     # Determine integral of beta * trend over data window.
     # This is the expected number of points in the reference Poisson process.
-    area.w.clip <- area.owin(w.clip)
+    area.w.clip <- area(w.clip)
     if(trendy) {
       tsummaries <- summarise.trend(trend, w=w.clip, a=area.w.clip)
-      En <- beta * unlist(lapply(tsummaries, function(x) { x$integral }))
+      En <- beta * sapply(tsummaries, getElement, name="integral")
     } else {
       En <- beta * area.w.clip
     }
@@ -303,7 +309,7 @@ rmh.default <- function(model,start=NULL,
            # with the expansion that has occurred.  
            if(expanded) {
 	     holnum <- if(spatstat.options("scalable")) round else ceiling
-             n.start <- holnum(n.start * area.owin(w.sim)/area.owin(w.clip))
+             n.start <- holnum(n.start * area(w.sim)/area(w.clip))
            }
            #
            npts.free <- sum(n.start) # The ``sum()'' is redundant if n.start
@@ -364,16 +370,16 @@ rmh.default <- function(model,start=NULL,
 # Integral of trend over the expanded window (or area of window):
 # Iota == Integral Of Trend (or) Area.
 
-  area.w.sim <- area.owin(w.sim)
+  area.w.sim <- area(w.sim)
   if(trendy) {
     if(verbose)
       cat("Evaluating trend integral...")
     tsummaries <- summarise.trend(trend, w=w.sim, a=area.w.sim)
-    nbg  <- unlist(lapply(tsummaries, function(x) { x$min < 0 }))
-    if(any(nbg))
+    mins  <- sapply(tsummaries, getElement, name="min")
+    if(any(mins < 0))
       stop("Trend has negative values")
-    iota <- unlist(lapply(tsummaries, function(x) { x$integral }))
-    tmax <- unlist(lapply(tsummaries, function(x) { x$max }))
+    iota <- sapply(tsummaries, getElement, name="integral")
+    tmax <- sapply(tsummaries, getElement, name="max")
   } else {
     iota <- area.w.sim
     tmax <- NULL
@@ -466,7 +472,8 @@ rmh.default <- function(model,start=NULL,
                            w.state=w.state,
                            x.condpp=x.condpp,
                            ptypes=ptypes,
-                           period=period)
+                           period=period,
+                           thin=thin)
 
   Model$internal <- list(a.s.empty=a.s.empty,
                          empty=if(a.s.empty) empty else NULL,
@@ -483,15 +490,38 @@ rmh.default <- function(model,start=NULL,
   class(InfoList) <- c("rmhInfoList", class(InfoList))
 
   # go
-  do.call("rmhEngine",
-          append(list(InfoList,
-                      verbose=verbose, snoop=snoop, kitchensink=TRUE),
-                 f.args))
+  if(nsim == 1 && drop) {
+    result <- do.call(rmhEngine,
+                      append(list(InfoList,
+                                  verbose=verbose,
+                                  snoop=snoop,
+                                  kitchensink=saveinfo),
+                             f.args))
+  } else {
+    result <- vector(mode="list", length=nsim)
+    if(verbose) {
+      splat("Generating", nsim, "point patterns...")
+      pstate <- list()
+    }
+    subverb <- verbose && (nsim == 1)
+    for(isim in 1:nsim) {
+      if(verbose) pstate <- progressreport(isim, nsim, state=pstate)
+      result[[isim]] <- do.call(rmhEngine,
+                                append(list(InfoList,
+                                            verbose=subverb,
+                                            snoop=snoop,
+                                            kitchensink=saveinfo),
+                                       f.args))
+    }
+    result <- as.solist(result)
+    if(verbose) splat("Done.\n")
+  }
+  return(result)
 }
 
 print.rmhInfoList <- function(x, ...) {
   cat("\nPre-digested Metropolis-Hastings algorithm parameters (rmhInfoList)\n")
-  print(as.listof(x))
+  print(as.anylist(x))
 }
 
 #---------------  rmhEngine -------------------------------------------
@@ -522,7 +552,8 @@ print.rmhInfoList <- function(x, ...) {
 
 rmhEngine <- function(InfoList, ...,
                        verbose=FALSE, kitchensink=FALSE,
-                       preponly=FALSE, snoop=FALSE) {
+                       preponly=FALSE, snoop=FALSE,
+                       overrideXstart=NULL, overrideclip=FALSE) {
 # Internal Use Only!
 # This is the interface to the C code.
 
@@ -554,7 +585,7 @@ rmhEngine <- function(InfoList, ...,
 
   trend <- model$trend
   trendy <- model$internal$trendy
-  betalist <- model$internal$betalist
+#  betalist <- model$internal$betalist
   beta <- model$internal$beta
   iota <- model$internal$iota
   tmax <- model$internal$tmax
@@ -591,7 +622,7 @@ rmhEngine <- function(InfoList, ...,
 #############################################  
   
   if(!exists(".Random.seed"))
-    runif(1)
+    runif(1L)
 
   saved.seed <- .Random.seed
   
@@ -610,9 +641,9 @@ rmhEngine <- function(InfoList, ...,
              none= {
                # Poisson process 
                if(!mtype)
-                 rpoispp(intensity, win=w.sim, ...)
+                 rpoispp(intensity, win=w.sim, ..., warnwin=FALSE)
                else
-                 rmpoispp(intensity, win=w.sim, types=types)
+                 rmpoispp(intensity, win=w.sim, types=types, warnwin=FALSE)
              },
              n.total = {
                # Binomial/multinomial process with fixed total number of points
@@ -638,7 +669,7 @@ rmhEngine <- function(InfoList, ...,
     if(condtype != "none")
       Xsim <- superimpose(Xsim, x.condpp, W=w.state)
     # clip result to output window
-    Xclip <- Xsim[w.clip]
+    Xclip <- if(!overrideclip) Xsim[w.clip] else Xsim
     attr(Xclip, "info") <- InfoList
     return(Xclip)
   }
@@ -655,7 +686,7 @@ rmhEngine <- function(InfoList, ...,
 #### Build starting state
 
   npts.cond  <- if(condtype != "none") x.condpp$n else 0
-  npts.total <- npts.free + npts.cond
+#  npts.total <- npts.free + npts.cond
 
 #### FIRST generate the 'free' points
   
@@ -678,7 +709,7 @@ rmhEngine <- function(InfoList, ...,
              },
              x = {
                # x.start given
-               as.integer(marks(x.start, dfok=FALSE))-1
+               as.integer(marks(x.start, dfok=FALSE))-1L
              },
              stop("internal error: start$given unrecognised")
              )
@@ -707,14 +738,21 @@ rmhEngine <- function(InfoList, ...,
     x <- c(x.condpp$x, x)
     y <- c(x.condpp$y, y)
     if(mtype)
-      Cmarks <- c(as.integer(marks(x.condpp))-1, Cmarks)
+      Cmarks <- c(as.integer(marks(x.condpp))-1L, Cmarks)
+  }
+
+  if(!is.null(overrideXstart)) {
+    #' override the previous data
+    x <- overrideXstart$x
+    y <- overrideXstart$y
+    if(mtype) Cmarks <- as.integer(marks(overrideXstart))-1L
   }
 
 # decide whether to activate visual debugger
   if(snoop) {
     Xinit <- ppp(x, y, window=w.sim)
     if(mtype)
-      marks(Xinit) <- Cmarks + 1
+      marks(Xinit) <- Cmarks + 1L
     if(verbose) cat("\nCreating debugger environment..")
     snoopenv <- rmhSnoopEnv(Xinit=Xinit, Wclip=w.clip, R=reach(model))
     if(verbose) cat("Done.\n")
@@ -734,7 +772,7 @@ rmhEngine <- function(InfoList, ...,
     
   ipar <- model$C.ipar
   iparlist <- if(ncif == 1) list(ipar) else model$C.iparlist
-  iparlen <- unlist(lapply(iparlist, length))
+  iparlen <- lengths(iparlist)
 
   beta <- model$internal$beta
   
@@ -746,34 +784,22 @@ rmhEngine <- function(InfoList, ...,
   p       <- control$p
   q       <- control$q
   nrep    <- control$nrep
-  fixcode <- control$fixcode
-  fixing  <- control$fixing
+#  fixcode <- control$fixcode
+#  fixing  <- control$fixing
   fixall  <- control$fixall
   nverb   <- control$nverb
   saving  <- control$saving
   nsave   <- control$nsave
   nburn   <- control$nburn
   track   <- control$track
-  
-  if(verbose)
-    cat("Proposal points...")
-           
-# If the pattern is multitype, generate the mark proposals (0 to ntypes-1)
-  Cmprop <- if(mtype) sample(Ctypes,nrep,TRUE,prob=ptypes) else 0
-		
-# Generate the ``proposal points'' in the expanded window.
-  xy <-
-    if(trendy)
-      rpoint.multi(nrep,trend,tmax,
-                   factor(Cmprop, levels=Ctypes),
-                   w.sim, ..., warn=FALSE)
-    else
-      runifpoint(nrep, w.sim, warn=FALSE)
-  xprop <- xy$x
-  yprop <- xy$y
+  thin    <- control$internal$thin
+  pstage  <- control$pstage %orifnull% "start"
+  if(pstage == "block" && !saving) pstage <- "start"
+  temper  <- FALSE
+  invertemp <- 1.0
 
   if(verbose)
-    cat("Start simulation.\n")
+    cat("Ready to simulate. ")
 
   storage.mode(ncif)   <- "integer"
   storage.mode(C.id)   <- "character"
@@ -781,8 +807,6 @@ rmhEngine <- function(InfoList, ...,
   storage.mode(ipar)    <- "double"
   storage.mode(iparlen) <- "integer"
   storage.mode(period) <- "double"
-  storage.mode(xprop)  <- storage.mode(yprop) <- "double"
-  storage.mode(Cmprop) <- "integer"
   storage.mode(ntypes) <- "integer"
   storage.mode(nrep)   <- "integer"
   storage.mode(p) <- storage.mode(q) <- "double"
@@ -792,12 +816,39 @@ rmhEngine <- function(InfoList, ...,
   storage.mode(fixall) <- "integer"
   storage.mode(npts.cond) <- "integer"
   storage.mode(track) <- "integer"
+  storage.mode(thin) <- "integer"
+  storage.mode(temper) <- "integer"
+  storage.mode(invertemp) <- "double"
 
+  if(pstage == "start" || !saving) {
+    #' generate all proposal points now.
+    if(verbose)
+      cat("Generating proposal points...")
+
+    #' If the pattern is multitype, generate the mark proposals (0 to ntypes-1)
+    Cmprop <- if(mtype) sample(Ctypes,nrep,TRUE,prob=ptypes) else 0
+    storage.mode(Cmprop) <- "integer"
+
+    #' Generate the ``proposal points'' in the expanded window.
+    xy <- if(trendy) {
+      rpoint.multi(nrep,trend,tmax,
+                   factor(Cmprop, levels=Ctypes),
+                   w.sim, ..., warn=FALSE)
+    } else runifpoint(nrep, w.sim, warn=FALSE)
+    xprop <- xy$x
+    yprop <- xy$y
+    storage.mode(xprop)  <- storage.mode(yprop) <- "double"
+  }
+  
   if(!saving) {
     # ////////// Single block /////////////////////////////////
+
     nrep0 <- 0
     storage.mode(nrep0)  <- "integer"
+
     # Call the Metropolis-Hastings C code:
+    if(verbose)
+      cat("Running Metropolis-Hastings.\n")
     out <- .Call("xmethas",
                  ncif,
                  C.id,
@@ -815,14 +866,17 @@ rmhEngine <- function(InfoList, ...,
                  npts.cond,
                  fixall,
                  track,
-                 snoopenv)
-#                 PACKAGE="spatstat")
+                 thin,
+                 snoopenv,
+                 temper,
+                 invertemp,
+                 PACKAGE="spatstat")
   
     # Extract the point pattern returned from C
-    X <- ppp(x=out[[1]], y=out[[2]], window=w.state, check=FALSE)
+    X <- ppp(x=out[[1L]], y=out[[2L]], window=w.state, check=FALSE)
     if(mtype) {
       # convert integer marks from C to R
-      marx <- factor(out[[3]], levels=0:(ntypes-1))
+      marx <- factor(out[[3L]], levels=0:(ntypes-1))
       # then restore original type levels
       levels(marx) <- types
       # glue to points
@@ -830,7 +884,7 @@ rmhEngine <- function(InfoList, ...,
     }
 
     # Now clip the pattern to the ``clipping'' window:
-    if(!control$expand$force.noexp)
+    if(!overrideclip && !control$expand$force.noexp)
       X <- X[w.clip]
 
     # Extract transition history:
@@ -852,10 +906,10 @@ rmhEngine <- function(InfoList, ...,
   } else {
     # ////////// Multiple blocks /////////////////////////////////
     # determine length of each block of simulations
-    nblocks <- as.integer(1 + ceiling((nrep - nburn)/nsave))
-    block <- c(nburn, rep.int(nsave, nblocks-1))
+    nblocks <- as.integer(1L + ceiling((nrep - nburn)/nsave))
+    block <- c(nburn, rep.int(nsave, nblocks-1L))
     block[nblocks] <- block[nblocks] - (sum(block)-nrep)
-    block <- block[block >= 1]
+    block <- block[block >= 1L]
     nblocks <- length(block)
     blockend <- cumsum(block)
     # set up list to contain the saved point patterns
@@ -865,6 +919,8 @@ rmhEngine <- function(InfoList, ...,
     yprev <- y
     Cmarksprev <- Cmarks
     #
+    thinFALSE <- as.integer(FALSE)
+    storage.mode(thinFALSE) <- "integer"
     # ................ loop .........................
     for(I in 1:nblocks) {
       # number of iterations for this block
@@ -873,14 +929,41 @@ rmhEngine <- function(InfoList, ...,
       # number of previous iterations
       nrep0 <- if(I == 1) 0 else blockend[I-1]
       storage.mode(nrep0)  <- "integer"
-      # proposals
-      seqI <- 1:nrepI
-      xpropI <- xprop[seqI]
-      ypropI <- yprop[seqI]
-      CmpropI <- Cmprop[seqI]
-      storage.mode(xpropI) <- storage.mode(ypropI) <- "double"
-      storage.mode(CmpropI) <- "integer"
-      # call
+      # Generate or extract proposals
+      switch(pstage,
+             start = {
+               #' extract proposals from previously-generated vectors
+               if(verbose)
+                 cat("Extracting proposal points...")
+               seqI <- 1:nrepI
+               xpropI <- xprop[seqI]
+               ypropI <- yprop[seqI]
+               CmpropI <- Cmprop[seqI]
+               storage.mode(xpropI) <- storage.mode(ypropI) <- "double"
+               storage.mode(CmpropI) <- "integer"
+             },
+             block = {
+               # generate 'nrepI' random proposals
+               if(verbose)
+                 cat("Generating proposal points...")
+               #' If the pattern is multitype, generate the mark proposals 
+               CmpropI <- if(mtype) sample(Ctypes,nrepI,TRUE,prob=ptypes) else 0
+               storage.mode(CmpropI) <- "integer"
+               #' Generate the ``proposal points'' in the expanded window.
+               xy <- if(trendy) {
+                 rpoint.multi(nrepI,trend,tmax,
+                              factor(Cmprop, levels=Ctypes),
+                              w.sim, ..., warn=FALSE)
+               } else runifpoint(nrepI, w.sim, warn=FALSE)
+               xpropI <- xy$x
+               ypropI <- xy$y
+               storage.mode(xpropI)  <- storage.mode(ypropI) <- "double"
+             })
+      # no thinning in subsequent blocks
+      if(I > 1) thin <- thinFALSE
+      #' call
+      if(verbose)
+        cat("Running Metropolis-Hastings.\n")
       out <- .Call("xmethas",
                    ncif,
                    C.id,
@@ -898,13 +981,16 @@ rmhEngine <- function(InfoList, ...,
                    npts.cond,
                    fixall,
                    track,
-                   snoopenv)
-#                   PACKAGE="spatstat")
+                   thin,
+                   snoopenv,
+                   temper,
+                   invertemp,
+                   PACKAGE = "spatstat")
       # Extract the point pattern returned from C
-      X <- ppp(x=out[[1]], y=out[[2]], window=w.state, check=FALSE)
+      X <- ppp(x=out[[1L]], y=out[[2L]], window=w.state, check=FALSE)
       if(mtype) {
         # convert integer marks from C to R
-        marx <- factor(out[[3]], levels=0:(ntypes-1))
+        marx <- factor(out[[3L]], levels=0:(ntypes-1))
         # then restore original type levels
         levels(marx) <- types
         # glue to points
@@ -912,7 +998,7 @@ rmhEngine <- function(InfoList, ...,
       }
       
       # Now clip the pattern to the ``clipping'' window:
-      if(!control$expand$force.noexp)
+      if(!overrideclip && !control$expand$force.noexp)
         X <- X[w.clip]
 
       # commit to list
@@ -938,23 +1024,25 @@ rmhEngine <- function(InfoList, ...,
       }
 
       # update 'previous state'
-      xprev <- out[[1]]
-      yprev <- out[[2]]
+      xprev <- out[[1L]]
+      yprev <- out[[2L]]
       Cmarksprev <- if(!mtype) 0 else out[[3]]
       storage.mode(xprev) <- storage.mode(yprev) <- "double"
       storage.mode(Cmarksprev) <- "integer"
 
-      # discard used proposals
-      xprop <- xprop[-seqI]
-      yprop <- yprop[-seqI]
-      Cmprop <- Cmprop[-seqI]
+      if(pstage == "start") {
+        #' discard used proposals
+        xprop <- xprop[-seqI]
+        yprop <- yprop[-seqI]
+        Cmprop <- Cmprop[-seqI]
+      }
     }
     # .............. end loop ...............................
     
     # Result of simulation is final state 'X'
     # Tack on the list of intermediate states
     names(Xlist) <- paste("Iteration", as.integer(blockend), sep="_")
-    attr(X, "saved") <- as.listof(Xlist)
+    attr(X, "saved") <- as.solist(Xlist)
   }
 
 # Append to the result information about how it was generated.
@@ -973,7 +1061,7 @@ rmhEngine <- function(InfoList, ...,
 
 summarise.trend <- local({
   # main function
-  summarise.trend <- function(trend, w, a=area.owin(w)) {
+  summarise.trend <- function(trend, w, a=area(w)) {
     tlist <- if(is.function(trend) || is.im(trend)) list(trend) else trend
     return(lapply(tlist, summarise1, w=w, a=a))
   }
@@ -985,8 +1073,8 @@ summarise.trend <- local({
     } else {
       Z  <- as.im(x, w)[w, drop=FALSE]
       ran <- range(Z)
-      mini <- ran[1]
-      maxi <- ran[2]
+      mini <- ran[1L]
+      maxi <- ran[2L]
       integ <- integral.im(Z)
     }
     return(list(min=mini, max=maxi, integral=integ))
