@@ -2,7 +2,7 @@
 #
 #    saturated.S
 #
-#    $Revision: 1.6 $	$Date: 2007/01/11 03:36:02 $
+#    $Revision: 1.8 $	$Date: 2015/10/21 09:06:57 $
 #
 #    Saturated pairwise process with user-supplied potential
 #
@@ -49,3 +49,11 @@ Saturated <- function(pot, name) {
   class(out) <- "interact"
   return(out)
 }
+
+Saturated <-
+    intermaker(Saturated,
+               list(creator="Saturated",
+                    name="saturated process with user-defined potential",
+                    par=formals(Saturated),
+                    parnames=list("the potential",
+                        "the name of the interaction")))
