@@ -3,7 +3,7 @@
 #
 #  Smooth the marks of a point pattern
 # 
-#  $Revision: 1.35 $  $Date: 2016/11/13 01:55:53 $
+#  $Revision: 1.44 $  $Date: 2017/08/16 05:40:56 $
 #
 
 smooth.ppp <- function(X, ..., weights=rep(1, npoints(X)), at="pixels") {
@@ -22,14 +22,34 @@ Smooth.solist <- function(X, ...) {
 
 Smooth.ppp <- function(X, sigma=NULL, ...,
                        weights=rep(1, npoints(X)), at="pixels",
-                       edge=TRUE, diggle=FALSE) {
+                       edge=TRUE, diggle=FALSE, geometric=FALSE) {
   verifyclass(X, "ppp")
-  if(!is.marked(X, dfok=TRUE))
-    stop("X should be a marked point pattern")
+  if(!is.marked(X, dfok=TRUE, na.action="fatal"))
+    stop("X should be a marked point pattern", call.=FALSE)
   X <- coerce.marks.numeric(X)
+  if(!all(is.finite(as.matrix(marks(X)))))
+    stop("Some mark values are Inf, NaN or NA", call.=FALSE)
   at <- pickoption("output location type", at,
                    c(pixels="pixels",
                      points="points"))
+  ## weights
+  weightsgiven <- !missing(weights) && !is.null(weights) 
+  if(weightsgiven) {
+    # convert to numeric
+    if(is.im(weights)) {
+      weights <- safelookup(weights, X) # includes warning if NA
+    } else if(is.expression(weights)) 
+      weights <- eval(weights, envir=as.data.frame(X), enclos=parent.frame())
+    if(length(weights) == 0)
+      weightsgiven <- FALSE
+  }
+  if(weightsgiven) {
+    check.nvector(weights, npoints(X))
+  } else weights <- NULL
+  ## geometric mean smoothing
+  if(geometric) 
+    return(ExpSmoothLog(X, sigma=sigma, ..., at=at,
+                        weights=weights, edge=edge, diggle=diggle))
   ## determine smoothing parameters
   ker <- resolve.2D.kernel(sigma=sigma, ...,
                            x=X, bwfun=bw.smoothppp, allow.zero=TRUE)
@@ -54,19 +74,6 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
     }
     return(Y)
   }
-  ## weights
-  weightsgiven <- !missing(weights) && !is.null(weights) 
-  if(weightsgiven) {
-    if(is.im(weights)) {
-      weights <- safelookup(weights, X) # includes warning if NA
-    } else if(is.expression(weights)) 
-      weights <- eval(weights, envir=as.data.frame(X), enclos=parent.frame())
-    if(length(weights) == 0 || (!is.null(dim(weights)) && nrow(weights) == 0))
-      weightsgiven <- FALSE
-  }
-  if(weightsgiven) {
-    check.nvector(weights, npoints(X))
-  } else weights <- NULL
 
   if(diggle) {
     ## absorb Diggle edge correction into weights vector
@@ -189,7 +196,7 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
                  if(!is.matrix(numerators))
                    numerators <- matrix(numerators, ncol=1)
                  ratio <- numerators/denominator
-                 if(any(badpoints <- apply(!is.finite(ratio), 1, any))) {
+                 if(any(badpoints <- matrowany(!is.finite(ratio)))) {
                    whichnnX <- nnwhich(X)
                    ratio[badpoints,] <-
                      as.matrix(marx[whichnnX[badpoints], , drop=FALSE])
@@ -340,7 +347,8 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                v       = as.double(vv),
                self    = as.integer(!leaveoneout),
                rmaxi   = as.double(cutoff),
-               result  = as.double(double(npts)))
+               result  = as.double(double(npts)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[oo] <- zz$result
     } else {
       wtsort <- weights[oo]
@@ -352,7 +360,8 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                self    = as.integer(!leaveoneout),
                rmaxi   = as.double(cutoff),
                weight  = as.double(wtsort),
-               result  = as.double(double(npts)))
+               result  = as.double(double(npts)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[oo] <- zz$result
     }
     if(any(nbg <- (is.infinite(result) | is.nan(result)))) {
@@ -388,7 +397,8 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  self    = as.integer(!leaveoneout),
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result
       } else {
         wtsort <- weights[oo]
@@ -401,7 +411,8 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  sig     = as.double(sd),
                  weight  = as.double(wtsort),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result
       }
     } else {
@@ -417,7 +428,8 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  self    = as.integer(!leaveoneout),
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result
       } else {
         wtsort <- weights[oo]
@@ -430,7 +442,8 @@ smoothpointsEngine <- function(x, values, sigma, ...,
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
                  weight  = as.double(wtsort),
-                 result  = as.double(double(npts)))
+                 result  = as.double(double(npts)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[oo] <- zz$result
       }
     }
@@ -517,10 +530,14 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
                        hmin=NULL, hmax=NULL, warn=TRUE) {
   stopifnot(is.ppp(X))
   stopifnot(is.marked(X))
+  X <- coerce.marks.numeric(X)
   # rearrange in ascending order of x-coordinate (for C code)
   X <- X[fave.order(X$x)]
   #
   marx <- marks(X)
+  dimmarx <- dim(marx)
+  if(!is.null(dimmarx))
+    marx <- as.matrix(as.data.frame(marx))
   # determine a range of bandwidth values
 #  n <- npoints(X)
   if(is.null(hmin) || is.null(hmax)) {
@@ -548,7 +565,9 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
   # compute cross-validation criterion
   for(i in seq_len(nh)) {
     yhat <- Smooth(X, sigma=h[i], at="points", leaveoneout=TRUE,
-                       sorted=TRUE)
+                   sorted=TRUE)
+    if(!is.null(dimmarx))
+      yhat <- as.matrix(as.data.frame(yhat))
     cv[i] <- mean((marx - yhat)^2)
   }
 
@@ -567,7 +586,8 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
   result <- bw.optim(cv, h, iopt,
                      hname="sigma",
                      creator="bw.smoothppp",
-                     criterion="Least Squares Cross-Validation")
+                     criterion="Least Squares Cross-Validation",
+                     unitname=unitname(X))
   return(result)
 }
 
@@ -681,7 +701,8 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                vd      = as.double(vd),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)))
+               result  = as.double(double(nquery)),
+               PACKAGE = "spatstat")
       if(sorted) result <- zz$result else result[ooq] <- zz$result
     } else {
       wtsort <- weights[ood]
@@ -696,7 +717,8 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                wd      = as.double(wtsort),
                rmaxi   = as.double(cutoff),
                sig     = as.double(sd),
-               result  = as.double(double(nquery)))
+               result  = as.double(double(nquery)),
+               PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[ooq] <- zz$result
       }
     } else {
@@ -714,7 +736,8 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                  vd      = as.double(vd),
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(nquery)))
+                 result  = as.double(double(nquery)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[ooq] <- zz$result
       } else {
         wtsort <- weights[ood]
@@ -729,7 +752,8 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
                  wd      = as.double(wtsort),
                  rmaxi   = as.double(cutoff),
                  sinv    = as.double(flatSinv),
-                 result  = as.double(double(nquery)))
+                 result  = as.double(double(nquery)),
+                 PACKAGE = "spatstat")
         if(sorted) result <- zz$result else result[ooq] <- zz$result
       }
     }
@@ -744,3 +768,51 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
   return(result)
 }
 
+ExpSmoothLog <- function(X, ..., at=c("pixels", "points"), weights=NULL) {
+  verifyclass(X, "ppp")
+  at <- match.arg(at)
+  if(!is.null(weights)) 
+    check.nvector(weights, npoints(X))
+  X <- coerce.marks.numeric(X)
+  marx <- marks(X)
+  d <- dim(marx)
+  if(!is.null(d) && d[2] > 1) {
+    switch(at,
+           points = {
+             Z <- lapply(unstack(X), ExpSmoothLog, ...,
+                         at=at, weights=weights)
+             Z <- do.call(data.frame(Z))
+           },
+           pixels = {
+             Z <- solapply(unstack(X), ExpSmoothLog, ...,
+                           at=at, weights=weights)
+           })
+    return(Z)
+  }
+  # vector or single column of numeric marks
+  v <- as.numeric(marx)
+  vmin <- min(v)
+  if(vmin < 0) stop("Negative values in geometric mean smoothing",
+                       call.=FALSE)
+  Y <- X %mark% log(v)
+  if(vmin > 0) {
+    Z <- Smooth(Y, ..., at=at, weights=weights)
+  } else {
+    yok <- is.finite(marks(Y))
+    YOK <- Y[yok]
+    weightsOK <- if(is.null(weights)) NULL else weights[yok]
+    switch(at,
+           points = {
+             Z <- rep(-Inf, npoints(X))
+             Z[yok] <- Smooth(YOK, ..., at=at, weights=weightsOK)
+           },
+           pixels = {
+             isfinite <- nnmark(Y %mark% yok, ...)
+             support <- solutionset(isfinite)
+             Window(YOK) <- support
+             Z <- as.im(-Inf, W=Window(Y), ...)
+             Z[support] <- Smooth(YOK, ..., at=at, weights=weightsOK)[]
+           })
+  }
+  return(exp(Z))
+}
