@@ -2,7 +2,7 @@
 #
 #    strauss.R
 #
-#    $Revision: 2.36 $	$Date: 2016/04/25 02:34:40 $
+#    $Revision: 2.37 $	$Date: 2017/06/05 10:31:58 $
 #
 #    The Strauss process
 #
@@ -90,13 +90,30 @@ Strauss <- local({
          return(y)
        },
        delta2 = function(X, inte, correction, ..., sparseOK=FALSE) {
-         if(!(correction %in% c("border", "none")))
-           return(NULL)
          r <- inte$par$r
          X <- as.ppp(X) # algorithm is the same for data and dummy points
          nX <- npoints(X)
-         cl <- closepairs(X, r, what="indices")
-         v <- sparseMatrix(i=cl$i, j=cl$j, x=1,
+	 switch(correction,
+	        none = ,
+		border = {
+                  cl <- closepairs(X, r, what="indices")
+		  weight <- 1
+		},
+		isotropic = ,
+		Ripley = {
+                  cl <- closepairs(X, r, what="ijd")
+		  weight <- edge.Ripley(X[cl$i], cl$d)
+		},
+		translate = {
+                  cl <- closepairs(X, r, what="all")
+		  weight <- edge.Trans(dx = cl$dx,
+		   	    	       dy = cl$dy,
+		                       W = Window(X),
+				       paired=TRUE)
+		},
+		return(NULL)
+		)
+         v <- sparseMatrix(i=cl$i, j=cl$j, x=as.numeric(weight),
                            dims=c(nX, nX))
          if(!sparseOK)
            v <- as.matrix(v)
@@ -124,7 +141,7 @@ strausscounts <- function(U, X, r, EqualPairs=NULL) {
   # subtract counts of identical pairs
   if(length(EqualPairs) > 0) {
     nU <- npoints(U)
-    idcount <- as.integer(table(factor(EqualPairs[,2], levels=1:nU)))
+    idcount <- as.integer(table(factor(EqualPairs[,2L], levels=1:nU)))
     answer <- answer - idcount
   }
   return(answer)
@@ -151,7 +168,8 @@ crosspaircounts <- function(X, Y, r) {
             xtarget  = as.double(Ysort$x),
             ytarget  = as.double(Ysort$y),
             rrmax    = as.double(r),
-            counts   = as.integer(integer(nX)))
+            counts   = as.integer(integer(nX)),
+            PACKAGE = "spatstat")
   answer <- integer(nX)
   answer[oX] <- out$counts
   return(answer)
@@ -172,7 +190,8 @@ closepaircounts <- function(X, r) {
             x      = as.double(Xsort$x),
             y      = as.double(Xsort$y),
             rmaxi  = as.double(r),
-            counts = as.integer(integer(nX)))
+            counts = as.integer(integer(nX)),
+            PACKAGE = "spatstat")
   answer <- integer(nX)
   answer[oX] <- out$counts
   return(answer)
