@@ -2,7 +2,7 @@
 #
 #    triplets.R
 #
-#    $Revision: 1.13 $	$Date: 2013/12/17 03:59:09 $
+#    $Revision: 1.17 $	$Date: 2016/12/30 01:44:07 $
 #
 #    The triplets interaction
 #
@@ -78,7 +78,7 @@ Triplets <- local({
     tcount <- .rowSums(tcount, nrow(tcount), ncol(tcount))
     # select triangles consisting only of data points
     triX <- matrix(mapUX[tri], nrow=nrow(tri))
-    isX <- apply(!is.na(triX), 1, all)
+    isX <- matrowall(!is.na(triX))
     triX <- triX[isX, , drop=FALSE]
     #
     if(nrow(triX) > 0) {
@@ -122,7 +122,7 @@ Triplets <- local({
            gamma <- exp(loggamma)
            return(list(param=list(gamma=gamma),
                        inames="interaction parameter gamma",
-                       printable=round(gamma,4)))
+                       printable=dround(gamma)))
          },
          valid = function(coeffs, self) {
            gamma <- ((self$interpret)(coeffs, self))$param$gamma
@@ -133,7 +133,7 @@ Triplets <- local({
          },
          irange = function(self, coeffs=NA, epsilon=0, ...) {
            r <- self$par$r
-           if(any(is.na(coeffs)))
+           if(anyNA(coeffs))
              return(r)
            loggamma <- coeffs[1]
            if(abs(loggamma) <= epsilon)
@@ -148,6 +148,8 @@ Triplets <- local({
   Triplets <- function(r) {
     instantiate.interact(BlankTripletsObject, list(r=r))
   }
+  Triplets <- intermaker(Triplets, BlankTripletsObject)
+  
   Triplets
 })
 
