@@ -3,7 +3,7 @@
 #
 #	A class 'owin' to define the "observation window"
 #
-#	$Revision: 4.146 $	$Date: 2014/04/21 03:32:08 $
+#	$Revision: 4.176 $	$Date: 2017/09/11 16:35:38 $
 #
 #
 #	A window may be either
@@ -68,9 +68,18 @@
     "Transpose matrices to get the standard presentation in R",
     "Example: image(result$xcol,result$yrow,t(result$d))")
 
-owin <- function(xrange=c(0,1), yrange=c(0,1),
+owin <- local({
+
+  isxy <- function(x) { (is.matrix(x) || is.data.frame(x)) && ncol(x) == 2 }
+  asxy <- function(xy) { list(x=xy[,1], y=xy[,2]) }
+
+  owin <- function(xrange=c(0,1), yrange=c(0,1),
                  ..., poly=NULL, mask=NULL, unitname=NULL, xy=NULL) {
 
+  # trap a common abuse of syntax
+  if(nargs() == 1 && !missing(xrange) && is.owin(xrange))
+    return(xrange)
+  
   unitname <- as.units(unitname)
 
   ## Exterminate ambiguities
@@ -104,9 +113,9 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
   if(!poly.given && !mask.given) {
     ######### rectangle #################
     if(check) {
-      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2] < xrange[1])
+      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
         stop("xrange should be a vector of length 2 giving (xmin, xmax)")
-      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2] < yrange[1])
+      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
         stop("yrange should be a vector of length 2 giving (ymin, ymax)")
     }
     w <- list(type="rectangle", xrange=xrange, yrange=yrange, units=unitname)
@@ -118,9 +127,9 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
     if(length(poly) == 0) {
       # empty polygon
       if(check) {
-        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2] < xrange[1])
+        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] < xrange[1L])
           stop("xrange should be a vector of length 2 giving (xmin, xmax)")
-        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2] < yrange[1])
+        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] < yrange[1L])
           stop("yrange should be a vector of length 2 giving (ymin, ymax)")
       }
       w <- list(type="polygonal", xrange=xrange, yrange=yrange,
@@ -129,8 +138,6 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       return(w)
     }
     # convert matrix or data frame to list(x,y)
-    isxy <- function(x) { (is.matrix(x) || is.data.frame(x)) && ncol(x) == 2 }
-    asxy <- function(xy) { list(x=xy[,1], y=xy[,2]) }
     if(isxy(poly)) {
       poly <- asxy(poly)
     } else if(is.list(poly) && all(unlist(lapply(poly, isxy)))) {
@@ -151,7 +158,7 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       # single boundary polygon
       bdry <- list(poly)
       if(check || calculate) {
-        w.area <- area.xypolygon(poly)
+        w.area <- Area.xypolygon(poly)
         if(w.area < 0)
           stop(paste("Area of polygon is negative -",
                      "maybe traversed in wrong direction?"))
@@ -160,7 +167,7 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       # multiple boundary polygons
       bdry <- poly
       if(check || calculate) {
-        w.area <- unlist(lapply(poly, area.xypolygon))
+        w.area <- unlist(lapply(poly, Area.xypolygon))
         if(sum(w.area) < 0)
           stop(paste("Area of window is negative;\n",
                      "check that all polygons were traversed",
@@ -168,21 +175,21 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       }
     }
 
-    actual.xrange <- range(unlist(lapply(bdry, function(a) a$x)))
+    actual.xrange <- range(unlist(lapply(bdry, getElement, name="x")))
     if(missing(xrange))
       xrange <- actual.xrange
     else if(check) {
-      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2] <= xrange[1])
+      if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] <= xrange[1L])
         stop("xrange should be a vector of length 2 giving (xmin, xmax)")
       if(!all(xrange == range(c(xrange, actual.xrange))))
         stop("polygon's x coordinates outside xrange")
     }
     
-    actual.yrange <- range(unlist(lapply(bdry, function(a) a$y)))
+    actual.yrange <- range(unlist(lapply(bdry, getElement, name="y")))
     if(missing(yrange))
       yrange <- actual.yrange
     else if(check) {
-      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2] <= yrange[1])
+      if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] <= yrange[1L])
         stop("yrange should be a vector of length 2 giving (ymin, ymax)")
       if(!all(yrange == range(c(yrange, actual.yrange))))
       stop("polygon's y coordinates outside yrange")
@@ -208,24 +215,43 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       }
     }
     if(check && fix) {
-      ## repair polygon data by invoking polyclip
-      ##        to intersect polygon with bounding rectangle
-      ##        (Streamlined version of intersect.owin)
-      ww <- lapply(bdry, reverse.xypolygon)
-      rr <- list(list(x=xrange[c(1,2,2,1)], y=yrange[c(2,2,1,1)]))
-      bb <- polyclip::polyclip(ww, rr, "intersection",
-                               fillA="nonzero", fillB="nonzero")
-      ## ensure correct polarity
-      totarea <- sum(unlist(lapply(bb, area.xypolygon)))
-      if(totarea < 0)
-        bb <- lapply(bb, reverse.xypolygon)
-      w$bdry <- bb
+      if(length(bdry) == 1 &&
+         length(bx <- bdry[[1L]]$x) == 4 &&
+         length(unique(bx)) == 2 &&
+         length(unique(bdry[[1L]]$y)) == 2) {
+        ## it's really a rectangle
+        if(Area.xypolygon(bdry[[1L]]) < 0)
+          w$bdry <- lapply(bdry, reverse.xypolygon)
+      } else {
+        ## repair polygon data by invoking polyclip
+        ##        to intersect polygon with larger-than-bounding rectangle
+        ##        (Streamlined version of intersect.owin)
+        ww <- lapply(bdry, reverse.xypolygon)
+        xrplus <- mean(xrange) + c(-1,1) * diff(xrange)
+        yrplus <- mean(yrange) + c(-1,1) * diff(yrange)
+        bignum <- (.Machine$integer.max^2)/2
+        epsclip <- max(diff(xrange), diff(yrange))/bignum
+        rr <- list(list(x=xrplus[c(1,2,2,1)], y=yrplus[c(2,2,1,1)]))
+        bb <- polyclip::polyclip(ww, rr, "intersection",
+                                 fillA="nonzero", fillB="nonzero", eps=epsclip)
+        ## ensure correct polarity
+        totarea <- sum(unlist(lapply(bb, Area.xypolygon)))
+        if(totarea < 0)
+          bb <- lapply(bb, reverse.xypolygon)
+        w$bdry <- bb
+      }
     }
     return(w)
-    
   } else if(mask.given) {
     ######### digital mask #####################
-    
+
+    if(is.data.frame(mask) &&
+       ncol(mask) %in% c(2,3) &&
+       sum(sapply(mask, is.numeric)) == 2) {
+      # data frame with 2 columns of coordinates
+      return(as.owin(W=mask, xy=xy))
+    }
+      
     if(!is.matrix(mask))
       stop(paste(sQuote("mask"), "must be a matrix"))
     if(!is.logical(mask))
@@ -253,8 +279,8 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       if(!evenly.spaced(yrow))
         stop("xy$y is not evenly spaced")
       # determine other parameters
-      xstep <- diff(xcol)[1]
-      ystep <- diff(yrow)[1]
+      xstep <- diff(xcol)[1L]
+      ystep <- diff(yrow)[1L]
       if(missing(xrange) && missing(yrange)) {
         xrange <- range(xcol) + c(-1,1) * xstep/2
         yrange <- range(yrow) + c(-1,1) * ystep/2
@@ -266,15 +292,15 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
         xrange <- c(0,nc)
         yrange <- c(0,nr)
       } else if(check) {
-        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2] <= xrange[1])
+        if(!is.vector(xrange) || length(xrange) != 2 || xrange[2L] <= xrange[1L])
           stop("xrange should be a vector of length 2 giving (xmin, xmax)")
-        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2] <= yrange[1])
+        if(!is.vector(yrange) || length(yrange) != 2 || yrange[2L] <= yrange[1L])
           stop("yrange should be a vector of length 2 giving (ymin, ymax)")
       }
       xstep <- diff(xrange)/nc
       ystep <- diff(yrange)/nr
-      xcol  <- seq(from=xrange[1]+xstep/2, to=xrange[2]-xstep/2, length.out=nc)
-      yrow  <- seq(from=yrange[1]+ystep/2, to=yrange[2]-ystep/2, length.out=nr)
+      xcol  <- seq(from=xrange[1L]+xstep/2, to=xrange[2L]-xstep/2, length.out=nc)
+      yrow  <- seq(from=yrange[1L]+ystep/2, to=yrange[2L]-ystep/2, length.out=nr)
     }
 
     out <- list(type     = "mask",
@@ -295,6 +321,9 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
   NULL
 }
 
+  owin
+})
+
 #
 #-----------------------------------------------------------------------------
 #
@@ -360,33 +389,68 @@ as.owin.tess <- function(W, ..., fatal=TRUE) {
   return(W$window)
 }
 
-as.owin.data.frame <- function(W, ..., fatal=TRUE) {
+as.owin.data.frame <- function(W, ..., step, fatal=TRUE) {
   if(!verifyclass(W, "data.frame", fatal=fatal))
     return(NULL)
-  if(ncol(W) != 3) {
-    whinge <- "need exactly 3 columns of data"
+  if(missing(step)) {
+    xstep <- ystep <- NULL
+  } else {
+    step <- ensure2vector(step)
+    xstep <- step[1L]
+    ystep <- step[2L]
+  }
+  if(!(ncol(W) %in% c(2,3))) {
+    whinge <- "need exactly 2 or 3 columns of data"
     if(fatal) stop(whinge)
     warning(whinge)
     return(NULL)
   }
-  mch <- match(c("x", "y"), names(W))
-  if(!any(is.na(mch))) {
-    ix <- mch[1]
-    iy <- mch[2]
-    iz <- (1:3)[-mch]
-  } else {
-    ix <- 1
-    iy <- 2
-    iz <- 3
-  }
+  if(twocol <- (ncol(W) == 2)) {
+    # assume data is a list of TRUE pixels
+    W <- cbind(W, TRUE)
+  } 
+  mch <- matchNameOrPosition(c("x", "y", "z"), names(W))
+  ix <- mch[1L]
+  iy <- mch[2L]
+  iz <- mch[3L]
   df <- data.frame(x=W[,ix], y=W[,iy], z=as.logical(W[,iz]))
-  # convert data frame (x,y,z) to logical matrix
-  m <- with(df, tapply(z, list(y, x), any))
-  # extract pixel coordinates
-  xy <- with(df, list(x=sort(unique(x)), y=sort(unique(y))))
-  # make binary mask
-  out <- owin(mask=m, xy=xy)
-  return(out)
+  with(df, {
+    xx <- sort(unique(x))
+    yy <- sort(unique(y))
+    jj <- match(x, xx)
+    ii <- match(y, yy)
+    ## make logical matrix (for incomplete x, y sequence)
+    ok <- checkbigmatrix(length(xx), length(yy), fatal=fatal)
+    if(!ok) return(NULL)
+    mm <- matrix(FALSE, length(yy), length(xx))
+    mm[cbind(ii,jj)] <- z
+    ## ensure xx and yy are complete equally-spaced sequences
+    fx <- fillseq(xx, step=xstep)
+    fy <- fillseq(yy, step=ystep)
+    xcol <- fx[[1L]]
+    yrow <- fy[[1L]]
+    ## trap very large matrices
+    ok <- checkbigmatrix(length(xcol), length(yrow), fatal=fatal)
+    if(!ok) return(NULL)
+    ## mapping from xx to xcol, yy to yrow
+    jjj <- fx[[2L]]
+    iii <- fy[[2L]]
+    ## make logical matrix for full sequence
+    m <- matrix(FALSE, length(yrow), length(xcol))
+    m[iii,jjj] <- mm
+    ## make binary mask
+    out <- owin(mask=m, xy=list(x=xcol, y=yrow))
+    ## warn if area fraction is small: may be a misuse of as.owin
+    if(twocol) {
+      pcarea <- 100 * nrow(df)/prod(dim(m))
+      if(pcarea < 1) 
+       warning(paste("Window occupies only",
+                     paste0(signif(pcarea, 2), "%"),
+                     "of frame area. Did you mean owin(poly=df) ?"),
+               call.=FALSE)
+    }
+    return(out)
+  })
 }
 
 as.owin.default <- function(W, ..., fatal=TRUE) {
@@ -397,6 +461,13 @@ as.owin.default <- function(W, ..., fatal=TRUE) {
   ##	a structure with entries xl, xu, yl, yu
   ##	an object with attribute "bbox"
 
+  if(inherits(W, "box3")) {
+    #' cannot be flattened
+    if(fatal)
+      stop("3D box cannot be converted to a 2D window")
+    return(NULL)
+  }
+  
   if(checkfields(W, c("xrange", "yrange"))) {
     Z <- owin(W$xrange, W$yrange)
     return(Z)
@@ -423,6 +494,14 @@ as.owin.default <- function(W, ..., fatal=TRUE) {
 #-----------------------------------------------------------------------------
 #
 #
+Frame <- function(X) { UseMethod("Frame") }
+
+"Frame<-" <- function(X, value) { UseMethod("Frame<-") }
+
+Frame.default <- function(X) { as.rectangle(X) }
+
+## .........................................................
+
 as.rectangle <- function(w, ...) {
   if(inherits(w, "owin"))
     return(owin(w$xrange, w$yrange, unitname=unitname(w)))
@@ -442,21 +521,25 @@ as.rectangle <- function(w, ...) {
 as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
 #	eps:		   grid mesh (pixel) size
 #	dimyx:		   dimensions of pixel raster
-#       xy:                coordinates of pixel raster  
+#       xy:                coordinates of pixel raster
+  nonamedargs <- is.null(eps) && is.null(dimyx) && is.null(xy)
+  uname <- as.units(NULL)
   if(!missing(w) && !is.null(w)) {
-    if(is.matrix(w))
+    if(is.data.frame(w)) return(owin(mask=w, xy=xy))
+    if(is.matrix(w)) {
+      w <- as.data.frame(w)
+      colnames(w) <- c("x", "y")
       return(owin(mask=w, xy=xy))
+    }
     w <- as.owin(w)
     uname <- unitname(w)
   } else {
-    uname <- as.units(NULL)
     if(is.null(xy)) 
       stop("If w is missing, xy is required")
   }
   # If it's already a mask, and no other arguments specified,
   # just return it.
-  if(!missing(w) && w$type == "mask" &&
-     is.null(eps) && is.null(dimyx) && is.null(xy))
+  if(!missing(w) && w$type == "mask" && nonamedargs)
     return(w)
   
 ##########################
@@ -467,14 +550,14 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
 # First determine row & column dimensions
     if(!is.null(dimyx)) {
       dimyx <- ensure2vector(dimyx)
-      nr <- dimyx[1]
-      nc <- dimyx[2]
+      nr <- dimyx[1L]
+      nc <- dimyx[2L]
     } else {
     # use pixel size 'eps'
       if(!is.null(eps)) {
         eps <- ensure2vector(eps)
-        nc <- diff(w$xrange)/eps[1]
-        nr <- diff(w$yrange)/eps[2]
+        nc <- diff(w$xrange)/eps[1L]
+        nr <- diff(w$yrange)/eps[2L]
         if(nr < 1 || nc < 1)
           warning("pixel size parameter eps > size of window")
         nr <- ceiling(nr)
@@ -483,13 +566,21 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
     # use spatstat defaults
         np <- spatstat.options("npixel")
         if(length(np) == 1)
-          nr <- nc <- np[1]
+          nr <- nc <- np[1L]
         else {
-          nr <- np[2]  
-          nc <- np[1]
+          nr <- np[2L]  
+          nc <- np[1L]
         }
       }
     }
+    if((mpix <- (nr * nc)/1048576) >= 10) {
+      whinge <- paste("Creating",
+                      articlebeforenumber(mpix),
+                      paste0(round(mpix, 1), "-megapixel"),
+                      "window mask")
+      message(whinge)
+      warning(whinge, call.=FALSE)
+    }
     # Initialise mask with all entries TRUE
     rasta <- owin(w$xrange, w$yrange, mask=matrix(TRUE, nr, nc))
   } else {
@@ -514,6 +605,15 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
       # derive other parameters
       nr <- length(y)
       nc <- length(x)
+      # check size
+      if((mpix <- (nr * nc)/1048576) >= 10) {
+        whinge <- paste("Creating",
+                        articlebeforenumber(mpix),
+                        paste0(round(mpix, 1), "-megapixel"),
+                        "window mask")
+        message(whinge)
+        warning(whinge, call.=FALSE)
+      }
       # x and y pixel sizes
       dx <- diff(x)
       if(diff(range(dx)) > 0.01 * mean(dx))
@@ -535,15 +635,23 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
                     xstep    = xstep,
                     ystep    = ystep,
                     warnings = .Spatstat.Image.Warning,
-                    xcol    = seq(from=xr[1], to=xr[2], length.out=nc),
-                    yrow    = seq(from=yr[1], to=yr[2], length.out=nr),
+                    xcol    = seq(from=xr[1L], to=xr[2L], length.out=nc),
+                    yrow    = seq(from=yr[1L], to=yr[2L], length.out=nr),
                     m       = matrix(TRUE, nr, nc),
                     units   = uname)
       class(rasta) <- "owin"
     }
-    # window may be implicit in this case.
-    if(missing(w))
-      w <- owin(xrange, yrange)
+    if(missing(w)) {
+      # No more window information
+      out <- rasta
+      if(!(identical(x, xy$x) && identical(y, xy$y))) {
+        ## xy is an enumeration of the TRUE pixels
+        out$m[] <- FALSE
+        ij <- cbind(i=match(xy$y, y), j=match(xy$x, x))
+        out$m[ij] <- TRUE
+      }
+      return(out)
+    }
   }
 
 ################################  
@@ -558,8 +666,8 @@ as.mask <- function(w, eps=NULL, dimyx=NULL, xy=NULL) {
              yrow <- rasta$yrow
              wx <- w$xrange
              wy <- w$yrange
-             badrow <- which(yrow > wy[2] | yrow < wy[1])
-             badcol <- which(xcol > wx[2] | xcol < wx[1])
+             badrow <- which(yrow > wy[2L] | yrow < wy[1L])
+             badcol <- which(xcol > wx[2L] | xcol < wx[1L])
              out$m[badrow , ] <- FALSE
              out$m[ , badcol] <- FALSE
            }
@@ -586,7 +694,7 @@ as.matrix.owin <- function(x, ...) {
 #
 #-----------------------------------------------------------------------------
 #
-as.polygonal <- function(W) {
+as.polygonal <- function(W, repair=FALSE) {
   verifyclass(W, "owin")
   switch(W$type,
          rectangle = {
@@ -597,6 +705,8 @@ as.polygonal <- function(W) {
                        check=FALSE))
          },
          polygonal = {
+           if(repair)
+             W <- owin(poly=W$bdry, unitname=unitname(W))
            return(W)
          },
          mask = {
@@ -604,14 +714,19 @@ as.polygonal <- function(W) {
            M <- W$m
            nr <- nrow(M)
            notM <- !M
-           out <- NULL
            xcol <- W$xcol
            yrow <- W$yrow
            xbracket <- 1.1 * c(-1,1) * W$xstep/2
            ybracket <- 1.1 * c(-1,1) * W$ystep/2
+           ## determine resolution for polyclip operations
+           p <- list(x0 = xcol[1],
+                     y0 = yrow[1],
+                     eps = max(W$xstep, W$ystep)/(2^31))
            # identify runs of TRUE entries in each column
            start <- M & rbind(TRUE, notM[-nr, ])
            finish <- M & rbind(notM[-1, ], TRUE)
+           #' build result
+           out <- NULL
            for(j in 1:ncol(M)) {
              xj <- xcol[j]
              # identify start and end positions in column j
@@ -627,14 +742,29 @@ as.polygonal <- function(W) {
                  yfrom <- yrow[starts[k]]
                  yto   <- yrow[finishes[k]]
                  yk <- sort(c(yfrom,yto))
-                 # make rectangle
-                 recto <- owin(xj+xbracket,yk+ybracket)
-                 # add to result
-                 out <- union.owin(out, recto)
+                 #' make rectangle boundary in reversed orientation
+                 xrect <- xj + xbracket
+                 yrect <- yk + ybracket
+                 recto <- list(list(x = xrect[c(1,2,2,1)],
+                                    y = yrect[c(2,2,1,1)]))
+                 #' add to result
+                 if(is.null(out)) {
+                   out <- recto
+                 } else {
+                   out <- polyclip::polyclip(out, recto, "union",
+                                             fillA="nonzero", fillB="nonzero",
+                                             eps = p$eps,
+                                             x0  = p$x0,
+                                             y0  = p$y0)
+                 }
                }
-               unitname(out) <- unitname(W)
              }
            }
+           if(is.null(out)) return(emptywindow(Frame(W)))
+           totarea <- sum(sapply(out, Area.xypolygon))
+           if(totarea < 0)
+             out <- lapply(out, reverse.xypolygon)
+           out <- owin(poly=out, check=FALSE, unitname=unitname(W))
            return(out)
          }
          )
@@ -666,37 +796,48 @@ validate.mask <- function(w, fatal=TRUE) {
       return(FALSE)
   }
 }
-             
-raster.x <- function(w, drop=FALSE) {
+
+dim.owin <- function(x) { return(x$dim) } ## NULL unless it's a mask
+
+## internal use only:
+
+rasterx.mask <- function(w, drop=FALSE) {
   validate.mask(w)
-  m <- w$m
-  x <- w$xcol[col(m)]
-  x <- if(drop) x[m, drop=TRUE] else array(x, dim=w$dim)
+  x <- w$xcol[col(w)]
+  x <- if(drop) x[w$m, drop=TRUE] else array(x, dim=w$dim)
   return(x)
 }
 
-raster.y <- function(w, drop=FALSE) {
+rastery.mask <- function(w, drop=FALSE) {
   validate.mask(w)
-  m <- w$m
-  y <- w$yrow[row(m)]
-  y <- if(drop) y[m, drop=TRUE] else array(y, dim=w$dim)
+  y <- w$yrow[row(w)]
+  y <- if(drop) y[w$m, drop=TRUE] else array(y, dim=w$dim)
   return(y)
 }
 
-raster.xy <- function(w, drop=FALSE) {
-  list(x=as.numeric(raster.x(w, drop=drop)),
-       y=as.numeric(raster.y(w, drop=drop)))
+rasterxy.mask <- function(w, drop=FALSE) {
+  validate.mask(w)
+  x <- w$xcol[col(w)]
+  y <- w$yrow[row(w)]
+  if(drop) {
+    m <- w$m
+    x <- x[m, drop=TRUE] 
+    y <- y[m, drop=TRUE]
+  }
+  return(list(x=as.numeric(x),
+              y=as.numeric(y)))
 }
-  
+
+
 nearest.raster.point <- function(x,y,w, indices=TRUE) {
   stopifnot(is.mask(w) || is.im(w))
-  nr <- w$dim[1]
-  nc <- w$dim[2]
+  nr <- w$dim[1L]
+  nc <- w$dim[2L]
   if(length(x) == 0) {
     cc <- rr <- integer(0)
   } else {
-    cc <- 1 + round((x - w$xcol[1])/w$xstep)
-    rr <- 1 + round((y - w$yrow[1])/w$ystep)
+    cc <- 1 + round((x - w$xcol[1L])/w$xstep)
+    rr <- 1 + round((y - w$yrow[1L])/w$ystep)
     cc <- pmax.int(1,pmin.int(cc, nc))
     rr <- pmax.int(1,pmin.int(rr, nr))
   }
@@ -719,7 +860,6 @@ mask2df <- function(w) {
 #------------------------------------------------------------------
 		
 complement.owin <- function(w, frame=as.rectangle(w)) {
-  wname <- short.deparse(substitute(w))
   w <- as.owin(w)
 
   if(reframe <- !missing(frame)) {
@@ -745,17 +885,17 @@ complement.owin <- function(w, frame=as.rectangle(w)) {
            # bounding box, in anticlockwise order
            box <- list(x=w$xrange[c(1,2,2,1)],
                        y=w$yrange[c(1,1,2,2)])
-           boxarea <- area.xypolygon(box)
+           boxarea <- Area.xypolygon(box)
                  
            # first check whether one of the current boundary polygons
            # is the bounding box itself (with + sign)
            if(reframe)
              is.box <- rep.int(FALSE, length(bdry))
            else {
-             nvert <- unlist(lapply(bdry, function(a) { length(a$x) }))
-             area <- unlist(lapply(bdry, area.xypolygon))
+             nvert <- lengths(lapply(bdry, getElement, name="x"))
+             areas <- sapply(bdry, Area.xypolygon)
              boxarea.mineps <- boxarea * (0.99999)
-             is.box <- (nvert == 4 & area >= boxarea.mineps)
+             is.box <- (nvert == 4 & areas >= boxarea.mineps)
              if(sum(is.box) > 1)
                stop("Internal error: multiple copies of bounding box")
              if(all(is.box)) {
@@ -800,10 +940,10 @@ inside.owin <- function(x, y, w) {
   xr <- w$xrange
   yr <- w$yrange
   eps <- sqrt(.Machine$double.eps)
-  frameok <- (x >= xr[1] - eps) & (x <= xr[2] + eps) & 
-             (y >= yr[1] - eps) & (y <= yr[2] + eps)
+  frameok <- (x >= xr[1L] - eps) & (x <= xr[2L] + eps) & 
+             (y >= yr[1L] - eps) & (y <= yr[2L] + eps)
  
-  if(all(!frameok))  # all points OUTSIDE window - no further work needed
+  if(!any(frameok))  # all points OUTSIDE window - no further work needed
     return(frameok)
 
   ok <- frameok
@@ -812,6 +952,15 @@ inside.owin <- function(x, y, w) {
            return(ok)
          },
          polygonal = {
+           ## check scale
+           framesize <- max(diff(xr), diff(yr))
+           if(framesize > 1e6 || framesize < 1e-6) {
+             ## rescale to avoid numerical overflow
+             scalefac <- framesize/100
+             w <- rescale(w, scalefac)
+             x <- x/scalefac
+             y <- y/scalefac
+           }
            xy <- list(x=x,y=y)
            bdry <- w$bdry
            total <- numeric(length(x))
@@ -854,33 +1003,32 @@ inside.owin <- function(x, y, w) {
 
 #-------------------------------------------------------------------------
   
-print.owin <- function(x, ...) {
+print.owin <- function(x, ..., prefix="window: ") {
   verifyclass(x, "owin")
   unitinfo <- summary(unitname(x))
   switch(x$type,
          rectangle={
-           rectname <- "window: rectangle ="
+           rectname <- paste0(prefix, "rectangle =")
          },
          polygonal={
-           cat("window:", "polygonal", "boundary", fill=TRUE)
+           splat(paste0(prefix, "polygonal boundary"))
            if(length(x$bdry) == 0)
-             cat("window is empty\n")
+             splat("window is empty")
            rectname <- "enclosing rectangle:"
          },
          mask={
-           cat("window: binary", "image mask", fill=TRUE)
+           splat(paste0(prefix, "binary image mask"))
            di <- x$dim
-           cat(di[1], "x", di[2], "pixel array (ny, nx)", fill=TRUE)
+           splat(di[1L], "x", di[2L], "pixel array (ny, nx)")
            rectname <- "enclosing rectangle:"
          }
          )
-  cat(rectname,
-      prange(zapsmall(x$xrange)),
-      "x",
-      prange(zapsmall(x$yrange)),
-      unitinfo$plural,
-      unitinfo$explain,
-      fill=TRUE)
+  splat(rectname,
+        prange(zapsmall(x$xrange)),
+        "x",
+        prange(zapsmall(x$yrange)),
+        unitinfo$plural,
+        unitinfo$explain)
   invisible(NULL)
 }
 
@@ -889,8 +1037,9 @@ summary.owin <- function(object, ...) {
   result <- list(xrange=object$xrange,
                  yrange=object$yrange,
                  type=object$type,
-                 area=area.owin(object),
+                 area=area(object),
                  units=unitname(object))
+  result$areafraction <- with(result, area/(diff(xrange) * diff(yrange)))
   switch(object$type,
          rectangle={
          },
@@ -900,12 +1049,11 @@ summary.owin <- function(object, ...) {
            if(npoly == 0) {
              result$areas <- result$nvertices <- numeric(0)
            } else if(npoly == 1) {
-             result$areas <- area.xypolygon(poly[[1]])
-             result$nvertices <- length(poly[[1]]$x)
+             result$areas <- Area.xypolygon(poly[[1L]])
+             result$nvertices <- length(poly[[1L]]$x)
            } else {
-             result$areas <- unlist(lapply(poly, area.xypolygon))
-             result$nvertices <- unlist(lapply(poly,
-                                               function(a) {length(a$x)}))
+             result$areas <- unlist(lapply(poly, Area.xypolygon))
+             result$nvertices <- lengths(lapply(poly, getElement, name="x"))
            }
            result$nhole <- sum(result$areas < 0)
          },
@@ -930,20 +1078,18 @@ print.summary.owin <- function(x, ...) {
          },
          polygonal={
            np <- x$npoly
-           cat("Window:", "polygonal", "boundary", fill=TRUE)
+           splat("Window: polygonal boundary")
            if(np == 0) {
-             cat("window is empty\n")
+             splat("window is empty")
            } else if(np == 1) {
-             cat("single connected", "closed polygon",
-                 "with",
-                 x$nvertices, 
-                 "vertices", fill=TRUE)
+             splat("single connected closed polygon with",
+                   x$nvertices, "vertices")
            } else {
              nh <- x$nhole
              holy <- if(nh == 0) "(no holes)" else
                      if(nh == 1) "(1 hole)" else
                      paren(paste(nh, "holes"))
-             cat(np, "separate polygons", holy, fill=TRUE)
+             splat(np, "separate polygons", holy)
              if(np > 0)
                print(data.frame(vertices=x$nvertices,
                                 area=signif(x$areas, 6),
@@ -956,30 +1102,53 @@ print.summary.owin <- function(x, ...) {
            rectname <- "enclosing rectangle:"
          },
          mask={
-           cat("binary image mask\n")
+           splat("binary image mask")
            di <- x$npixels
-           cat(paste(di[1], "x", di[2], "pixel array (ny, nx)\n"))
-           cat(paste("pixel size:",
-                     signif(x$xstep,3), "by", signif(x$ystep,3),
-                     pluralunits, "\n"))
+           splat(di[1L], "x", di[2L], "pixel array (ny, nx)")
+           splat("pixel size:",
+                 signif(x$xstep,3), "by", signif(x$ystep,3),
+                 pluralunits)
            rectname <- "enclosing rectangle:"
          }
          )
-  cat(rectname,
-      prange(zapsmall(x$xrange)),
-      "x",
-      prange(zapsmall(x$yrange)),
-      pluralunits, 
-      fill=TRUE)
+  splat(rectname,
+        prange(zapsmall(x$xrange)),
+        "x",
+        prange(zapsmall(x$yrange)),
+        pluralunits)
   Area <- signif(x$area, 6)
-  cat("Window area =", Area, "square",
-      if(Area == 1) singularunits else pluralunits,
-      fill=TRUE)
+  splat("Window area =", Area, "square",
+        if(Area == 1) singularunits else pluralunits)
   if(!is.null(ledge <- unitinfo$legend))
-    cat(ledge, "\n")
+    splat(ledge)
+  if(x$type != "rectangle")
+    splat("Fraction of frame area:", signif(x$areafraction, 3))
   return(invisible(x))
 }
 
+as.data.frame.owin <- function(x, ..., drop=TRUE) {
+  stopifnot(is.owin(x))
+  switch(x$type,
+         rectangle = { x <- as.polygonal(x) },
+         polygonal = { },
+         mask = {
+           xy <- rasterxy.mask(x, drop=drop)
+           if(!drop) xy <- append(xy, list(inside=as.vector(x$m)))
+           return(as.data.frame(xy, ...))
+         })
+  b <- x$bdry
+  ishole <- sapply(b, is.hole.xypolygon)
+  sign <- (-1)^ishole
+  b <- lapply(b, as.data.frame, ...)
+  nb <- length(b)
+  if(nb == 1)
+    return(b[[1L]])
+  dfs <- mapply(cbind, b, id=as.list(seq_len(nb)), sign=as.list(sign),
+                SIMPLIFY=FALSE)
+  df <- do.call(rbind, dfs)
+  return(df)
+}
+
 discretise <- function(X,eps=NULL,dimyx=NULL,xy=NULL) {
   verifyclass(X,"ppp")
   W <- X$window
@@ -1005,3 +1174,20 @@ pixelcentres <- function (X, W=NULL,...) {
   Y <- as.ppp(raster.xy(X,drop=TRUE),W=W)
   return(Y)
 }
+
+owin2polypath <- function(w) {
+  w <- as.polygonal(w)
+  b <- w$bdry
+  xvectors <- lapply(b, getElement, name="x")
+  yvectors <- lapply(b, getElement, name="y")
+  xx <- unlist(lapply(xvectors, append, values=NA, after=FALSE))[-1]
+  yy <- unlist(lapply(yvectors, append, values=NA, after=FALSE))[-1]
+  return(list(x=xx, y=yy))
+}
+
+## generics which extract and assign the window of some object
+
+Window <- function(X, ...) { UseMethod("Window") }
+
+"Window<-" <- function(X, ..., value) { UseMethod("Window<-") }
+
